package cn.estsh.i3plus.ext.mes.api.base;

import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.mes.bean.MesBom;
import cn.estsh.i3plus.pojo.mes.bean.MesPart;
import io.swagger.annotations.ApiOperation;

import java.util.List;

/**
 * @Description:
 * @CreateDate 2024/04/16
 * @Author mingliang.li
 */
public interface IMesBomService extends IBaseMesService<MesBom> {

    @ApiOperation("查询生产版本信息")
    List<String> findBomVersionByPartNo(MesBom mesBom);

    @ApiOperation("根据生产版本查询BomCode")
    List<String> findBomCodeByPartNoAndBomVersion(MesBom mesBom);

    /**
     * 查询MES组织模型数据返回树结构
     *
     * @param bom 组织代码
     * @return 组织模型数据返回树结构
     */
    @ApiOperation(value = "查询BOM树结构")
    List queryMesDataTreeByFirstFloor(MesBom bom);

    /**
     * 查询MES组织模型数据返回树结构
     *
     * @param bom 组织代码
     * @return 组织模型数据返回树结构
     */
    @ApiOperation(value = "查询BOM树结构")
    ListPager<MesBom> queryMesDataTree(MesBom bom, Pager pager);

    /**
     * 查询MES组织模型数据返回树结构
     *
     * @param bom 组织代码
     * @return 组织模型数据返回树结构
     */
    @ApiOperation(value = "查询BOM树结构平铺")
    ListPager<MesBom> queryMesDataTreeAll(MesBom bom, Pager pager);

    @ApiOperation(value = "查询bom信息")
    List<MesBom> findMesBomByPartNoAndBomVersion(String partNo, String organizeCode, String bomVersion);

    @ApiOperation(value = "递归查询bom数据")
    List<MesBom> findBomList(String organizeCode, MesPart mesPart, String bomVersion);

    /**
     * 根据生产版本和零件号查询bom
     * @param organizeCode
     * @param bomVersion
     * @param partNo
     * @return
     */
    MesBom findBom(String organizeCode, String bomVersion,String partNo);

    /**
     * 只根据总成查询最新的bom
     * @param organizeCode
     * @param partNo
     * @return
     */
    MesBom findBom(String organizeCode, String partNo);

    @ApiOperation(value = "bom反向查询")
    List<MesBom> queryMesBomReverse(MesBom mesBom);
}
