import cn.estsh.i3plus.mes.apiservice.util.DateUtil
import cn.estsh.i3plus.platform.common.tool.TimeTool
import cn.estsh.i3plus.platform.plugin.datasource.DynamicDataSourceProxy
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack
import cn.estsh.i3plus.pojo.mes.bean.MesCrossOrganizeCfg
import cn.estsh.i3plus.pojo.mes.dbinterface.MesInterfaceDataMapper
import cn.estsh.i3plus.pojo.mes.repository.IMesCrossOrganizeCfgRepository
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil
import lombok.Getter
import lombok.Setter
import org.slf4j.Logger
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate
import org.springframework.util.CollectionUtils
import org.springframework.util.StringUtils

import javax.annotation.Resource
import java.sql.Connection

/**
 * @Description : 跨工厂防错
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/7/22 11:03
 * @Modify:
 * */
class MesToPiscesProductSn {

    public static final Logger LOGGER = LoggerFactory.getLogger(MesToPiscesProductSn.class)

    @Autowired
    private IMesCrossOrganizeCfgRepository mesCrossOrganizeCfgRepository;

    @Resource(name = "mesDataSource")
    private DynamicDataSourceProxy mesDataSource;

    @Getter
    @Setter
    private Connection mesConn;

    def readData(MesInterfaceDataMapper mapper) throws Exception {
        String organizeCode = mapper.getOrganizeCode();
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.CROSS_ORGANIZE_CFG_TYPE.REVERSE_SYNC.getValue(), "type", packBean)
        List<MesCrossOrganizeCfg> crossOrganizeCfgList = mesCrossOrganizeCfgRepository.findByHqlWhere(packBean);
        if (CollectionUtils.isEmpty(crossOrganizeCfgList)) {
            LOGGER.info("未查询到跨工厂配置");
            return new ArrayList<>();
        }
        LOGGER.info("跨工厂配置个数{}",crossOrganizeCfgList.size());

        List<Map<String, Object>> srcData = new ArrayList<>();
        List<String> updateList = new ArrayList<>();
        //循环处理数据
        for (MesCrossOrganizeCfg crossOrganizeCfg : crossOrganizeCfgList) {
            Map<String, Object> params = new HashMap<>();
            params.put("srcOrganizeCode", crossOrganizeCfg.getOrganizeCode())
            params.put("partNo", crossOrganizeCfg.getPartNo())
            if (StringUtils.isEmpty(crossOrganizeCfg.getLastCreateDatetime())) {
                params.put("createDateTime", TimeTool.pareDateToString(new Date(0)));
            } else {
                params.put("createDateTime", crossOrganizeCfg.getLastCreateDatetime())
            }

            String sql = "select * from " + mapper.getSrcTableName() + " where organize_code = :srcOrganizeCode and part_no = :partNo and create_date_time >= :createDateTime and " + mapper.getSrcWhere() + " order by " + mapper.getSrcOrderBy() + " limit " + mapper.getSrcGetLimit()
            LOGGER.info("工厂:{}，物料:{}，时间:{}，查询sql:{}",crossOrganizeCfg.getOrganizeCode(),crossOrganizeCfg.getPartNo(),params.get("createDateTime"),sql);
            //查询待同步的数据
            List<Map<String, Object>> dataList = queryDataTable(sql, params);
            if (!CollectionUtils.isEmpty(dataList)) {
                LOGGER.info("工厂:{}，物料:{}，查询行数:{}",crossOrganizeCfg.getSrcOrganizeCode(),crossOrganizeCfg.getPartNo(),dataList.size());

                //目标工厂
                for (Map<String, Object> rowMap : dataList) {
                    rowMap.put("DEST_ORGANIZE_CODE", crossOrganizeCfg.getSrcOrganizeCode())
                }
                srcData.addAll(dataList);

                String createDateTime = TimeTool.pareDateToString(DateUtil.addMinutes(TimeTool.parseDate(dataList.iterator().next().get("create_date_time")), -5));
                //更新配置
                updateList.add("update mes_cross_organize_cfg set last_create_date_time = '"+createDateTime+"',last_exe_time = '"+ TimeTool.getNowTime(true)+"' where id= " + crossOrganizeCfg.getId() + " ;")
            }
        }
        //更新同步标识
        if (!CollectionUtils.isEmpty(updateList) && updateList.size() > 0) {
            this.mesConn = mesDataSource.getWriteConnectionWithoutPool();
            mesDataSource.executeAsBatch(updateList, mesConn)
            mesDataSource.closeConnectionWithoutPoll(this.mesConn)
        }
        return srcData;
    }

    private List<Map<String, Object>> queryDataTable(String sql, Map<String, Object> params) {
        NamedParameterJdbcTemplate namedJdbcTemplate = new NamedParameterJdbcTemplate(mesDataSource.getDataSource())
        return namedJdbcTemplate.queryForList(sql, params);
    }
}