import cn.estsh.i3plus.ext.mes.api.base.IMesPartSapService
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack
import cn.estsh.i3plus.pojo.mes.bean.MesCustomerCarModel
import cn.estsh.i3plus.pojo.mes.bean.MesPartProdGroup
import cn.estsh.i3plus.pojo.mes.bean.MesPartSap
import cn.estsh.i3plus.pojo.mes.bean.MesProductVersion
import cn.estsh.i3plus.pojo.mes.dbinterface.MesInterfaceDataMapper
import cn.estsh.i3plus.pojo.mes.repository.MesCustomerCarModelRepository
import cn.estsh.i3plus.pojo.mes.repository.MesPartProdGroupRepository
import cn.estsh.i3plus.pojo.mes.repository.MesProductVersionRepository
import org.slf4j.Logger
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.util.StringUtils

/**
 * @Description : 工单同步
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/7/12 12:28
 * @Modify:
 * */
class MesToPiscesWorkOrder {

    public static final Logger LOGGER = LoggerFactory.getLogger(MesToPiscesWorkOrder.class)

    @Autowired
    private MesPartProdGroupRepository mesPartProdGroupRepository;

    @Autowired
    private MesProductVersionRepository mesProductVersionRepository;

    @Autowired
    private IMesPartSapService mesPartSapService;

    @Autowired
    private MesCustomerCarModelRepository mesCustomerCarModelRepository;

    def filterData(MesInterfaceDataMapper mapper, List<Map<String, Object>> srcData) throws Exception {
        if (srcData == null || srcData.size() == 0) {
            return srcData
        }
        Map<String, MesProductVersion> mesProductVersionMap = new HashMap<>();
        Map<String, MesPartSap> mesPartSapMap = new HashMap<>();
        Map<String, MesPartProdGroup> mesPartProdGroupMap = new HashMap<>();
        Map<String, MesCustomerCarModel> mesCustomerCarModelMap = new HashMap<>();
        String organizeCode = mapper.getOrganizeCode();
        for (Map<String, Object> rowMap : srcData) {
            if (!StringUtils.isEmpty(rowMap.get("part_no")) && !StringUtils.isEmpty(rowMap.get("product_version")) && !StringUtils.isEmpty(rowMap.get("erp_work_center"))) {
                MesProductVersion mesProductVersion = getMesProductVersion(organizeCode, String.valueOf(rowMap.get("part_no")), String.valueOf(rowMap.get("product_version")), String.valueOf(rowMap.get("erp_work_center")), mesProductVersionMap)
                if (!Objects.isNull(mesProductVersion)) {
                    rowMap.put("BOM_CODE", mesProductVersion.getAlternativePartList())
                } else {
                    LOGGER.info("物料{}生产版本{}工作中心{}，信息不存在", String.valueOf(rowMap.get("part_no")), String.valueOf(rowMap.get("product_version")), String.valueOf(rowMap.get("erp_work_center")))
                }
            }
            //获取物料信息
            if (!StringUtils.isEmpty(rowMap.get("part_no"))) {
                try {
                    if (Objects.isNull(mesPartSapMap) || !mesPartSapMap.containsKey(String.valueOf(rowMap.get("part_no")))) {
                        mesPartSapMap.put(String.valueOf(rowMap.get("part_no")), mesPartSapService.getMesPartSapByPartNo(String.valueOf(rowMap.get("part_no")), organizeCode))
                    }
                    MesPartSap mesPartSap = mesPartSapMap.get(String.valueOf(rowMap.get("part_no")))
                    rowMap.put("UOM", mesPartSap.getUnit())
                } catch (Exception e) {
                    LOGGER.info("物料{},信息不存在", String.valueOf(rowMap.get("part_no")))
                }
            }
            //获取零件生产组
            if (!StringUtils.isEmpty(rowMap.get("part_prod_group_code"))) {
                MesPartProdGroup mesPartProdGroup = getMesPartProdGroup(organizeCode, String.valueOf(rowMap.get("part_prod_group_code")), mesPartProdGroupMap);
                if (!Objects.isNull(mesPartProdGroup)) {
                    rowMap.put("PRODUCT_GROUP_ID", mesPartProdGroup.getId())
                } else {
                    LOGGER.info("零件生产组{}信息不存在", String.valueOf(rowMap.get("part_prod_group_code")))
                }
            }

            //获取车型名称
            if (!StringUtils.isEmpty(rowMap.get("car_model_code"))) {
                MesCustomerCarModel mesCustomerCarModel = getMesCustomerCarModel(organizeCode, String.valueOf(rowMap.get("car_model_code")), mesCustomerCarModelMap)
                if (!Objects.isNull(mesCustomerCarModel)) {
                    rowMap.put("VEHICLE_NAME", mesCustomerCarModel.getCarModelName())
                } else {
                    LOGGER.info("车型代码{}信息不存在", String.valueOf(rowMap.get("car_model_code")))
                }
            }
        }
        return srcData;
    }

    private MesProductVersion getMesProductVersion(String organizeCode, String partNo, String productVersion, String workCenterCode, Map<String, MesProductVersion> mesProductVersionMap) {
        if (Objects.isNull(mesProductVersionMap) || !mesProductVersionMap.containsKey(partNo + productVersion + workCenterCode)) {
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(partNo, "partNo", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(workCenterCode, "workCenterCode", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(productVersion, "productVersion", ddlPackBean);
            mesProductVersionMap.put(partNo + productVersion + workCenterCode, mesProductVersionRepository.getByProperty(ddlPackBean))
        }
        return mesProductVersionMap.get(partNo + productVersion + workCenterCode)
    }

    private MesPartProdGroup getMesPartProdGroup(String organizeCode, String partProdGroupCode, Map<String, MesPartProdGroup> mesProductVersionMap) {
        if (Objects.isNull(mesProductVersionMap) || !mesProductVersionMap.containsKey(partProdGroupCode)) {
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(partProdGroupCode, "partProdGroupCode", ddlPackBean);
            mesProductVersionMap.put(partProdGroupCode, mesPartProdGroupRepository.getByProperty(ddlPackBean))
        }
        return mesProductVersionMap.get(partProdGroupCode);
    }

    private MesCustomerCarModel getMesCustomerCarModel(String organizeCode, String carModelCode, Map<String, MesCustomerCarModel> mesCustomerCarModelMap) {
        if (Objects.isNull(mesCustomerCarModelMap) || !mesCustomerCarModelMap.containsKey(carModelCode)) {
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(carModelCode, "carModelCode", ddlPackBean);
            mesCustomerCarModelMap.put(carModelCode, mesCustomerCarModelRepository.getByProperty(ddlPackBean));
        }
        return mesCustomerCarModelMap.get(carModelCode)
    }
}