import cn.estsh.i3plus.platform.common.tool.TimeTool
import cn.estsh.i3plus.platform.plugin.datasource.DynamicDataSourceProxy
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack
import cn.estsh.i3plus.pojo.base.util.StringUtil
import cn.estsh.i3plus.pojo.mes.bean.*
import cn.estsh.i3plus.pojo.mes.dbinterface.MesInterfaceDataMapper
import cn.estsh.i3plus.pojo.mes.repository.*
import lombok.Getter
import lombok.Setter
import org.slf4j.Logger
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate
import org.springframework.util.CollectionUtils
import org.springframework.util.StringUtils

import javax.annotation.Resource
import java.sql.Connection

/**
 * @Description : 加工记录同步 MES2PISCES
 * @Reference :
 * @Author : wangjie
 * @CreateDate 2024/7/12 12:28
 * @Modify:
 * */
class MesProductionRecordToPisces {

    public static final Logger LOGGER = LoggerFactory.getLogger(MesProductionRecordToPisces.class)

    private static final String[] workCenterCodeList = ["B19C901A", "B15C901A", "C06C901A", "C06C901B", "C09C901A", "C12C901A"] as String[];

    @Autowired
    private MesEquipmentRepository equipmentRepository;

    @Autowired
    private MesProduceSnRepository produceSnRepository;

    @Autowired
    private MesWorkOrderRepository workOrderRepository;

    @Resource(name = "pisces3DataSource")
    private DynamicDataSourceProxy piscesDataSource;

    @Resource(name = "mesDataSource")
    private DynamicDataSourceProxy mesDataSource;

    @Getter
    @Setter
    private Connection mesConn;

    @Autowired
    private MesWorkCenterRepository mesWorkCenterRepository;

    @Autowired
    private MesWorkCellRepository mesWorkCellRepository;

    @Autowired
    private MesProcessRepository mesProcessRepository;

    def filterData(MesInterfaceDataMapper mapper, List<Map<String, Object>> srcData) throws Exception {
        if (srcData == null || srcData.size() == 0) {
            return srcData
        }
        String organizeCode = mapper.getOrganizeCode();
        Map<String, MesWorkCenter> mesWorkCenterMap = new HashMap<>();
        Map<String, MesWorkCell> mesWorkCellMap = new HashMap<>();
        Map<String, MesProcess> mesProcessMap = new HashMap<>();
        List<String> idList = new ArrayList<>();
        List<Map<String, Object>> destData = new ArrayList<>();
        for (Map<String, Object> rowMap : srcData) {

            //条码信息
            Map<String, Object> params = new HashMap<>();
            params.put("partNo", rowMap.get("part_no"))
            params.put("productSn", rowMap.get("product_sn"))
            String sql = "select top 1 ID from MES.TT_PC_PART_BARCODE where PART_NO = :partNo and BARCODE = :productSn order by GENERATION_TIME desc";
            List<Map<String, Object>> dataMap = queryDataTable(sql, params);

            if (!CollectionUtils.isEmpty(dataMap) && dataMap.size() > 0) {
                rowMap.put("BARCODE_ID", dataMap.iterator().next().get("ID"))
                idList.add(rowMap.get("id").toString());
            } else {
                LOGGER.info("条码{}物料{}条码信息未同步,信息不存在", rowMap.get("product_sn"), rowMap.get("part_no"))
                continue
            }
            String workCenterCode = String.valueOf(rowMap.get("work_center_code"));
            if (!workCenterCodeList.contains(workCenterCode)) {
                String workOrderNo = String.valueOf(rowMap.get("work_order_no"));
                MesWorkOrder workOrderDb = getWorkOrderDb(mapper.getOrganizeCode(), workOrderNo);
                if (!Objects.isNull(workOrderDb)) {
                    rowMap.put("WORK_ORDER_FID", workOrderDb.getFid())
                } else {
                    rowMap.put("WORK_ORDER_FID", null)
                }
            } else {
                rowMap.put("WORK_ORDER_FID", null)
            }
            //工作中心
            MesWorkCenter mesWorkCenter = getMesWorkCenter(organizeCode, workCenterCode, mesWorkCenterMap)
            if (!Objects.isNull(mesWorkCenter)) {
                rowMap.put("work_center_code", mesWorkCenter.getDescription())
            } else {
                LOGGER.info("工作中心{},信息不存在", workCenterCode)
            }
            //工位
            if (!StringUtil.isEmpty(rowMap.get("work_cell_code"))) {
                String workCellCode = String.valueOf(rowMap.get("work_cell_code"));
                MesWorkCell mesWorkCell = getMesWorkCell(organizeCode, workCellCode, mesWorkCellMap)
                if (!Objects.isNull(mesWorkCell)) {
                    rowMap.put("work_cell_code", mesWorkCell.getDescription())
                } else {
                    LOGGER.info("工位{},信息不存在", workCellCode)
                }
            }
            //工序信息
            if (!StringUtil.isEmpty(rowMap.get("process_code"))) {
                String processCode = String.valueOf(rowMap.get("process_code"));
                MesProcess mesProcess = getMesProcess(organizeCode, processCode, mesProcessMap)
                if (!Objects.isNull(mesProcess)) {
                    rowMap.put("process_code", mesProcess.getDescription())
                } else {
                    LOGGER.info("工序{},信息不存在", processCode)
                }
            }
            //开班记录
            if (!StringUtil.isEmpty(rowMap.get("work_center_code")) && !StringUtil.isEmpty(rowMap.get("process_code")) && !StringUtil.isEmpty(rowMap.get("shift_code"))) {
                params = new HashMap<>();
                params.put("workCenterCode", rowMap.get("work_center_code"))
                params.put("processCode", rowMap.get("process_code"))
                params.put("shiftCode", 1 == Integer.parseInt(rowMap.get("shift_code").toString()).intValue() ? "早班" : "晚班")
                sql = "select top 1 ID from MES.TL_PS_SHIFT_START where ASSEMBLY_LINE = :workCenterCode and SHIFT_NAME = :shiftCode and SHIFT_GROUP_CODE = :processCode order by ID desc";
                dataMap = queryDataTable(sql, params);
                if (!CollectionUtils.isEmpty(dataMap) && dataMap.size() > 0) {
                    rowMap.put("SHIFT_START_ID", dataMap.iterator().next().get("ID"))
                }
            }
            if (StringUtil.isEmpty(rowMap.get("SHIFT_START_ID"))) {
                rowMap.put("SHIFT_START_ID", null)
            }

            String equipmentCode = String.valueOf(rowMap.get("equipment_code"));
            MesEquipment equipmentDb = getEquipmentDb(mapper.getOrganizeCode(), equipmentCode);
            if (!Objects.isNull(equipmentDb)) {
                rowMap.put("EQUIPMENT_ID", equipmentDb.getEquipId())
            }
            destData.add(rowMap);
        }

        //更新同步标识
        if (!CollectionUtils.isEmpty(idList) && idList.size() > 0) {
            List<String> updateList = new ArrayList<>();
            updateList.add("update mes_production_record set system_sync_date_time='"+ TimeTool.getNowTime(true)+"',system_sync_status = 1 where id in (" + String.join(",",idList) + ") ;")
            this.mesConn = mesDataSource.getWriteConnectionWithoutPool();
            mesDataSource.executeAsBatch(updateList, mesConn)
            mesDataSource.closeConnectionWithoutPoll(this.mesConn)
        }

        return destData;
    }

    MesWorkOrder getWorkOrderDb(String organizeCode, String workOrderNo) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(workOrderNo, "workOrderNo", ddlPackBean);
        MesWorkOrder workOrderDb = workOrderRepository.getByProperty(ddlPackBean);
        return workOrderDb;
    }

    def getEquipmentDb(String organizeCode, String equipmentCode) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(equipmentCode, "equipmentCode", ddlPackBean);
        MesEquipment equipmentDb = equipmentRepository.getByProperty(ddlPackBean);
        return equipmentDb;
    }

    private List<Map<String, Object>> queryDataTable(String sql, Map<String, Object> params) {
        NamedParameterJdbcTemplate namedJdbcTemplate = new NamedParameterJdbcTemplate(piscesDataSource.getDataSource())
        List<Map<String, Object>> dataMap = namedJdbcTemplate.queryForList(sql, params)
        return dataMap;
    }

    private MesWorkCenter getMesWorkCenter(String organizeCode, String workCenterCode, Map<String, MesWorkCenter> mesWorkCenterMap) {
        if (Objects.isNull(mesWorkCenterMap) || !mesWorkCenterMap.containsKey(workCenterCode)) {
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(workCenterCode, "workCenterCode", ddlPackBean);
            mesWorkCenterMap.put(workCenterCode, mesWorkCenterRepository.getByProperty(ddlPackBean))
        }
        return mesWorkCenterMap.get(workCenterCode);
    }

    private MesWorkCell getMesWorkCell(String organizeCode, String workCenterCell, Map<String, MesWorkCell> mesWorkCellMap) {
        if (Objects.isNull(mesWorkCellMap) || !mesWorkCellMap.containsKey(workCenterCell)) {
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(workCenterCell, "workCellCode", ddlPackBean);
            mesWorkCellMap.put(workCenterCell, mesWorkCellRepository.getByProperty(ddlPackBean))
        }
        return mesWorkCellMap.get(workCenterCell);
    }


    private MesProcess getMesProcess(String organizeCode, String processCode, Map<String, MesProcess> mesProcessMap) {
        if (Objects.isNull(mesProcessMap) || !mesProcessMap.containsKey(processCode)) {
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(processCode, "processCode", ddlPackBean);
            mesProcessMap.put(processCode, mesProcessRepository.getByProperty(ddlPackBean))
        }
        return mesProcessMap.get(processCode);
    }

    def saveSrcData(MesInterfaceDataMapper mapper, List<Map<String, Object>> srcData) throws Exception {
        if (srcData == null || srcData.size() == 0 || StringUtils.isEmpty(mapper.getSrcTableName())) {
            return srcData
        }
    }

}