package cn.estsh.i3plus.ext.mes.apiservice.controller.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesLabelTemplateService;
import cn.estsh.i3plus.ext.mes.apiservice.config.AuthUtilExt;
import cn.estsh.i3plus.ext.mes.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesLabelTemplate;
import cn.estsh.i3plus.pojo.mes.bean.MesLabelTemplateFieldInfo;
import cn.estsh.i3plus.pojo.mes.bean.MesLabelTemplateFieldValue;
import cn.estsh.i3plus.pojo.mes.bean.MesSpotCheckOrder;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Description : 打印模板
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/7/10 13:39
 * @Modify:
 **/
@RestController
@RequestMapping(MesCommonConstant.MES_YANFEN + "/mesLabelTemplate")
@Api(tags = "打印模板")
public class MesLabelTemplateController extends BaseMesController<MesLabelTemplate> {

    @Autowired
    private IMesLabelTemplateService mesLabelTemplateService;

    @GetMapping("/query-template-param")
    @ApiOperation(value = "查询打印模板和明细")
    public ResultBean queryMesLabelTemplateAndParamListByPager(MesLabelTemplate labelTemplate, Pager pager) {

        try {
            return ResultBean.success("查询成功").setListPager(mesLabelTemplateService.queryMesLabelTemplateAndParamListByPager(labelTemplate, pager));
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping("/query-param-field")
    @ApiOperation(value = "查询模板明细字段信息")
    public ResultBean queryMesLabelTemplateParamField(MesLabelTemplateFieldInfo request) {
        try {
            return ResultBean.success("查询成功").setResultList(mesLabelTemplateService.queryMesLabelTemplateParamField(request));
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping("/save-param-field")
    @ApiOperation(value = "保存模板明细字段信息")
    public ResultBean saveMesLabelTemplateParamField(@RequestBody List<MesLabelTemplateFieldInfo> templateFieldInfos) {
        try {
            mesLabelTemplateService.saveMesLabelTemplateParamField(templateFieldInfos);
            return ResultBean.success("保存成功");
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping("/query-param-part-value")
    @ApiOperation(value = "查询模板明细字段零件内容")
    public ResultBean queryMesLabelTemplateParamPartValue(String partNo, String labelTemplate, String organizeCode) {
        try {
            return ResultBean.success("查询成功").setResultList(mesLabelTemplateService.queryMesLabelTemplateParamPartValue(partNo, labelTemplate, organizeCode));
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping("/save-param-field-value")
    @ApiOperation(value = "保存模板明细字段零件内容")
    public ResultBean saveMesLabelTemplateParamPartValue(@RequestBody List<MesLabelTemplateFieldValue> templateFieldValues) {
        try {
            mesLabelTemplateService.saveMesLabelTemplateParamPartValue(templateFieldValues);
            return ResultBean.success("保存成功");
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }
}
