package cn.estsh.i3plus.ext.mes.apiservice.controller.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesShippingOrderManagementDetailService;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.ext.mes.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.ext.mes.pojo.model.MesHandleShippingErrorModel;
import cn.estsh.i3plus.ext.mes.pojo.model.MesShippingOrderManagementDetailModel;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrder;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrderPart;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesShippingOrderManagementDetail;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Objects;

@Api("发运单管理明细")
@RestController
@RequestMapping(MesCommonConstant.MES_YANFEN + "/mesShippingOrderManagementDetail")

public class MesShippingOrderManagementDetailController extends BaseMesController<MesShippingOrderManagementDetail> {

    @Autowired
    private IMesShippingOrderManagementDetailService shippingOrderManagementDetailService;

    @PostMapping(value = "/query-group")
    @ApiOperation(value = "发运单管理明细-根据零件汇总")
    public ResultBean insertMesProdOrg(MesShippingOrderManagementDetail detail, Pager pager) {
        try {

            detail.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());

            return ResultBean.success("添加成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setListPager(shippingOrderManagementDetailService.queryDetailGroupByPartNo(detail,pager));
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @ApiOperation(value = "发运单明细", notes = "发运单明细")
    @GetMapping("/master/query")
    public ResultBean queryMesShippingDetailsByPager(MesShippingOrderManagementDetailModel mesShippingOrderManagementDetailModel, Pager pager) {
        try {
            return ResultBean.success("查询成功").setListPager(shippingOrderManagementDetailService.queryMesShippingDetailsByPager(mesShippingOrderManagementDetailModel,pager));
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @ApiOperation(value = "发运单异常明细", notes = "发运单异常明细")
    @GetMapping("/master/error/query")
    public ResultBean queryMesShippingErrorDetailsByPager(MesShippingOrderManagementDetailModel mesShippingOrderManagementDetailModel, Pager pager) {
        try {
            return ResultBean.success("查询成功").setListPager(shippingOrderManagementDetailService.queryMesShippingErrorDetailsByPager(mesShippingOrderManagementDetailModel,pager));
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @ApiOperation(value = "cockpit发运处理", notes = "cockpit发运处理")
    @PostMapping("/handle-cockpit-shipping-error")
    public ResultBean handleCockpitShippingError(@RequestBody MesHandleShippingErrorModel request) {
        try {
            MesShippingOrderManagementDetail detail = shippingOrderManagementDetailService.get(request.getDetail().getId());
            if (detail == null) {
                return ResultBean.fail("无效发运单明细");
            }
            if (detail.getPid() == null) {
                return ResultBean.fail("发运单明细未关联有效发运单");
            }
            shippingOrderManagementDetailService.doHandleCockpitShippingError(detail, request.getWorkOrder());
            return ResultBean.success("处理成功");
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @ApiOperation(value = "根据客户零件匹配零件号", notes = "根据客户零件匹配零件号")
    @GetMapping("/master/part/supplyBycustPartNo")
    public ResultBean supplyBycustPartNo(Long[] ids) {
        try {
            return shippingOrderManagementDetailService.supplyBycustPartNo(ids);
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @ApiOperation(value = "根据CBR匹配零件号", notes = "根据CBR匹配零件号")
    @GetMapping("/master/part/supplyByCBR")
    public ResultBean supplyByCBR(Long[] ids) {
        try {
            return shippingOrderManagementDetailService.supplyByCBR(ids);
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }
    @ApiOperation(value = "修改数量", notes = "修改数量")
    @GetMapping("/master/part/updateQty")
    public ResultBean updateQty(Long[] ids) {
        try {
            return shippingOrderManagementDetailService.updateQty(ids);
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @ApiOperation(value = "发运同步", notes = "发运同步")
    @GetMapping("/master/part/syncShip")
    public ResultBean syncShip(Long[] ids) {
        try {
            return shippingOrderManagementDetailService.syncShip(ids);
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @ApiOperation(value = "发运退回", notes = "发运退回")
    @PutMapping("/doShippingBack")
    public ResultBean doShippingBack(@RequestBody Long[] idList) {
        try {
            if(Objects.isNull(idList) || idList.length <= 0) MesException.throwMesBusiException("选择数据无效,请检查");
            shippingOrderManagementDetailService.doShippingBack(idList,AuthUtil.getOrganizeCode(),AuthUtil.getSessionUser().getUserName());
            return ResultBean.success("退回成功");
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/ship-part/query-item-part")
    @ApiOperation(value = "发运零件子零件明细")
    public ResultBean queryDetailMesWorkOrderPart(MesWorkOrderPart detail, Pager pager) {
        try {

            detail.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());

            return ResultBean.success("查询成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setListPager(shippingOrderManagementDetailService.queryDetailMesWorkOrderPart(detail,pager));
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }
    @GetMapping(value = "/ship-order/query-item-part")
    @ApiOperation(value = "发运单子零件明细")
    public ResultBean queryMesShippingOrderManagementDetail(MesWorkOrderPart detail, Pager pager) {
        try {

            detail.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());

            return ResultBean.success("查询成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setListPager(shippingOrderManagementDetailService.queryDetailMesWorkOrderPart(detail,pager));
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


}
