package cn.estsh.i3plus.ext.mes.apiservice.controller.base.file;

import cn.estsh.i3plus.ext.mes.api.base.file.IMesModuleExcelService;
import cn.estsh.i3plus.ext.mes.apiservice.controller.base.BaseMesController;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.ext.mes.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesModuleExcel;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

/**
 * @Description : 模块EXCEL管理表
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/6/17 14:54
 * @Modify:
 **/
@RestController
@RequestMapping(MesCommonConstant.MES_YANFEN + "/mesModuleExcel")
@Api(tags = "模块EXCEL模板管理")
public class MesModuleExcelController extends BaseMesController<MesModuleExcel> {

    @Autowired
    private IMesModuleExcelService moduleExcelService;

    @GetMapping(value = "/queryExt")
    @ApiOperation(value = "按条件分页查询模块EXCEL模板信息")
    public ResultBean queryModuleExcelByPager(MesModuleExcel mesModuleExcel, Pager pager) {
        try {
            mesModuleExcel.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setListPager(moduleExcelService.queryModuleExcelByPager(mesModuleExcel, pager));
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/insertExt")
    @ApiOperation(value = "新增模块EXCEL模板信息")
    public ResultBean insertModuleExcel(MultipartFile file, MesModuleExcel mesModuleExcel) {
        try {
            if (file == null) {
                MesException.throwMesBusiException("上传的文件为null，后台未能接收到");
            }

            ValidatorBean.beginValid(mesModuleExcel)
                    .notNull(MesExtConstWords.MODULE_CODE, mesModuleExcel.getModuleCode())
                    .notNull(MesExtConstWords.MODULE_NAME, mesModuleExcel.getModuleName());

            mesModuleExcel.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            ConvertBean.serviceModelUpdate(mesModuleExcel, AuthUtil.getSessionUser().getUserName());
            moduleExcelService.insertModuleExcel(file, mesModuleExcel);

            return ResultBean.success("添加成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/updateExt")
    @ApiOperation(value = "更新模块EXCEL模板信息")
    public ResultBean updateModuleExcel(MultipartFile file, MesModuleExcel mesModuleExcel) {
        try {
            ValidatorBean.beginValid(mesModuleExcel)
                    .notNull(MesExtConstWords.ID, mesModuleExcel.getId())
                    .notNull(MesExtConstWords.MODULE_CODE, mesModuleExcel.getModuleCode())
                    .notNull(MesExtConstWords.MODULE_NAME, mesModuleExcel.getModuleName());

            mesModuleExcel.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            ConvertBean.serviceModelUpdate(mesModuleExcel, AuthUtil.getSessionUser().getUserName());
            moduleExcelService.updateModuleExcel(file, mesModuleExcel);
            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
