package cn.estsh.i3plus.ext.mes.apiservice.controller.report;

import cn.estsh.i3plus.ext.mes.api.base.IMesDowntimeRecordService;
import cn.estsh.i3plus.ext.mes.api.base.IMesProductionRecordService;
import cn.estsh.i3plus.ext.mes.api.base.IMesWorkOrderService;
import cn.estsh.i3plus.ext.mes.api.busi.report.IMesYfReportService;
import cn.estsh.i3plus.ext.mes.apiservice.config.AuthUtilExt;
import cn.estsh.i3plus.ext.mes.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.ext.mes.pojo.model.board.MesWorkOrderBoardReqModel;
import cn.estsh.i3plus.ext.mes.pojo.model.board.MesWorkOrderBoardResModel;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.mes.bean.MesCokpitKanbanData;
import cn.estsh.i3plus.pojo.mes.bean.MesDowntimeRecord;
import cn.estsh.i3plus.pojo.mes.bean.MesProduceSn;
import cn.estsh.i3plus.pojo.mes.bean.MesProductionRecord;
import cn.estsh.i3plus.pojo.mes.model.MesProductionRecordModel;
import cn.estsh.i3plus.pojo.mes.model.report.*;
import cn.estsh.impp.framework.base.controller.MesBaseController;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.drools.core.rule.Collect;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @Description : Mes业务报表
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/6/28 14:17
 * @Modify:
 **/
@Api("Mes业务报表")
@RestController
@RequestMapping(MesCommonConstant.MES_YANFEN + "/report")
public class MesBusiReportController extends MesBaseController {

    @Autowired
    private IMesYfReportService mesYfReportService;

    @Autowired
    private IMesProductionRecordService mesProductionRecordService;

    @Autowired
    private IMesDowntimeRecordService mesDowntimeRecordService;

    @Autowired
    private IMesWorkOrderService mesWorkOrderService;

    @ApiOperation(value = "查询追溯报表", notes = "查询追溯报表")
    @GetMapping("/traceability/find")
    public ResultBean findTraceabilityReport(MesProduceSn mesProduceSn) {
        try {
            ValidatorBean.beginValid(mesProduceSn).notNull("productSn", mesProduceSn.getProductSn());
            mesProduceSn.setOrganizeCode(AuthUtilExt.getOrganizeCode());
            return ResultBean.success("查询成功").setResultList(mesYfReportService.findTraceabilityReport(mesProduceSn));
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @ApiOperation(value = "零件条码状态查询", notes = "零件条码状态查询")
    @GetMapping("/mesProduceSn/query")
    public ResultBean queryMesProduceSn(MesProduceSn mesProduceSn, Pager pager) {
        try {
            return ResultBean.success("查询成功").setListPager(mesYfReportService.queryMesProduceSn(mesProduceSn,pager));
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


    @ApiOperation(value = "产品加工记录", notes = "产品加工记录")
    @GetMapping("/mesProductionRecord/query")
    public ResultBean queryMesProductionRecord(MesProductionRecordModel mesProductionRecordModel, Pager pager) {
        try {
            return ResultBean.success("查询成功").setListPager(mesYfReportService.queryMesProductionRecord(mesProductionRecordModel,pager));
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @ApiOperation(value = "线边库存报表", notes = "线边库存报表")
    @GetMapping("/mesProductionOffLine/query")
    public ResultBean queryMesProductionOffLine(MesProductionRecordModel mesProductionRecordModel, Pager pager) {
        try {
            ValidatorBean.beginValid(mesProductionRecordModel)
                    .notNull("partNo", mesProductionRecordModel.getPartNo())
                    .notNull("organizeCode", mesProductionRecordModel.getOrganizeCode())
                    .notNull("onLineWorkCellCode",mesProductionRecordModel.getOnLineWorkCellCode())
                    .notNull("offLineWorkCellCode",mesProductionRecordModel.getOffLineWorkCellCode())
                    .notNull("createDateTimeStart",mesProductionRecordModel.getCreateDateTimeStart())
                    .notNull("createDateTimeEnd",mesProductionRecordModel.getCreateDateTimeEnd());
            return ResultBean.success("查询成功").setListPager(mesProductionRecordService.queryMesProductionOffLine(mesProductionRecordModel,pager));
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @ApiOperation(value = "对账报表", notes = "对账报表")
    @GetMapping("/product/account/query")
    public ResultBean queryProductAccount(MesProductAccountModel productAccountModel, Pager pager) {
        try {/*
            ValidatorBean.beginValid(productAccountModel)
                    .notNull("equipmentCode", productAccountModel.getEquipmentCode());*/
            /*productAccountModel.setStartTime("2024-09-12 08:00:00");
            productAccountModel.setEndTime("2024-09-13 08:00:00");
            productAccountModel.setEquipmentCode("760BFoaming");
            productAccountModel.setPartNo("Z0020667");*/

            List<MesProductAccountResModel> mesProductAccountResModels = mesProductionRecordService.queryProductAccount(productAccountModel);
            pager.setTotalRows(CollectionUtils.isEmpty(mesProductAccountResModels) ? 0 : mesProductAccountResModels.size());
            List<MesProductAccountResModel> mesProductAccountResModelsRes = mesProductionRecordService.transferAccount(mesProductAccountResModels,productAccountModel, pager);

            return ResultBean.success("查询成功").setResultList(mesProductAccountResModelsRes).setPager(pager);
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            e.printStackTrace();
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @ApiOperation(value = "BTO工单看板", notes = "BTO工单看板")
    @GetMapping("/order/board/query")
    public ResultBean queryProductAccount(MesWorkOrderBoardReqModel workOrderBoardReqModel) {
        try {
            List<MesWorkOrderBoardResModel> workOrderBoardResModels = mesWorkOrderService.orderBoard(workOrderBoardReqModel);


            return ResultBean.success("查询成功").setResultList(workOrderBoardResModels);
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            e.printStackTrace();
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @ApiOperation(value = "设备停机记录报表", notes = "设备停机记录报表")
    @GetMapping("/equipment/downtime/record")
    public ResultBean downtimeRecord(MesDowntimeRecord mesDowntimeRecord, Pager pager) {
        try {

            ListPager<MesDowntimeRecordResModel> recordList = mesDowntimeRecordService.queryRecordList(mesDowntimeRecord, pager);

            return ResultBean.success("查询成功").setListPager(recordList);
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            e.printStackTrace();
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


    @ApiOperation(value = "看板数据", notes = "看板数据")
    @PostMapping("/mesCokpitKanbanData/query")
    public ResultBean queryMesCokpitKanbanData(MesCokpitKanbanData kanbanData, Pager pager) {
        try {
            ValidatorBean.beginValid(kanbanData)
                    .notNull("organizeCode", kanbanData.getOrganizeCode());
            return ResultBean.success("查询成功").setListPager(mesYfReportService.queryMesCokpitKanbanData(kanbanData,pager));
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @ApiOperation(value = "通用导出报表查询", notes = "通用导出报表查询")
    @PostMapping("/generalExport/query")
    public ResultBean queryGeneralExport(MesGeneralExportReportModel model, Pager pager) {
        try {
            ValidatorBean.beginValid(model)
                    .notNull("organizeCode", model.getOrganizeCode());
            return ResultBean.success("查询成功").setListPager(mesYfReportService.queryGeneralExport(model,pager));
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @ApiOperation(value = "通用发运报表查询", notes = "通用发运报表查询")
    @PostMapping("/generalShipping/query")
    public ResultBean queryGeneralShipping(MesGeneralShippingReportModel model, Pager pager) {
        try {
            ValidatorBean.beginValid(model)
                    .notNull("organizeCode", model.getOrganizeCode());
            return ResultBean.success("查询成功").setListPager(mesYfReportService.queryGeneralShipping(model,pager));
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


}
