package cn.estsh.i3plus.ext.mes.apiservice.cus_suit;

import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.plugin.datasource.DynamicDataSourceProxy;
import lombok.Getter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.util.concurrent.TimeUnit;

/**
 * @Description : pisces数据源配置
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/7/9 11:14
 * @Modify:
 **/
@Configuration
public class PiscesProperties {

    public static final Logger LOGGER = LoggerFactory.getLogger(CommonConstWords.SYSTEM_LOG);

    /***********************
     * pisces数据源配置
     */
    @Value("${impp.pisces.datasource.isopen}")
    @Getter
    private boolean piscesIsOpen;

    @Value("${impp.pisces.datasource.driver-class-name}")
    @Getter
    private String piscesDriverClassName;

    @Value("${impp.pisces.datasource.alias}")
    @Getter
    private String piscesDsAlias;

    @Value("${impp.pisces.datasource.jdbc-url}")
    @Getter
    private String piscesDsJdbcUrl;

    @Value("${impp.pisces.datasource.username}")
    @Getter
    private String piscesDsUsername;

    @Value("${impp.pisces.datasource.password}")
    @Getter
    private String piscesDsPassword;

    @ConditionalOnExpression("'${impp.pisces.datasource.isopen:false}' == 'true'")
    @Bean(name = "piscesDataSource")
    public DynamicDataSourceProxy initPiscesCustomerSet(){
        LOGGER.info("【初始化客户定制化...】");

        /**
         * 注册客户定制数据源
         */
        LOGGER.info("【客户定制化：pisces数据源加载...】");

        return DynamicDataSourceProxy.initDataSourceFactory(this.getPiscesDriverClassName(),
                this.getPiscesDsJdbcUrl(), this.getPiscesDsUsername(), this.getPiscesDsPassword(),TimeUnit.MINUTES.toMillis(30));
    }
}
