package cn.estsh.i3plus.ext.mes.apiservice.daoimpl;

import cn.estsh.i3plus.ext.mes.apiservice.dao.IMesCrossOrganizeCfgDao;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesCrossOrganizeCfg;
import cn.estsh.i3plus.pojo.mes.bean.MesProduceSn;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.Date;
import java.util.List;

/**
 * @Description : 跨工厂防错
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/7/4 17:59
 * @Modify:
 **/
@Service
public class MesCrossOrganizeCfgDaoImpl implements IMesCrossOrganizeCfgDao {

    @Autowired
    private EntityManager entityManager;

    @Override
    public List<MesProduceSn> findMesProduceSnByOrganizeCodeAndPart(MesCrossOrganizeCfg mesCrossOrganizeCfg, Integer pageSize) {
        String sql = "select srcSn from MesProduceSn srcSn left join MesProduceSn destSn on srcSn.serialNumber = destSn.serialNumber " +
                " and destSn.organizeCode=:destOrganizeCode and destSn.isDeleted=:isDeleted and destSn.isValid = :isValid " +
                " where srcSn.createDatetime >= :createDatetime and srcSn.organizeCode = :srcOrganizeCode " +
                " and srcSn.isDeleted=:isDeleted and srcSn.isValid = :isValid and srcSn.partNo = :partNo " +
                " and srcSn.snStatus in (10,20) and srcSn.qcStatus in (10,15) and destSn.serialNumber is null " +
                " order by srcSn.createDatetime asc";
        Query query = entityManager.createQuery(sql, MesProduceSn.class);
        query.setParameter("srcOrganizeCode", mesCrossOrganizeCfg.getSrcOrganizeCode());
        query.setParameter("destOrganizeCode", mesCrossOrganizeCfg.getOrganizeCode());
        query.setParameter("isValid", CommonEnumUtil.VALID);
        query.setParameter("isDeleted", CommonEnumUtil.FALSE);
        query.setParameter("partNo", mesCrossOrganizeCfg.getPartNo());
        if (StringUtils.isEmpty(mesCrossOrganizeCfg.getLastCreateDatetime())) {
            query.setParameter("createDatetime", TimeTool.pareDateToString(new Date(0)));
        } else {
            query.setParameter("createDatetime", mesCrossOrganizeCfg.getLastCreateDatetime());
        }
        query.setFirstResult(0).setMaxResults(pageSize);
        return query.getResultList();
    }
}
