package cn.estsh.i3plus.ext.mes.apiservice.daoimpl;

import cn.estsh.i3plus.ext.mes.apiservice.dao.IMesLoadingListDetailDao;
import cn.estsh.i3plus.ext.mes.pojo.model.MesLoadingListDetailModel;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesLoadingListDetail;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.ArrayList;
import java.util.List;

/**
 * @Description :SAP生产计划
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/5/6 15:52
 * @Modify:
 **/
@Service
@Slf4j
public class MesLoadingListDetailDaoImpl implements IMesLoadingListDetailDao {

    @Autowired
    private EntityManager entityManager;


    @Override
    public List<MesLoadingListDetailModel> queryLoadingListDetailModel(MesLoadingListDetail detail, Pager pager) {

        StringBuffer hql = new StringBuffer("select mll.order_code ,mll.cust_name ,mll.cust_organize_name ,mll.shipping_group_name , ");
        hql.append(" mll.status , mll.truck_no ,mll.weather , ms.shipping_code ,ms.start_scan_time ,ms.end_scan_time ,mlld.status as shipStatus , ");
        hql.append(" mlld.create_date_time , mll.create_user ,mll.modify_date_time , ms.modify_user , ");
        hql.append(" mll.cust_code , mll.cust_organize_code ,mll.shipping_group_code,mlld.remark  ");
        hql.append(" from mes_loading_list as mll");
        hql.append(" inner join  mes_loading_list_detail as mlld on mll.id = mlld.pid");
        hql.append(" and mll.organize_code = mlld.organize_code and mll.is_deleted = mlld.is_deleted and mll.is_valid = mlld.is_valid  ");
        hql.append(" left join  mes_shipping as ms on mlld.shipping_code = ms.shipping_code  ");
        hql.append(" and ((mlld.organize_code = ms.organize_code and mlld.is_deleted = ms.is_deleted and mlld.is_valid = ms.is_valid) or (ms.id is null))  ");
        hql.append(" where mll.organize_code = :organizeCode ");
        hql.append(" and mll.is_deleted = :isDeleted ");
        hql.append(" and mll.is_valid = :isValid ");
        hql.append(" and mlld.pid = :pid ");

        if (!StringUtils.isEmpty(detail.getShippingCode())) {
            hql.append(" and mlld.shipping_code = :shippingCode ");
        }
        if (!StringUtils.isEmpty(detail.getStatus())) {
            hql.append(" and ms.status = :status ");
        }

        Query query = entityManager.createNativeQuery(hql.toString());

        query.setParameter("organizeCode", detail.getOrganizeCode());
        query.setParameter("isValid", CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
        query.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        query.setParameter("pid", detail.getPid());

        if (!StringUtils.isEmpty(detail.getShippingCode())) {
            query.setParameter("shippingCode", detail.getShippingCode());
        }
        if (!StringUtils.isEmpty(detail.getStatus())) {
            query.setParameter("status", detail.getStatus());
        }

        List list = query.getResultList();

        List<MesLoadingListDetailModel> modelList = new ArrayList<>();

        for (Object result : list) {
            Object[] cells = (Object[]) result;
            MesLoadingListDetailModel model = new MesLoadingListDetailModel();
            model.setOrderCode(StringUtils.isEmpty(cells[0]) ? "" : String.valueOf(cells[0]));
            model.setCustName(StringUtils.isEmpty(cells[1]) ? "" : String.valueOf(cells[1]));
            model.setCustOrganizeName(StringUtils.isEmpty(cells[2]) ? "" : String.valueOf(cells[2]));
            model.setShippingGroupName(StringUtils.isEmpty(cells[3]) ? "" : String.valueOf(cells[3]));
            model.setStatus(StringUtils.isEmpty(cells[4]) ? 0 : Integer.parseInt(String.valueOf(cells[4])));
            model.setTruckNo(StringUtils.isEmpty(cells[5]) ? "" : String.valueOf(cells[5]));
            model.setWeather(StringUtils.isEmpty(cells[6]) ? 0 : Integer.parseInt(String.valueOf(cells[6])));
            model.setShippingCode(StringUtils.isEmpty(cells[7]) ? "" : String.valueOf(cells[7]));
            model.setStartScanTime(StringUtils.isEmpty(cells[8]) ? "" : String.valueOf(cells[8]));
            model.setEndScanTime(StringUtils.isEmpty(cells[9]) ? "" : String.valueOf(cells[9]));
            model.setShippingStatus(StringUtils.isEmpty(cells[10]) ? 0 : Integer.parseInt(String.valueOf(cells[10])));
            model.setCreateDatetime(StringUtils.isEmpty(cells[11]) ? "" : String.valueOf(cells[11]));
            model.setCreateUser(StringUtils.isEmpty(cells[12]) ? "" : String.valueOf(cells[12]));
            model.setModifyDatetime(StringUtils.isEmpty(cells[13]) ? "" : String.valueOf(cells[13]));
            model.setModifyUser(StringUtils.isEmpty(cells[14]) ? "" : String.valueOf(cells[14]));
            model.setCustCode(StringUtils.isEmpty(cells[15]) ? "" : String.valueOf(cells[15]));
            model.setCustOrganizeCode(StringUtils.isEmpty(cells[16]) ? "" : String.valueOf(cells[16]));
            model.setShippingGroupCode(StringUtils.isEmpty(cells[17]) ? "" : String.valueOf(cells[17]));
            model.setRemark(StringUtils.isEmpty(cells[18]) ? "" : String.valueOf(cells[18]));

            modelList.add(model);
        }

        return modelList;

    }

    @Override
    public int queryLoadingListDetailModelCount(MesLoadingListDetail detail) {
        StringBuffer hql = new StringBuffer("select count(1)  ");
        hql.append(" from mes_loading_list as mll");
        hql.append(" inner join  mes_loading_list_detail as mlld on mll.id = mlld.pid");
        hql.append(" and mll.organize_code = mlld.organize_code and mll.is_deleted = mlld.is_deleted and mll.is_valid = mlld.is_valid  ");
        hql.append(" left join  mes_shipping as ms on mlld.shipping_code = ms.shipping_code  ");
        hql.append(" and ((mlld.organize_code = ms.organize_code and mlld.is_deleted = ms.is_deleted and mlld.is_valid = ms.is_valid) or (ms.id is null))  ");
        hql.append(" where mll.organize_code = :organizeCode ");
        hql.append(" and mll.is_deleted = :isDeleted ");
        hql.append(" and mll.is_valid = :isValid ");
        hql.append(" and mlld.pid = :pid ");

        if (!StringUtils.isEmpty(detail.getShippingCode())) {
            hql.append(" and mlld.shipping_code = :shippingCode ");
        }
        if (!StringUtils.isEmpty(detail.getStatus())) {
            hql.append(" and ms.status = :status ");
        }

        Query query = entityManager.createNativeQuery(hql.toString());

        query.setParameter("organizeCode", detail.getOrganizeCode());
        query.setParameter("isValid", CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
        query.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        query.setParameter("pid", detail.getPid());

        if (!StringUtils.isEmpty(detail.getShippingCode())) {
            query.setParameter("shippingCode", detail.getShippingCode());
        }
        if (!StringUtils.isEmpty(detail.getStatus())) {
            query.setParameter("status", detail.getStatus());
        }

        List list = query.getResultList();

        if (CollectionUtils.isEmpty(list)) {
            return 0;
        }

        return list.size();
    }
}
