package cn.estsh.i3plus.ext.mes.apiservice.daoimpl;

import cn.estsh.i3plus.ext.mes.apiservice.dao.IMesModuleExcelDao;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesModuleExcel;
import cn.estsh.i3plus.pojo.mes.model.MesModuleExcelModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.List;

/**
 * @Description : 模块EXCEL管理
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/6/17 14:30
 * @Modify:
 **/
@Service
public class MesModuleExcelDaoImpl implements IMesModuleExcelDao {

    @Autowired
    private EntityManager entityManager;

    @Override
    public int queryModuleExcelCount(MesModuleExcel mesModuleExcel) {
        StringBuffer hql = new StringBuffer();
        hql.append(" select count(distinct mem.id) ");

        packHqlForQueryModuleExcelManage(hql, mesModuleExcel);

        Query query = entityManager.createQuery(hql.toString(), Long.class);

        packParameterForQueryModuleExcelManage(query, mesModuleExcel);

        return ((Long) query.getSingleResult()).intValue();
    }

    @Override
    public List<MesModuleExcelModel> queryModuleExcelByPager(MesModuleExcel mesModuleExcel, Pager pager) {
        StringBuffer hql = new StringBuffer();
        hql.append(" select new " + MesModuleExcelModel.class.getName() + "(mem.id, mem.moduleCode, mem.moduleName, mem.fileId, f.fileUrl, f.fileOriginName, " +
                " mem.organizeCode, mem.createUser, mem.createDatetime, mem.modifyUser, mem.modifyDatetime ) ");

        packHqlForQueryModuleExcelManage(hql, mesModuleExcel);

        hql.append(" order by mem.modifyDatetime desc ");
        Query query = entityManager.createQuery(hql.toString(), MesModuleExcelModel.class);

        packParameterForQueryModuleExcelManage(query, mesModuleExcel);

        query.setMaxResults(pager.getPageSize());
        if (pager.getCurrentPage() == 0) {
            pager.setCurrentPage(1);
        }
        query.setFirstResult(pager.getCurrentPage() * pager.getPageSize() - pager.getPageSize());
        return query.getResultList();
    }

    private void packHqlForQueryModuleExcelManage(StringBuffer hql, MesModuleExcel mesModuleExcel) {
        hql.append(" from MesModuleExcel as mem ");
        hql.append(" left join MesFile as f on (mem.fileId = f.id and f.organizeCode = :organizeCode and f.isValid = :isValid1 and f.isDeleted = :isDeleted) ");
        hql.append(" where mem.organizeCode = :organizeCode ");
        hql.append(" and mem.isValid = :isValid ");
        hql.append(" and mem.isDeleted = :isDeleted ");
        if (!StringUtils.isEmpty(mesModuleExcel.getModuleCode())) {
            //主数据单独下载模版时不能模糊查询
            if (!StringUtils.isEmpty(mesModuleExcel.getQueryIsAllowLike()) && mesModuleExcel.getQueryIsAllowLike().compareTo(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()) == 0) {
                hql.append(" and mem.moduleCode like :moduleCode ");
            } else {
                hql.append(" and mem.moduleCode = :moduleCode ");
            }
        }
        if (!StringUtils.isEmpty(mesModuleExcel.getModuleName())) {
            hql.append(" and mem.moduleName like :moduleName ");
        }
    }

    private void packParameterForQueryModuleExcelManage(Query query, MesModuleExcel mesModuleExcel) {
        query.setParameter(MesExtConstWords.ORGANIZE_CODE, mesModuleExcel.getOrganizeCode());
        query.setParameter(MesExtConstWords.IS_VALID, mesModuleExcel.getIsValid());
        query.setParameter(MesExtConstWords.IS_VALID1, CommonEnumUtil.IS_VAILD.VAILD.getValue());
        query.setParameter(MesExtConstWords.IS_DELETED, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        if (!StringUtils.isEmpty(mesModuleExcel.getModuleCode())) {
            if (!StringUtils.isEmpty(mesModuleExcel.getQueryIsAllowLike()) && mesModuleExcel.getQueryIsAllowLike().compareTo(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()) == 0) {
                query.setParameter(MesExtConstWords.MODULE_CODE, "%" + mesModuleExcel.getModuleCode() + "%");
            } else {
                query.setParameter(MesExtConstWords.MODULE_CODE, mesModuleExcel.getModuleCode());
            }
        }
        if (!StringUtils.isEmpty(mesModuleExcel.getModuleName())) {
            query.setParameter(MesExtConstWords.MODULE_NAME, "%" + mesModuleExcel.getModuleName() + "%");
        }
    }
}
