package cn.estsh.i3plus.ext.mes.apiservice.daoimpl;

import cn.estsh.i3plus.ext.mes.apiservice.dao.IMesWorkOrderDao;
import cn.estsh.i3plus.ext.mes.apiservice.utils.BeanUtil;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.model.MesWorkOrderModel;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import lombok.extern.slf4j.Slf4j;
import org.hibernate.SQLQuery;
import org.hibernate.transform.Transformers;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * @Description :SAP生产计划
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/5/6 15:52
 * @Modify:
 **/
@Service
@Slf4j
public class MesWorkOrderDaoImpl implements IMesWorkOrderDao {

    @Autowired
    private EntityManager entityManager;

    @Override
    public List<MesWorkOrderModel> sumMesWorkOrderQtyToSap(String organizeCode) {
        StringBuffer hql = new StringBuffer("select workOrder.organize_code as organizeCode,workOrder.part_no as partNo, " +
                " date_format(workOrder.plan_start_time, '%Y-%m-%d' ) as planStartTime, workOrder.product_version as productVersion, " +
                " sum(case when workOrder.work_order_status = 40 then workOrder.reported_qty else workOrder.qty end ) as qty  " +
                " from mes_work_order as workOrder left join mes_part_sap as partSap on workOrder.part_no = partSap.part_no" +
                " where workOrder.is_deleted=:isDeleted and workOrder.is_valid = :isValid and workOrder.organize_code = :organizeCode " +
                " and partSap.is_deleted=:isDeleted and partSap.is_valid = :isValid and partSap.organize_code = :organizeCode " +
                " and partSap.is_test_piece = :isTestPiece and workOrder.work_order_type = :workOrderType and workOrder.order_flag != :orderFlag " +
                " and workOrder.work_order_status in (20,30,40,50) and workOrder.plan_start_time >= :today" +
                " group by workOrder.organize_code,workOrder.part_no, date_format(workOrder.plan_start_time, '%Y-%m-%d' ), workOrder.product_version ");
        Query query = entityManager.createNativeQuery(hql.toString());
        query.setParameter("organizeCode", organizeCode);
        query.setParameter("isValid", CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
        query.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        query.setParameter("isTestPiece", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        query.setParameter("today", TimeTool.getToday());
        query.setParameter("workOrderType", MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue());
        query.setParameter("orderFlag", MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue());
        query.unwrap(SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);

        List<HashMap<String, Object>> hashMaps = query.getResultList();
        List<MesWorkOrderModel> mesWorkOrderList =new ArrayList<>();
        for (HashMap<String, Object> hashMap : hashMaps) {
            try {
                mesWorkOrderList.add((MesWorkOrderModel) BeanUtil.populateBean(hashMap, MesWorkOrderModel.class));
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        return mesWorkOrderList;
    }
}
