package cn.estsh.i3plus.ext.mes.apiservice.daoimpl.report;

import cn.estsh.i3plus.ext.mes.apiservice.dao.report.IAssemblyPartReportDao;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.model.report.AssemblyPartReportModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.List;

/**
 * @Description : 装配件查询
 * @Reference :
 * @Author : PZJ
 * @CreateDate : 2024/9/29 10:21
 * @Modify:
 **/
@Service
public class AssemblyPartReportDaoImpl implements IAssemblyPartReportDao {
    @Autowired
    private EntityManager entityManager;

    @Override
    public ListPager<AssemblyPartReportModel> queryReport(AssemblyPartReportModel model, Pager pager) {
        StringBuilder dataQueryHql = new StringBuilder();
        dataQueryHql.append("SELECT new ").append(AssemblyPartReportModel.class.getName()).append("(");
        dataQueryHql.append("a.organizeCode,");
        dataQueryHql.append("a.custSn,");
        dataQueryHql.append("a.serialNumber,");
        dataQueryHql.append("a.workCenterCode,");
        dataQueryHql.append("a.workCellCode,");
        dataQueryHql.append("a.partNo,");
        dataQueryHql.append("a.partName,");
        dataQueryHql.append("a.matchType,");
        dataQueryHql.append("a.matchRule,");
        dataQueryHql.append("a.assemblyPartNo,");
        dataQueryHql.append("a.assemblyPartName,");
        dataQueryHql.append("a.assemblySn,");
        dataQueryHql.append("a.productSn,");
        dataQueryHql.append("a.isSkip,");
        dataQueryHql.append("a.createDatetime,");
        dataQueryHql.append("a.createUser,");
        dataQueryHql.append("b.workOrderNo,");
        dataQueryHql.append("b.lotNo,");
        dataQueryHql.append("c.equipmentName,");
        dataQueryHql.append("c.equipId ");
        dataQueryHql.append(") ");

        StringBuilder countQueryHql = new StringBuilder();
        countQueryHql.append("select count(1)");
        StringBuilder commonQueryHql = new StringBuilder();
        commonQueryHql.append("FROM MesProductionAssembly a ");
        commonQueryHql.append("JOIN MesProductionRecord b ON a.organizeCode=b.organizeCode AND a.productionRecordId = b.id ");
        commonQueryHql.append("JOIN MesEquipment c ON c.organizeCode=b.organizeCode AND b.equipmentCode = c.equipmentCode ");
        commonQueryHql.append("WHERE 1=1 ");
        commonQueryHql.append("AND a.organizeCode = :organizeCode ");
        commonQueryHql.append("AND b.organizeCode = :organizeCode ");
        commonQueryHql.append("AND c.organizeCode = :organizeCode ");
        commonQueryHql.append("AND a.isDeleted = :isDeleted ");
        commonQueryHql.append("AND b.isDeleted = :isDeleted ");
        commonQueryHql.append("AND c.isDeleted = :isDeleted ");
        commonQueryHql.append("AND a.isValid = :isValid ");
        commonQueryHql.append("AND b.isValid = :isValid ");
        commonQueryHql.append("AND c.isValid = :isValid ");
        if (!StringUtils.isEmpty(model.getCustSn())) {
            commonQueryHql.append("AND a.custSn like :custSn ");
        }
        // like 条码
        if (!StringUtils.isEmpty(model.getSerialNumber())) {
            commonQueryHql.append("AND a.serialNumber like :serialNumber ");
        }
        // = 装配件条码确认规则
        if (!StringUtils.isEmpty(model.getMatchRule())) {
            commonQueryHql.append("AND a.matchRule = :matchRule ");
        }
        // like 零件号
        if (!StringUtils.isEmpty(model.getPartNo())) {
            commonQueryHql.append("AND a.partNo like :partNo ");
        }
        //like 主零件号
        if (!StringUtils.isEmpty(model.getPartName())) {
            commonQueryHql.append("AND a.partName like :partName ");
        }
        //like 装配零件号
        if (!StringUtils.isEmpty(model.getAssemblyPartNo())) {
            commonQueryHql.append("AND a.assemblyPartNo like :assemblyPartNo ");
        }
        //like 装配件零件名称
        if (!StringUtils.isEmpty(model.getAssemblyPartName())) {
            commonQueryHql.append("AND a.assemblyPartName like :assemblyPartName ");
        }
        //like 装配条码
        if (!StringUtils.isEmpty(model.getAssemblySn())) {
            commonQueryHql.append("AND a.assemblySn like :assemblySn ");
        }
        // = 是否跳过
        if (model.getIsSkip() != null) {
            commonQueryHql.append("AND a.isSkip = :isSkip ");
        }
        //like 创建人
        if (!StringUtils.isEmpty(model.getCreateUser())) {
            commonQueryHql.append("AND a.createUser like :createUser ");
        }
        //范围时间 创建时间
        if (!StringUtils.isEmpty(model.getCreateDateTimeStart()) && !StringUtils.isEmpty(model.getCreateDateTimeEnd())) {
            commonQueryHql.append("AND a.createDatetime between :createDateTimeStart and :createDateTimeEnd ");
        }
        //like 产线
        if (!StringUtils.isEmpty(model.getWorkCenterCode())) {
            commonQueryHql.append("AND a.workCenterCode like :workCenterCode ");
        }
        //like 工位编号
        if (!StringUtils.isEmpty(model.getWorkCellCode())) {
            commonQueryHql.append("AND a.workCellCode like :workCellCode ");
        }
        if (model.getMatchType() != null) {
            commonQueryHql.append("AND a.matchRule = :matchRule ");
        }
        //like 加工单号
        if (!StringUtils.isEmpty(model.getWorkOrderNo())) {
            commonQueryHql.append("AND b.workOrderNo like :workOrderNo ");
        }
        //like 设备名称
        if (!StringUtils.isEmpty(model.getEquipmentName())) {
            commonQueryHql.append("AND c.equipmentName like :equipmentName ");
        }
        if (model.getEquipId() != null) {
            commonQueryHql.append("AND c.equipId = :equipId ");
        }
        commonQueryHql.append("ORDER BY a.createDatetime DESC ");

        Query dataQuery = entityManager.createQuery(dataQueryHql.append(commonQueryHql).toString(), AssemblyPartReportModel.class);
        setQueryParams(dataQuery, model);
        Query countQuery = entityManager.createQuery(countQueryHql.append(commonQueryHql).toString());
        setQueryParams(countQuery, model);

        //本次查询满足条件的总数
        int count = Integer.parseInt(String.valueOf(countQuery.getSingleResult()));
        pager = PagerHelper.getPager(pager, count);
        dataQuery.setFirstResult(pager.getStartRow()).setMaxResults(pager.getPageSize());
        List<AssemblyPartReportModel> resultList = dataQuery.getResultList();
        return new ListPager<>(resultList, pager);
    }

    private void setQueryParams(Query dataQuery, AssemblyPartReportModel model) {
        dataQuery.setParameter("organizeCode", model.getOrganizeCode());
        dataQuery.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        dataQuery.setParameter("isValid", CommonEnumUtil.IS_VAILD.VAILD.getValue());
        if (!StringUtils.isEmpty(model.getCustSn())) {
            dataQuery.setParameter("custSn", model.getCustSn() + "%");
        }
        if (!StringUtils.isEmpty(model.getSerialNumber())) {
            dataQuery.setParameter("serialNumber", model.getSerialNumber() + "%");
        }
        if (model.getMatchType() != null) {
            dataQuery.setParameter("matchType", model.getMatchType());
        }
        if (!StringUtils.isEmpty(model.getPartNo())) {
            dataQuery.setParameter("partNo", model.getPartNo() + "%");
        }
        if (!StringUtils.isEmpty(model.getPartName())) {
            dataQuery.setParameter("partName", model.getPartName() + "%");
        }
        if (!StringUtils.isEmpty(model.getAssemblyPartNo())) {
            dataQuery.setParameter("assemblyPartNo", model.getAssemblyPartNo() + "%");
        }
        if (!StringUtils.isEmpty(model.getAssemblyPartName())) {
            dataQuery.setParameter("assemblyPartName", model.getAssemblyPartName() + "%");
        }
        if (!StringUtils.isEmpty(model.getAssemblySn())) {
            dataQuery.setParameter("assemblySn", model.getAssemblySn() + "%");
        }
        if (model.getIsSkip() != null) {
            dataQuery.setParameter("isSkip", model.getIsSkip());
        }
        if (!StringUtils.isEmpty(model.getCreateUser())) {
            dataQuery.setParameter("createUser", model.getCreateUser() + "%");
        }
        if (!StringUtils.isEmpty(model.getCreateDateTimeStart()) && !StringUtils.isEmpty(model.getCreateDateTimeEnd())) {
            dataQuery.setParameter("createDateTimeStart", model.getCreateDateTimeStart());
            dataQuery.setParameter("createDateTimeEnd", model.getCreateDateTimeEnd());
        }
        if (!StringUtils.isEmpty(model.getWorkCenterCode())) {
            dataQuery.setParameter("workCenterCode", model.getWorkCenterCode() + "%");
        }
        if (!StringUtils.isEmpty(model.getWorkCellCode())) {
            dataQuery.setParameter("workCellCode", model.getWorkCellCode() + "%");
        }
        if (model.getMatchType() != null) {
            dataQuery.setParameter("matchRule", model.getMatchRule());
        }
        if (!StringUtils.isEmpty(model.getWorkOrderNo())) {
            dataQuery.setParameter("workOrderNo", model.getWorkOrderNo() + "%");
        }
        if (!StringUtils.isEmpty(model.getEquipmentName())) {
            dataQuery.setParameter("equipmentName", model.getEquipmentName() + "%");
        }
        if (model.getEquipId() != null) {
            dataQuery.setParameter("equipId", model.getEquipId());
        }
    }
}
