package cn.estsh.i3plus.ext.mes.apiservice.daoimpl.report;

import cn.estsh.i3plus.ext.mes.apiservice.dao.report.IMesProductionRecordReportExtDao;
import cn.estsh.i3plus.ext.mes.pojo.model.report.MesProductionRecordReportExtModel;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.ArrayList;
import java.util.List;

/**
 * @Description : 加工记录合并批量汇报查询报表
 * @Reference :
 * @Author : wangjie
 * @CreateDate : 2024/9/29 10:21
 * @Modify:
 **/
@Slf4j
@Service
public class MesProductionRecordReportExtDao implements IMesProductionRecordReportExtDao {

    @Autowired
    private EntityManager entityManager;

    @Override
    public ListPager<MesProductionRecordReportExtModel> queryProductionRecordUnionWorkOrderLogReport(
            MesProductionRecordReportExtModel model, Pager pager, List<Integer> reportStatusList2Pr, List<Integer> reportStatusList2Wo) {

        List<MesProductionRecordReportExtModel> resultList = new ArrayList<>();

        StringBuilder queryHql = new StringBuilder();
        StringBuilder countHql = new StringBuilder();

        StringBuilder unionQueryHql = new StringBuilder();
        StringBuilder countQueryHql = new StringBuilder();
        StringBuilder prQueryHql = new StringBuilder();
        StringBuilder woQueryHql = new StringBuilder();
        StringBuilder endHql = new StringBuilder();
        StringBuilder limitHql = new StringBuilder();

        if (!CollectionUtils.isEmpty(reportStatusList2Pr) && !CollectionUtils.isEmpty(reportStatusList2Wo)) {
            unionQueryHql.append("SELECT " +
                    " o.workCenterCode, " +
                    " o.workCellCode, " +
                    " o.partNo, " +
                    " o.partName, " +
                    " o.shiftCode, " +
                    " o.equipmentCode, " +
                    " o.equipmentName, " +
                    " o.reportStatus, " +
                    " o.dataSource, " +
                    " o.statisticsQty " +
                    " FROM " +
                    " (( ");

            countQueryHql.append("SELECT count(1) " +
                    " FROM " +
                    " (( ");
        } else {
            countQueryHql.append("SELECT count(1) FROM ( ");
        }

        if (!CollectionUtils.isEmpty(reportStatusList2Pr)) {
            prQueryHql.append("" +
                    " SELECT " +
                    " a.work_center_code AS workCenterCode, " +
                    " a.work_cell_code AS workCellCode, " +
                    " a.part_no AS partNo, " +
                    " a.part_name AS partName, " +
                    " a.shift_code AS shiftCode, " +
                    " a.equipment_code AS equipmentCode, " +
                    " a.equipment_name AS equipmentName, " +
                    " a.report_status AS reportStatus, " +
                    " 1 AS dataSource, " +
                    " SUM( qty ) AS statisticsQty " +
                    " FROM ");

            prQueryHql.append(" mes_production_record a " +
                    " WHERE " +
                    " a.organize_code = :organizeCode " +
                    " AND a.is_deleted = :isDeleted " +
                    " AND a.is_valid = :isValid " +
                    " AND a.complete_date_time >= :completeDateTimeStart " +
                    " AND a.complete_date_time <= :completeDateTimeEnd ");
            if (reportStatusList2Pr.size() == 1) {
                prQueryHql.append(" AND a.report_status = :reportStatus ");
            } else {
                prQueryHql.append(" AND a.report_status in (:reportStatus) ");
            }
            if (!StringUtils.isEmpty(model.getPartNo())) {
                prQueryHql.append(" AND a.part_no = :partNo ");
            }
            if (!StringUtils.isEmpty(model.getWorkCenterCode())) {
                prQueryHql.append(" AND a.work_center_code = :workCenterCode ");
            }
            if (!StringUtils.isEmpty(model.getWorkCellCode())) {
                prQueryHql.append(" AND a.work_cell_code = :workCellCode ");
            }
            if (!StringUtils.isEmpty(model.getShiftCode())) {
                prQueryHql.append(" AND a.shift_code = :shiftCode ");
            }
            if (!StringUtils.isEmpty(model.getEquipmentCode())) {
                prQueryHql.append(" AND a.equipment_code = :equipmentCode ");
            }
            prQueryHql.append(" GROUP BY " +
                    " a.work_center_code, " +
                    " a.work_cell_code, " +
                    " a.part_no, " +
                    " a.shift_code, " +
                    " a.report_status " +
                    " ORDER BY " +
                    " a.complete_date_time DESC ");

        }

        if (!CollectionUtils.isEmpty(reportStatusList2Pr) && !CollectionUtils.isEmpty(reportStatusList2Wo)) {
            prQueryHql.append(" ) UNION ALL ( ");
        }

        if (!CollectionUtils.isEmpty(reportStatusList2Wo)) {
            woQueryHql.append("" +
                    " SELECT " +
                    " b.work_center_code AS workCenterCode, " +
                    " ' ' AS workCellCode, " +
                    " b.part_no AS partNo, " +
                    " b.part_name AS partName, " +
                    " b.shift_code AS shiftCode, " +
                    " ' ' AS equipmentCode, " +
                    " ' ' AS equipmentName, " +
                    " b.type AS reportStatus, " +
                    " 2 AS dataSource, " +
                    " SUM( modify_qty ) AS statisticsQty " +
                    " FROM ");

            woQueryHql.append(" mes_work_order_log b " +
                    " WHERE " +
                    " b.organize_code = :organizeCode " +
                    " AND b.is_deleted = :isDeleted " +
                    " AND b.is_valid = :isValid " +
                    " AND b.modify_date_time >= :completeDateTimeStart " +
                    " AND b.modify_date_time <= :completeDateTimeEnd ");
            if (reportStatusList2Wo.size() == 1) {
                woQueryHql.append(" AND b.type = :messageType ");
            } else {
                woQueryHql.append(" AND b.type in (:messageType) ");
            }
            if (!StringUtils.isEmpty(model.getPartNo())) {
                woQueryHql.append(" AND b.part_no = :partNo ");
            }
            if (!StringUtils.isEmpty(model.getWorkCenterCode())) {
                woQueryHql.append(" AND b.work_center_code = :workCenterCode ");
            }
            if (!StringUtils.isEmpty(model.getShiftCode())) {
                woQueryHql.append(" AND b.shift_code = :shiftCode ");
            }
            woQueryHql.append(" GROUP BY " +
                    " b.work_center_code, " +
                    " b.part_no, " +
                    " b.shift_code " +
                    " ORDER BY " +
                    " b.modify_date_time DESC ");

        }

        if (!CollectionUtils.isEmpty(reportStatusList2Pr) && !CollectionUtils.isEmpty(reportStatusList2Wo)) {
            endHql.append(" )) AS o ");
        } else {
            endHql.append(" ) AS o ");
        }

        limitHql.append(" LIMIT :startRow , :pageSize ");

        if (!CollectionUtils.isEmpty(reportStatusList2Pr) && !CollectionUtils.isEmpty(reportStatusList2Wo)) {
            queryHql.append(unionQueryHql).append(prQueryHql).append(woQueryHql).append(endHql).append(limitHql);
            countHql.append(countQueryHql).append(prQueryHql).append(woQueryHql).append(endHql);
        } else {
            if (!CollectionUtils.isEmpty(reportStatusList2Pr)) {
                queryHql.append(prQueryHql).append(limitHql);
                countHql.append(countQueryHql).append(prQueryHql).append(endHql);
            } else {
                queryHql.append(woQueryHql).append(limitHql);
                countHql.append(countQueryHql).append(woQueryHql).append(endHql);
            }
        }

//        log.info("queryHql: --- {}", queryHql.toString());
//        log.info("countHql: --- {}", countHql.toString());

        Query dataQuery = entityManager.createNativeQuery(queryHql.toString());
        setQueryParams(dataQuery, model, reportStatusList2Pr, reportStatusList2Wo, pager);
        Query countQuery = entityManager.createNativeQuery(countHql.toString());
        setQueryParams(countQuery, model, reportStatusList2Pr, reportStatusList2Wo, null);

        Object o = countQuery.getSingleResult();

        int count = Integer.parseInt(o.toString());

        List list = dataQuery.getResultList();

        if (!CollectionUtils.isEmpty(list)) {

            for (Object object : list) {

                if (null == object) continue;

                Object[] objectArr = (Object[]) object;

                if (null == objectArr || objectArr.length != 10) continue;

                MesProductionRecordReportExtModel result = new MesProductionRecordReportExtModel(
                        objectArr[0], objectArr[1], objectArr[2], objectArr[3], objectArr[4], objectArr[5], objectArr[6], objectArr[7], objectArr[8], objectArr[9]);
                if (!StringUtils.isEmpty(result.getDataSource()) && !StringUtils.isEmpty(result.getReportStatus())) {
                    if (result.getDataSource().compareTo(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()) == 0) {
                        if (result.getReportStatus().compareTo(MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_20.getValue()) == 0) result.setReportStatusName(MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_20.getDescription());
                        else result.setReportStatusName(MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_10.getDescription());
                    } else {
                        result.setReportStatusName(MesExtEnumUtil.WORK_ORDER_LOG_TYPE.valueOfDescription(result.getReportStatus()));
                    }
                }

                resultList.add(result);
            }
        }

        pager = PagerHelper.getPager(pager, count);
        return new ListPager(resultList, pager);

    }

    private void setQueryParams(Query query, MesProductionRecordReportExtModel model, List<Integer> reportStatusList2Pr, List<Integer> reportStatusList2Wo, Pager pager) {
        query.setParameter(MesExtConstWords.ORGANIZE_CODE, model.getOrganizeCode());
        query.setParameter(MesExtConstWords.IS_DELETED, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        query.setParameter(MesExtConstWords.IS_VALID, CommonEnumUtil.IS_VAILD.VAILD.getValue());
        query.setParameter(MesExtConstWords.COMPLETE_DATE_TIME_START, model.getCompleteDateTimeStart());
        query.setParameter(MesExtConstWords.COMPLETE_DATE_TIME_END, model.getCompleteDateTimeEnd());
        if (!CollectionUtils.isEmpty(reportStatusList2Pr)) {
            if (reportStatusList2Pr.size() == 1) {
                query.setParameter(MesExtConstWords.REPORT_STATUS, reportStatusList2Pr.get(0));
            } else {
                query.setParameter(MesExtConstWords.REPORT_STATUS, reportStatusList2Pr);
            }
        }
        if (!CollectionUtils.isEmpty(reportStatusList2Wo)) {
            if (reportStatusList2Wo.size() == 1) {
                query.setParameter(MesExtConstWords.MESSAGE_TYPE, reportStatusList2Wo.get(0));
            } else {
                query.setParameter(MesExtConstWords.MESSAGE_TYPE, reportStatusList2Wo);
            }
        }
        if (!StringUtils.isEmpty(model.getPartNo())) {
            query.setParameter(MesExtConstWords.PART_NO, model.getPartNo());
        }
        if (!StringUtils.isEmpty(model.getWorkCenterCode())) {
            query.setParameter(MesExtConstWords.WORK_CENTER_CODE, model.getWorkCenterCode());
        }
        if (!StringUtils.isEmpty(model.getWorkCellCode())) {
            query.setParameter(MesExtConstWords.WORK_CELL_CODE, model.getWorkCellCode());
        }
        if (!StringUtils.isEmpty(model.getShiftCode())) {
            query.setParameter(MesExtConstWords.SHIFT_CODE, model.getShiftCode());
        }
        if (!StringUtils.isEmpty(model.getEquipmentCode())) {
            query.setParameter(MesExtConstWords.EQUIPMENT_CODE, model.getEquipmentCode());
        }
        if (null != pager) {
            Integer startRow = (pager.getCurrentPage() - 1) * pager.getPageSize();
            query.setParameter("startRow", startRow);
            query.setParameter("pageSize", pager.getPageSize());
        }
    }

}
