package cn.estsh.i3plus.ext.mes.apiservice.daoimpl.report;

import cn.estsh.i3plus.ext.mes.apiservice.dao.report.IMesYfReportDao;
import cn.estsh.i3plus.ext.mes.pojo.model.report.MesDeviceDataQueryReportConditionModel;
import cn.estsh.i3plus.ext.mes.pojo.model.report.MesDeviceDataQueryReportResultModel;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.model.MesProductionRecordModel;
import cn.estsh.i3plus.pojo.mes.model.report.MesGeneralExportReportModel;
import cn.estsh.i3plus.pojo.mes.model.report.MesGeneralShippingReportModel;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Description : 报表-数据方法实现
 * @Reference :
 * @Author : logic
 * @CreateDate : 2024/6/15 13:30
 * @Modify:
 **/
@Service
public class MesYfReportDaoImpl implements IMesYfReportDao {

    @Autowired
    private EntityManager entityManager;

    @Override
    public ListPager<MesDeviceDataQueryReportResultModel> queryMesDeviceDataByPager(MesDeviceDataQueryReportConditionModel queryReportModel, Pager pager) {
        StringBuilder dataQuerySql = new StringBuilder();
        StringBuilder countQuerySql = new StringBuilder();
        StringBuilder commonQuerySql = new StringBuilder();
        dataQuerySql.append("select new ")
                .append(MesDeviceDataQueryReportResultModel.class.getName())
                .append("(organizeCode, workCenterCode, workCellCode, workOrderNo, serialNumber, ")
                .append("logType, message, equipmentCode, equipmentName, createDatetime, createUser, " +
                        "modifyDatetime, modifyUser) ");
        countQuerySql.append("select count(1) ");
        commonQuerySql.append("from MesWorkCellScanMonitorLog " +
                "where isValid = :isValid and isDeleted = :isDeleted and organizeCode = :organizeCode " +
                "and createDatetime >= :createDateTimeStart and createDatetime <= :createDateTimeEnd ");
        //非空查询条件拼接语句
        if (!StringUtils.isEmpty(queryReportModel.getWorkCenterCode())) {
            commonQuerySql.append("and workCenterCode = :workCenterCode ");
        }
        if (!StringUtils.isEmpty(queryReportModel.getWorkCellCode())) {
            commonQuerySql.append("and workCellCode = :workCellCode ");
        }
        if (!StringUtils.isEmpty(queryReportModel.getWorkOrderNo())) {
            commonQuerySql.append("and workOrderNo = :workOrderNo ");
        }
        if (!StringUtils.isEmpty(queryReportModel.getSerialNumber())) {
            commonQuerySql.append("and serialNumber = :serialNumber ");
        }
        if (!StringUtils.isEmpty(queryReportModel.getEquipmentCode())) {
            commonQuerySql.append("and equipmentCode = :equipmentCode ");
        }
        if (!StringUtils.isEmpty(queryReportModel.getLogType())) {
            commonQuerySql.append("and logType = :logType ");
        }
        Query dataQuery = entityManager.createQuery(dataQuerySql.append(commonQuerySql).toString(), MesDeviceDataQueryReportResultModel.class);
        dataQuery.setParameter("isValid", CommonEnumUtil.IS_VAILD.VAILD.getValue());
        dataQuery.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        dataQuery.setParameter("organizeCode", queryReportModel.getOrganizeCode());
        dataQuery.setParameter("createDateTimeStart", queryReportModel.getCreateDateTimeStart());
        dataQuery.setParameter("createDateTimeEnd", queryReportModel.getCreateDateTimeEnd());
        Query countQuery = entityManager.createQuery(countQuerySql.append(commonQuerySql).toString());
        countQuery.setParameter("isValid", CommonEnumUtil.IS_VAILD.VAILD.getValue());
        countQuery.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        countQuery.setParameter("organizeCode", queryReportModel.getOrganizeCode());
        countQuery.setParameter("createDateTimeStart", queryReportModel.getCreateDateTimeStart());
        countQuery.setParameter("createDateTimeEnd", queryReportModel.getCreateDateTimeEnd());
        if (!StringUtils.isEmpty(queryReportModel.getWorkCenterCode())) {
            dataQuery.setParameter("workCenterCode", queryReportModel.getWorkCenterCode());
            countQuery.setParameter("workCenterCode", queryReportModel.getWorkCenterCode());
        }
        if (!StringUtils.isEmpty(queryReportModel.getWorkCellCode())) {
            dataQuery.setParameter("workCellCode", queryReportModel.getWorkCellCode());
            countQuery.setParameter("workCellCode", queryReportModel.getWorkCellCode());
        }
        if (!StringUtils.isEmpty(queryReportModel.getWorkOrderNo())) {
            dataQuery.setParameter("workOrderNo", queryReportModel.getWorkOrderNo());
            countQuery.setParameter("workOrderNo", queryReportModel.getWorkOrderNo());
        }
        if (!StringUtils.isEmpty(queryReportModel.getSerialNumber())) {
            dataQuery.setParameter("serialNumber", queryReportModel.getSerialNumber());
            countQuery.setParameter("serialNumber", queryReportModel.getSerialNumber());
        }
        if (!StringUtils.isEmpty(queryReportModel.getEquipmentCode())) {
            dataQuery.setParameter("equipmentCode", queryReportModel.getEquipmentCode());
            countQuery.setParameter("equipmentCode", queryReportModel.getEquipmentCode());
        }
        if (!StringUtils.isEmpty(queryReportModel.getLogType())) {
            dataQuery.setParameter("logType", queryReportModel.getLogType());
            countQuery.setParameter("logType", queryReportModel.getLogType());
        }
        //本次查询满足条件的总数
        int count = Integer.parseInt(String.valueOf(countQuery.getSingleResult()));
        pager = PagerHelper.getPager(pager, count);
        //查询本页数据
        dataQuery.setMaxResults(pager.getPageSize());
        dataQuery.setFirstResult(pager.getCurrentPage() * pager.getPageSize() - pager.getPageSize());
        List<MesDeviceDataQueryReportResultModel> resultList = dataQuery.getResultList();
        return new ListPager<>(resultList, pager);
    }

    @Override
    public ListPager<MesGeneralExportReportModel> queryGeneralExport(MesGeneralExportReportModel mesGeneralExportReportModel, Pager pager) {
        StringBuilder dataHql = new StringBuilder();
        dataHql.append(" select new ").append(MesGeneralExportReportModel.class.getName()).append("(workOrder.organizeCode,workOrder.workCenterCode,workOrder.id,workOrder.workOrderNo,workOrder.custSn,partType.partTypeName,assembly.assemblySn,assembly.createDatetime,workOrder.shippingTime)");
        //共用HQL
        String commonHql = getGeneralExportCommonHql(mesGeneralExportReportModel);
        //行数
        Query countQuery = entityManager.createQuery("select count(1) " + commonHql);
        setGeneralExportParameter(mesGeneralExportReportModel, countQuery);
        //数据
        Query dataQuery = entityManager.createQuery(dataHql.append(commonHql).append(getGeneralExportOrderBy(mesGeneralExportReportModel)).toString(), MesGeneralExportReportModel.class);
        setGeneralExportParameter(mesGeneralExportReportModel, dataQuery);
        if (Objects.isNull(pager)) return new ListPager<>(dataQuery.getResultList(), pager);

        int count = Integer.parseInt(String.valueOf(countQuery.getSingleResult()));
        if (count <= 0) return new ListPager<>(new ArrayList<>(), pager);
        pager = PagerHelper.getPager(pager, count);
        dataQuery.setFirstResult(pager.getStartRow()).setMaxResults(pager.getPageSize());
        return new ListPager<>(dataQuery.getResultList(), pager);
    }


    @Override
    public ListPager<MesGeneralShippingReportModel> queryGeneralShipping(MesGeneralShippingReportModel model, Pager pager, List<String> shippingGroupList) {
        StringBuilder dataHql = new StringBuilder();
        dataHql.append(" select new ").append(MesGeneralShippingReportModel.class.getName()).append("(s.id,s.status,s.shippingCode,wo.workCenterCode,s.seq,wo.custOrderNo,sd.partNo,wo.custPartNo,wo.carModelCode,cgg.upperColour,s.endScanTime,sr.startCarNo,sr.endCarNo,sr.expectedArrivalTime,sr.createDatetime)");
        //共用HQL
        String commonHql = getGeneralShippingCommonHql(model, shippingGroupList);
        //行数
        Query countQuery = entityManager.createQuery("select count(1) " + commonHql);
        setGeneralShippingParameter(model, countQuery, shippingGroupList);
        //数据
        Query dataQuery = entityManager.createQuery(dataHql.append(commonHql).append(getGeneralShippingOrderBy(model)).toString(), MesGeneralShippingReportModel.class);
        setGeneralShippingParameter(model, dataQuery, shippingGroupList);
        if (Objects.isNull(pager)) return new ListPager<>(dataQuery.getResultList(), pager);

        int count = Integer.parseInt(String.valueOf(countQuery.getSingleResult()));
        if (count <= 0) return new ListPager<>(new ArrayList<>(), pager);
        pager = PagerHelper.getPager(pager, count);
        dataQuery.setFirstResult(pager.getStartRow()).setMaxResults(pager.getPageSize());
        return new ListPager<>(dataQuery.getResultList(), pager);
    }

    @Override
    public ListPager<MesProductionRecordModel> queryMesProductionRecord(MesProductionRecordModel model, Pager pager) {
        StringBuilder dataHql = new StringBuilder();
        dataHql.append(" select new ").append(MesProductionRecordModel.class.getName()).append("(record,wo.workOrderStatus,wo.orderFlag)");
        //共用HQL
        String commonHql = getMesProductionRecordCommonHql(model);
        //行数
        Query countQuery = entityManager.createQuery("select count(1) " + commonHql);
        setMesProductionRecordParameter(model, countQuery);
        //数据
        Query dataQuery = entityManager.createQuery(dataHql.append(commonHql).append(getMesProductionRecordOrderBy(model)).toString(), MesProductionRecordModel.class);
        setMesProductionRecordParameter(model, dataQuery);
        if (Objects.isNull(pager)) return new ListPager<>(dataQuery.getResultList(), pager);

        int count = Integer.parseInt(String.valueOf(countQuery.getSingleResult()));
        if (count <= 0) return new ListPager<>(new ArrayList<>(), pager);
        pager = PagerHelper.getPager(pager, count);
        dataQuery.setFirstResult(pager.getStartRow()).setMaxResults(pager.getPageSize());
        return new ListPager<>(dataQuery.getResultList(), pager);
    }

    private void setGeneralExportParameter(MesGeneralExportReportModel model, Query query) {
        query.setParameter("isValid", CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
        query.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        query.setParameter("organizeCode", model.getOrganizeCode());
        query.setParameter("workOrderStatus", Stream.of(MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue(),MesExtEnumUtil.ORDER_STATUS.COMPLETE.getValue(),MesExtEnumUtil.ORDER_STATUS.SHIPPING.getValue()).collect(Collectors.toList()));
        if (!StringUtils.isEmpty(model.getShippingTimeStart())) query.setParameter("shippingTimeStart", model.getShippingTimeStart());
        if (!StringUtils.isEmpty(model.getShippingTimeEnd())) query.setParameter("shippingTimeEnd", model.getShippingTimeEnd());
    }

    private String getGeneralExportOrderBy(MesGeneralExportReportModel model) {
        StringBuilder orderBy = new StringBuilder();
        if (StringUtils.isEmpty(model.getOrderByParam()) || StringUtils.isEmpty(model.getAscOrDesc())) {
            model.setOrderByParam("createDatetime");
            model.setAscOrDesc(CommonEnumUtil.ASC_OR_DESC.DESC.getValue());
        }
        return orderBy.append(" order by assembly.").append(model.getOrderByParam()).append(" ").append(CommonEnumUtil.ASC_OR_DESC.DESC.getValue() == model.getAscOrDesc() ? "desc" : "asc").toString();
    }

    private String getGeneralExportCommonHql(MesGeneralExportReportModel model) {
        StringBuilder commonHql = new StringBuilder();
        commonHql.append(" from MesWorkOrderAssembly orderAssembly left join MesWorkOrder workOrder on workOrder.organizeCode = :organizeCode and workOrder.isDeleted=:isDeleted and workOrder.isValid=:isValid and orderAssembly.workOrderNo = workOrder.workOrderNo ");
        commonHql.append(" left join MesProductionAssembly assembly on assembly.organizeCode=:organizeCode and assembly.isDeleted=:isDeleted and assembly.isValid=:isValid and workOrder.custSn = assembly.custSn and orderAssembly.assemblyPartNo = assembly.assemblyPartNo ");
        commonHql.append(" left join MesProdRuleSortCfg sortCfg on sortCfg.organizeCode = :organizeCode and sortCfg.isDeleted=:isDeleted and sortCfg.isValid=:isValid and orderAssembly.pid = sortCfg.id ");
        commonHql.append(" left join MesPartType partType on partType.organizeCode = :organizeCode and partType.isDeleted=:isDeleted and partType.isValid=:isValid and sortCfg.partTypeCode = partType.partTypeCode ");
        commonHql.append(" where orderAssembly.organizeCode=:organizeCode and orderAssembly.isDeleted=:isDeleted and orderAssembly.isValid=:isValid and workOrder.organizeCode = :organizeCode");
        commonHql.append(" and workOrder.isDeleted=:isDeleted and workOrder.isValid=:isValid and assembly.organizeCode = :organizeCode and assembly.isDeleted=:isDeleted and assembly.isValid=:isValid");
        commonHql.append(" and sortCfg.organizeCode = :organizeCode and sortCfg.isDeleted=:isDeleted and sortCfg.isValid=:isValid and partType.organizeCode = :organizeCode and partType.isDeleted=:isDeleted and partType.isValid=:isValid ");
        commonHql.append(" and partType.assemblyDescription = '通用追溯' and workOrder.workOrderStatus in (:workOrderStatus) ");
        commonHql.append(" and assembly.assemblySn != '' and assembly.assemblySn is not null ");
        if (!StringUtils.isEmpty(model.getWorkCenterCode())) commonHql.append(" and workOrder.workCenterCode like '%").append(model.getWorkCenterCode()).append("%' ");
        if (!StringUtils.isEmpty(model.getCustSn())) commonHql.append(" and workOrder.custSn like '%").append(model.getCustSn()).append("%' ");
        if (!StringUtils.isEmpty(model.getAssemblySn())) commonHql.append(" and assembly.assemblySn like '%").append(model.getAssemblySn()).append("%' ");
        if (!StringUtils.isEmpty(model.getWorkOrderNo())) commonHql.append(" and workOrder.workOrderNo like '%").append(model.getWorkOrderNo()).append("%' ");
        if (!StringUtils.isEmpty(model.getShippingTimeStart())) commonHql.append(" and workOrder.shippingTime >= :shippingTimeStart ");
        if (!StringUtils.isEmpty(model.getShippingTimeEnd())) commonHql.append(" and workOrder.shippingTime <= :shippingTimeEnd ");
        return commonHql.toString();
    }

    private void setGeneralShippingParameter(MesGeneralShippingReportModel model, Query query, List<String> shippingGroupList) {
        query.setParameter("isValid", CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
        query.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        query.setParameter("organizeCode", model.getOrganizeCode());
        if(!CollectionUtils.isEmpty(shippingGroupList)) query.setParameter("shippingGroupList", shippingGroupList);
        if (!StringUtils.isEmpty(model.getShippingTimeStart())) query.setParameter("shippingTimeStart", model.getShippingTimeStart());
        if (!StringUtils.isEmpty(model.getShippingTimeEnd())) query.setParameter("shippingTimeEnd", model.getShippingTimeEnd());
        if (!StringUtils.isEmpty(model.getArrivalTimeStart())) query.setParameter("arrivalTimeStart", model.getArrivalTimeStart());
        if (!StringUtils.isEmpty(model.getArrivalTimeEnd())) query.setParameter("arrivalTimeEnd", model.getArrivalTimeEnd());
        if (!StringUtils.isEmpty(model.getReceivingTimeStart())) query.setParameter("receivingTimeStart", model.getReceivingTimeStart());
        if (!StringUtils.isEmpty(model.getReceivingTimeEnd())) query.setParameter("receivingTimeEnd", model.getReceivingTimeEnd());
    }

    private String getGeneralShippingOrderBy(MesGeneralShippingReportModel model) {
        StringBuilder orderBy = new StringBuilder();
        if (StringUtils.isEmpty(model.getOrderByParam()) || StringUtils.isEmpty(model.getAscOrDesc())) {
            model.setOrderByParam("createDatetime");
            model.setAscOrDesc(CommonEnumUtil.ASC_OR_DESC.DESC.getValue());
        }
        return orderBy.append(" order by s.").append(model.getOrderByParam()).append(" ").append(CommonEnumUtil.ASC_OR_DESC.DESC.getValue() == model.getAscOrDesc() ? "desc" : "asc").toString();
    }

    private String getGeneralShippingCommonHql(MesGeneralShippingReportModel model, List<String> shippingGroupList) {
        StringBuilder commonHql = new StringBuilder();
        commonHql.append(" from MesShippingOrderManagement s left join MesShippingOrderManagementDetail sd on s.id = sd.pid and sd.organizeCode = :organizeCode and sd.isDeleted=:isDeleted and sd.isValid=:isValid ");
        commonHql.append(" left join MesWorkOrder wo on sd.logicSeq = wo.workOrderSeq and wo.organizeCode = :organizeCode and wo.isDeleted=:isDeleted and wo.isValid=:isValid ");
        commonHql.append(" left join MesCimGmGepics cgg on cgg.csnNo = wo.custOrderNo and cgg.organizeCode = :organizeCode and cgg.isDeleted=:isDeleted and cgg.isValid=:isValid and cgg.barcode !='' and cgg.barcode is not null ");
        commonHql.append(" left join MesSupplyclientRunsheet sr on s.shippingCode = sr.runSheetNo and sr.organizeCode = :organizeCode and sr.isDeleted=:isDeleted and sr.isValid=:isValid ");
        commonHql.append(" where s.organizeCode = :organizeCode and s.isDeleted=:isDeleted and s.isValid=:isValid and sd.organizeCode = :organizeCode and sd.isDeleted=:isDeleted and sd.isValid=:isValid and wo.organizeCode = :organizeCode and wo.isDeleted=:isDeleted and wo.isValid=:isValid");
        if(!CollectionUtils.isEmpty(shippingGroupList))  commonHql.append(" and s.shippingGroupCode in (:shippingGroupList)");
        if (!StringUtils.isEmpty(model.getShippingCode())) commonHql.append(" and s.shippingCode like '%").append(model.getShippingCode()).append("%' ");
        if (!StringUtils.isEmpty(model.getWorkCenterCode())) commonHql.append(" and wo.workCenterCode like '%").append(model.getWorkCenterCode()).append("%' ");
        if (!StringUtils.isEmpty(model.getSeq())) commonHql.append(" and s.seq like '%").append(model.getSeq()).append("%' ");
        if (!StringUtils.isEmpty(model.getCustOrderCode())) commonHql.append(" and wo.custOrderNo like '%").append(model.getCustOrderCode()).append("%' ");
        if (!StringUtils.isEmpty(model.getPartNo())) commonHql.append(" and sd.partNo like '%").append(model.getPartNo()).append("%' ");
        if (!StringUtils.isEmpty(model.getCustPartNo())) commonHql.append(" and wo.custPartNo like '%").append(model.getCustPartNo()).append("%' ");
        if (!StringUtils.isEmpty(model.getVehicleNo())) commonHql.append(" and wo.carModelCode like '%").append(model.getVehicleNo()).append("%' ");
        if (!StringUtils.isEmpty(model.getUpperColour())) commonHql.append(" and cgg.upperColour like '%").append(model.getUpperColour()).append("%' ");
        if (!StringUtils.isEmpty(model.getShippingTimeStart())) commonHql.append(" and s.endScanTime >= :shippingTimeStart ");
        if (!StringUtils.isEmpty(model.getShippingTimeEnd())) commonHql.append(" and s.endScanTime <= :shippingTimeEnd ");
        if (!StringUtils.isEmpty(model.getStartCarNo())) commonHql.append(" and sr.startCarNo like '%").append(model.getStartCarNo()).append("%' ");
        if (!StringUtils.isEmpty(model.getEndCarNo())) commonHql.append(" and sr.endCarNo like '%").append(model.getEndCarNo()).append("%' ");
        if (!StringUtils.isEmpty(model.getArrivalTimeStart())) commonHql.append(" and sr.expectedArrivalTime >= :arrivalTimeStart ");
        if (!StringUtils.isEmpty(model.getArrivalTimeEnd())) commonHql.append(" and sr.expectedArrivalTime <= :arrivalTimeEnd ");
        if (!StringUtils.isEmpty(model.getShippingTimeStart())) commonHql.append(" and s.endScanTime >= :shippingTimeStart ");
        if (!StringUtils.isEmpty(model.getShippingTimeEnd())) commonHql.append(" and s.endScanTime <= :shippingTimeEnd ");
        if (!StringUtils.isEmpty(model.getReceivingTimeStart())) commonHql.append(" and sr.createDatetime >= :receivingTimeStart ");
        if (!StringUtils.isEmpty(model.getReceivingTimeEnd())) commonHql.append(" and sr.createDatetime <= :receivingTimeEnd ");
        return commonHql.toString();
    }

    private String getMesProductionRecordCommonHql(MesProductionRecordModel model) {
        StringBuilder commonHql = new StringBuilder();
        commonHql.append(" from MesProductionRecord record left join MesWorkOrder wo on wo.workOrderNo = record.workOrderNo ");
        commonHql.append(" where record.organizeCode = :organizeCode and record.isDeleted=:isDeleted and record.isValid=:isValid ");
        if (!StringUtils.isEmpty(model.getProductSn())) commonHql.append(" and record.productSn like '%").append(model.getProductSn()).append("%' ");
        if (!StringUtils.isEmpty(model.getWorkOrderNo())) commonHql.append(" and record.workOrderNo like '%").append(model.getWorkOrderNo()).append("%' ");
        if (!StringUtils.isEmpty(model.getWorkCenterCode())) commonHql.append(" and record.workCenterCode like '%").append(model.getWorkCenterCode()).append("%' ");
        if (!StringUtils.isEmpty(model.getWorkCellCode())) commonHql.append(" and record.workCellCode like '%").append(model.getWorkCellCode()).append("%' ");
        if (!StringUtils.isEmpty(model.getCompleteDateTimeStart())) commonHql.append(" and record.completeDateTime >= :completeDateTimeStart ");
        if (!StringUtils.isEmpty(model.getCompleteDateTimeEnd())) commonHql.append(" and record.completeDateTime <= :completeDateTimeEnd ");
        if (!StringUtils.isEmpty(model.getEquipmentCode())) commonHql.append(" and record.equipmentCode like '%").append(model.getEquipmentCode()).append("%' ");
        if (!StringUtils.isEmpty(model.getEquipmentName())) commonHql.append(" and record.equipmentName like '%").append(model.getEquipmentName()).append("%' ");
        if (!StringUtils.isEmpty(model.getMouldNo())) commonHql.append(" and record.mouldNo like '%").append(model.getMouldNo()).append("%' ");
        if (!StringUtils.isEmpty(model.getShiftGroup())) commonHql.append(" and record.shiftGroup like '%").append(model.getShiftGroup()).append("%' ");
        if (!StringUtils.isEmpty(model.getShiftCode())) commonHql.append(" and record.shiftCode like '%").append(model.getShiftCode()).append("%' ");
        if (!StringUtils.isEmpty(model.getReportStatus()) && model.getReportStatus() > MesExtConstWords.ZERO) commonHql.append(" and record.reportStatus = :reportStatus ");
        if (!StringUtils.isEmpty(model.getReportType()) && model.getReportType() > MesExtConstWords.ZERO) commonHql.append(" and record.reportType = :reportType ");
        if (!StringUtils.isEmpty(model.getIsComplete()) && model.getIsComplete() > MesExtConstWords.ZERO) commonHql.append(" and record.isComplete = :isComplete ");
        if (!StringUtils.isEmpty(model.getWorkOrderStatus()) && model.getWorkOrderStatus() > MesExtConstWords.ZERO) commonHql.append(" and wo.workOrderStatus = :workOrderStatus ");
        if (!StringUtils.isEmpty(model.getOrderFlag())) commonHql.append(" and wo.orderFlag = :orderFlag ");
        if (!StringUtils.isEmpty(model.getIsTest())) commonHql.append(" and record.isTest like '%").append(model.getIsTest()).append("%' ");
        if (!StringUtils.isEmpty(model.getPartNo())) commonHql.append(" and record.partNo like '%").append(model.getPartNo()).append("%' ");
        if (!StringUtils.isEmpty(model.getPartName())) commonHql.append(" and record.partName like '%").append(model.getPartName()).append("%' ");

        return commonHql.toString();
    }

    private void setMesProductionRecordParameter(MesProductionRecordModel model, Query query) {
        query.setParameter("isValid", CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
        query.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        query.setParameter("organizeCode", model.getOrganizeCode());
        if(!StringUtils.isEmpty(model.getCompleteDateTimeStart())) query.setParameter("completeDateTimeStart", model.getCompleteDateTimeStart());
        if (!StringUtils.isEmpty(model.getCompleteDateTimeEnd())) query.setParameter("completeDateTimeEnd", model.getCompleteDateTimeEnd());
        if (!StringUtils.isEmpty(model.getReportStatus()) && model.getReportStatus() > MesExtConstWords.ZERO) query.setParameter("reportStatus", model.getReportStatus());
        if (!StringUtils.isEmpty(model.getOrderFlag()))  query.setParameter("orderFlag", model.getOrderFlag());
        if (!StringUtils.isEmpty(model.getReportType()) && model.getReportType() > MesExtConstWords.ZERO) query.setParameter("reportType", model.getReportType());
        if (!StringUtils.isEmpty(model.getIsComplete()) && model.getIsComplete() > MesExtConstWords.ZERO) query.setParameter("isComplete", model.getIsComplete());
        if (!StringUtils.isEmpty(model.getWorkOrderStatus()) && model.getWorkOrderStatus() > MesExtConstWords.ZERO) query.setParameter("workOrderStatus", model.getWorkOrderStatus());
    }

    private String getMesProductionRecordOrderBy(MesProductionRecordModel model) {
        StringBuilder orderBy = new StringBuilder();
        if (StringUtils.isEmpty(model.getOrderByParam()) || StringUtils.isEmpty(model.getAscOrDesc())) {
            model.setOrderByParam("createDatetime");
            model.setAscOrDesc(CommonEnumUtil.ASC_OR_DESC.DESC.getValue());
        }
        return orderBy.append(" order by record.").append(model.getOrderByParam()).append(" ").append(CommonEnumUtil.ASC_OR_DESC.DESC.getValue() == model.getAscOrDesc() ? "desc" : "asc").toString();
    }

}
