package cn.estsh.i3plus.ext.mes.apiservice.daoimpl.report;

import cn.estsh.i3plus.ext.mes.apiservice.dao.report.IProductInspectionSubReportDao;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesQmsSuspicious;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.List;

/**
 * @Description : 检验产品数据子报表
 * @Reference :
 * @Author : PZJ
 * @CreateDate : 2024/9/29 13:51
 * @Modify:
 **/
@Service
public class ProductInspectionSubReportDaoImpl implements IProductInspectionSubReportDao {

    @Autowired
    private EntityManager entityManager;

    @Override
    public ListPager<MesQmsSuspicious> queryReport(MesQmsSuspicious model, Pager pager) {
        StringBuilder dataQueryHql = new StringBuilder();
        StringBuilder countQueryHql= new StringBuilder();
        StringBuilder commonQueryHql = new StringBuilder();
        dataQueryHql.append("select new ")
                .append(MesQmsSuspicious.class.getName()).append(" ( ");
        dataQueryHql.append("mes_qms_suspicious.inspectionDate AS inspectionDate,");
        dataQueryHql.append("mes_qms_suspicious.partNo AS partNo,");
        dataQueryHql.append("mes_qms_suspicious.partName AS partName,");
        dataQueryHql.append("mes_qms_suspicious.inspectionQty AS inspectionQty,");
        dataQueryHql.append("mes_qms_suspicious.qualifiedQty AS qualifiedQty,");
        dataQueryHql.append("mes_qms_suspicious.unqualifiedQty AS unqualifiedQty,");
        dataQueryHql.append("mes_qms_suspicious.misjudgmentQty AS misjudgmentQty,");
        dataQueryHql.append("mes_qms_suspicious.scrappedQty AS scrappedQty,");
        dataQueryHql.append("mes_qms_suspicious.reworkQualified AS reworkQualified,");
        dataQueryHql.append("mes_qms_suspicious.reworkScrapped AS reworkScrapped,");
        dataQueryHql.append("mes_qms_suspicious.reworkQty AS reworkQty,");
        dataQueryHql.append("CASE WHEN inspectionQty = 0 THEN 0 ELSE (qualifiedQty / inspectionQty * 100) END AS firstPassRate,");
        dataQueryHql.append("CASE WHEN inspectionQty = 0 THEN 0 ELSE (unqualifiedQty / inspectionQty * 100) END AS defectRate,");
        dataQueryHql.append("CASE WHEN inspectionQty = 0 THEN 0 ELSE ((qualifiedQty + misjudgmentQty + reworkQualified) / inspectionQty * 100) END AS finalPassRate,");
        dataQueryHql.append("CASE WHEN inspectionQty = 0 THEN 0 ELSE ((scrappedQty + reworkScrapped) / inspectionQty * 100) END AS finalScrapRate,");
        dataQueryHql.append("mes_qms_suspicious.organizeCode AS organizeCode");
        dataQueryHql.append(") ");

        countQueryHql.append("select count(1) ");

        commonQueryHql.append("FROM MesQmsSuspicious mes_qms_suspicious ");
        commonQueryHql.append("WHERE 1=1 ");
        commonQueryHql.append("AND mes_qms_suspicious.organizeCode = :organizeCode ");
        commonQueryHql.append("AND mes_qms_suspicious.isDeleted = :isDeleted ");
        commonQueryHql.append("AND mes_qms_suspicious.isValid = :isValid ");
        if (!StringUtils.isEmpty(model.getInspectionStartDate()) && !StringUtils.isEmpty(model.getInspectionEndDate())){
            commonQueryHql.append("AND mes_qms_suspicious.createDatetime BETWEEN :inspectionStartDate AND :inspectionEndDate ");
        }
        if(!StringUtils.isEmpty(model.getPartName())){
            commonQueryHql.append("AND mes_qms_suspicious.partName like :partName ");
        }
        if(!StringUtils.isEmpty(model.getPartNo())){
            commonQueryHql.append("AND mes_qms_suspicious.partNo like :partNo ");
        }

        Query dataQuery = entityManager.createQuery(dataQueryHql.append(commonQueryHql).toString(), MesQmsSuspicious.class);
        setQueryParams(dataQuery, model);
        Query countQuery = entityManager.createQuery(countQueryHql.append(commonQueryHql).toString());
        setQueryParams(countQuery, model);

        int count = Integer.parseInt(String.valueOf(countQuery.getSingleResult()));
        pager = PagerHelper.getPager(pager, count);
        dataQuery.setMaxResults(pager.getPageSize());
        dataQuery.setFirstResult(pager.getCurrentPage() * pager.getPageSize() - pager.getPageSize());
        List<MesQmsSuspicious> resultList = dataQuery.getResultList();
        return new ListPager<>(resultList, pager);
    }

    private void setQueryParams(Query dataQuery, MesQmsSuspicious model) {
        dataQuery.setParameter("organizeCode", model.getOrganizeCode());
        dataQuery.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        dataQuery.setParameter("isValid", CommonEnumUtil.IS_VAILD.VAILD.getValue());
        if (!StringUtils.isEmpty(model.getInspectionStartDate()) && !StringUtils.isEmpty(model.getInspectionEndDate())){
            dataQuery.setParameter("inspectionStartDate", model.getInspectionStartDate());
            dataQuery.setParameter("inspectionEndDate", model.getInspectionEndDate());
        }
        if(!StringUtils.isEmpty(model.getPartName())){
            dataQuery.setParameter("partName", model.getPartName() + "%");
        }
        if(!StringUtils.isEmpty(model.getPartNo())){
            dataQuery.setParameter("partNo", model.getPartNo() + "%");
        }
    }
}
