package cn.estsh.i3plus.ext.mes.apiservice.dbinterface;

import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.mes.apiservice.serviceimpl.engine.script.EngineScriptManager;
import cn.estsh.i3plus.mes.apiservice.util.datatable.DataRow;
import cn.estsh.i3plus.mes.apiservice.util.datatable.DataSet;
import cn.estsh.i3plus.mes.apiservice.util.datatable.DataTable;
import cn.estsh.i3plus.platform.plugin.datasource.DynamicDataSourceProxy;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.wms.dbinterface.UpdateSyncItem;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description : 业务数据写入
 * 按照配置的规则把业务数据写入业务库
 * @Reference :
 * @Author : rock.yu
 * @CreateDate : 2019-06-11 19:34
 * @Modify: 1. 2019-10-14 19:13 增加了调用脚本引擎处理数据的逻辑
 **/
@Component
@NoArgsConstructor
public class WmsSAPDbWriter extends WmsSAPDbSyncBase {

    public static final Logger LOGGER = LoggerFactory.getLogger(WmsSAPDbWriter.class);

    public static final int REQ_COUNT = 20;

    @Getter
    private DynamicDataSourceProxy srcDataSourceProxy;
    @Getter
    private DynamicDataSourceProxy destDataSourceProxy;

    // 脚本引擎
    @Autowired
    private EngineScriptManager engineScriptManager;

    @Resource(name = "redisMes")
    private ImppRedis redisMes;

    @Getter
    @Setter
    private WmsSAPDbExpression wmsSAPDbExpression;

    // 数据库读写用的数据源
    @Getter
    @Setter
    private Connection srcConn;
    @Getter
    @Setter
    private Connection destConn;

    public WmsSAPDbWriter(DynamicDataSourceProxy srcDataSourceProxy, DynamicDataSourceProxy destDataSourceProxy) {
        this.srcDataSourceProxy = srcDataSourceProxy;
        this.destDataSourceProxy = destDataSourceProxy;
    }

    /**
     * 保存来源和目标数据
     * 如果 originDataList 不为空，则表示来源数据集被脚本过滤过，需要使用原始查询到的数据集回写来源表的更新标识
     */
    public void saveData(List<Map<String, Object>> srcDataList, DataSet destDataSet, List<Map<String, Object>> originDataList) throws Exception {
        // 先保存目标数据
        // 配置了使用脚本引擎保存数据
        try {
            if (this.dataMapper.getUseScriptPush() == CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()) {
                callScript(destDataSet, WmsSAPDbConstWords.SCRIPT_PUSH_FUN_NAME);
            } else {
                saveDestData(destDataSet);
            }
        } catch (Exception e) {
            LOGGER.info(ExceptionUtils.getStackTrace(e));
            //同步失败，更新同步状态为同步失败
            if (this.dataMapper.getUseScriptMark() == CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()) {
                callScriptMark(srcDataList, originDataList);
            } else {
                callSqlMark(srcDataList, originDataList, false);
            }
            return;
        }

        // 再更新来源数据的同步标志
        // 配置了使用脚本引擎标记来源数据
        if (this.dataMapper.getUseScriptMark() == CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()) {
            callScriptMark(srcDataList, originDataList);
        } else {
            callSqlMark(srcDataList, originDataList,true);
        }
    }

    /**
     * 执行SQL回写来源表的更新标记
     * @param srcDataList 来源数据集
     * @param originDataList 原始查询到的数据集
     * @throws Exception
     */
    private void callSqlMark(List<Map<String, Object>> srcDataList, List<Map<String, Object>> originDataList,boolean isSuccess)
            throws Exception {
        if (originDataList != null) {
            saveSrcData(originDataList,isSuccess);
        } else {
            saveSrcData(srcDataList,isSuccess);
        }
    }

    /**
     * 执行脚本回写来源表的更新标记
     * @param srcDataList 来源数据集
     * @param originDataList 原始查询到的数据集
     * @throws Exception
     */
    private void callScriptMark(List<Map<String, Object>> srcDataList, List<Map<String, Object>> originDataList)
            throws Exception {
        if (originDataList != null) {
            callScript(originDataList, WmsSAPDbConstWords.SCRIPT_MARK_FUN_NAME);
        } else {
            callScript(srcDataList, WmsSAPDbConstWords.SCRIPT_MARK_FUN_NAME);
        }
    }

    /**
     * 调用脚本引擎处理数据
     * @param data 数据集
     * @param funName 调用脚本中的方法名称
     * @throws Exception 脚本执行异常
     */
    private void callScript(Object data, String funName) throws Exception {
        // 调用脚本引擎写入目标数据
        String scriptNo = this.dataMapper.getScriptNo();

        // 如果脚本编号配置的是 groovy 文件，就直接调用文件，便于运行中打断点调试
        if (scriptNo.indexOf(".groovy") > 0) {
            engineScriptManager.invokeGroovyScript(
                    scriptNo,
                    funName,
                    this.dataMapper,
                    data);
        } else {
            engineScriptManager.invokeFuncation(
                    this.dataMapper.getOrganizeCode(),
                    scriptNo,
                    funName,
                    this.dataMapper,
                    data);
        }
    }

    /**
     * 保存目标数据
     *
     * @param destDataSet
     * @throws Exception
     */
    public void saveDestData(DataSet destDataSet) throws Exception {
        if (destDataSet == null || destDataSet.getDataTables().size() == 0) {
            LOGGER.error("no data in  dest dataset!");
            throw new IllegalArgumentException("no data in  dest dataset!");
        }

        List<String> sqlStringList = new ArrayList<>();
        // 保存主键查询的 SQL 语句，用于判断表内数据行是否重复
        Map<String, Integer> sqlCacheMap = new HashMap<>();

        // 循环处理每一行目标表数据
        for (DataTable table : destDataSet.getDataTables()) {
            for (DataRow destRow : table.getRows()) {
                // 生成 SQL 语句
                sqlStringList.add(packRowSQL(destRow, sqlCacheMap));
            }
        }

        try {
            // 作为一个事务批量执行
            destDataSourceProxy.executeAsBatch(sqlStringList, destConn);
            for(String sql : sqlStringList) {
                LOGGER.info(sql);
            }
//            LOGGER.info(sqlStringList.toString());
        } catch (Exception e) {
            // 包装异常信息，把 SQL 加进去，便于调试
            throw new SQLException("执行SQL异常" + e.getMessage() + " "
                    + getSqlStrings(sqlStringList));
        }
    }

    /**
     * 包装 SQL 语句，供测试参考
     *
     * @param sqlStringList
     * @return
     */
    private String getSqlStrings(List<String> sqlStringList) {
        if (sqlStringList == null) {
            return null;
        }

        StringBuffer sb = new StringBuffer();
        int index = 0;

        for (String sql : sqlStringList) {
            sb.append(String.valueOf(index) + " < ");
            sb.append(sql);
            sb.append(">  ");
        }

        return sb.toString();
    }

    /**
     * 更新来源数据
     *
     * @param srcDataList
     * @throws Exception
     */
    private void saveSrcData(List<Map<String, Object>> srcDataList,boolean isSuccess) throws Exception {
        if (srcDataList == null || srcDataList.size() == 0) {
            LOGGER.error("no data in src datalist!");
            throw new IllegalArgumentException("no data in src datalist!");
        }

        List<String> sqlStringList = new ArrayList<>();
        // 存放插入备份表的SQL语句列表
        List<String> insertSqlStringList = new ArrayList<>();
        // 存放删除来源表的SQL语句列表
        List<String> deleteSqlStringList = new ArrayList<>();

        List<String> saveAndRemoveList = new ArrayList<>();

        String sqlString;

        HashMap<String, Object> srcPkMap = new HashMap<>();

        // 循环处理每一行数据
        for (Map<String, Object> srcRow : srcDataList) {
            // 先清空主键 Map
            srcPkMap.clear();
            if (StringUtil.isEmpty(this.getSrcPkColumn())) {
                LOGGER.error("WmsSAPDbWriter.saveSrcData params srcPkColumn is null!");
                throw new Exception("WmsSAPDbWriter.saveSrcData params srcPkColumn is null");
            }
            // 构建主键 Map
            for (String column : this.getSrcPkColumn()) {
                srcPkMap.put(column, srcRow.get(column));
            }

            // 把来源表的数据行放到表达式中，确保表达式可以引用
            HashMap<String, Object> updateValueMap = getUpdateSyncMap(srcRow,isSuccess);
            // 存在来源表不需要更新同步标志的情况
            if (updateValueMap.size() > 0) {
                // 拼接更新来源表同步标志的 SQL LIST
                sqlString = srcDataSourceProxy.packUpdateSQL(this.dataMapper.getSrcTableName(), srcPkMap,
                        updateValueMap);
                // 如果设置了分组处理，在回写来源表时就要加上原始的 Where 条件，以防止多更新数据
                if (StringUtils.isNotBlank(this.dataMapper.getSrcGroupColumns())) {
                    // 先去掉原来拼接的分号
                    sqlString = sqlString.replace(";"," ") + " and (" + this.dataMapper.getSrcWhere() + "); ";
                }

                sqlStringList.add(sqlString);
            }

            // 添加处理完成的数据及时剪切到备份表的逻辑
            if (!StringUtils.isBlank(this.dataMapper.getCutToTableName())) {
                // 插入历史表的语句
                sqlString = srcDataSourceProxy.packInsertSQL(this.dataMapper.getCutToTableName(), (HashMap<String, Object>)srcRow);
                insertSqlStringList.add(sqlString);

                // 删除来源表的语句
                sqlString = srcDataSourceProxy.packDeleteSQL(this.dataMapper.getSrcTableName(), srcPkMap);
//                // 如果设置了分组处理，在回写来源表时就要加上原始的 Where 条件，以防止多更新数据
//                if (StringUtils.isNotBlank(this.dataMapper.getSrcGroupColumns())) {
//                    // 先去掉原来拼接的分号
//                    sqlString = sqlString.replace(";"," ") + " and (" + this.dataMapper.getSrcWhere() + "); ";
//                }
                deleteSqlStringList.add(sqlString);
            }
        }

        try {
            if (sqlStringList.size() > 0) {
                // 去掉重复的 SQL
                List<String> uniqeSqlList = sqlStringList.stream().distinct().collect(Collectors.toList());
                // 批量更新来源表的同步标志
                srcDataSourceProxy.executeAsBatch(uniqeSqlList, srcConn);
            }
        } catch (Exception e) {
            // 包装异常信息，把 SQL 加进去，便于调试
            throw new SQLException("执行SQL异常" + e.getMessage() + " "
                    + getSqlStrings(sqlStringList));
        }
        try {
            if (insertSqlStringList.size() > 0) {
                saveAndRemoveList.addAll(insertSqlStringList);
            }
            if (deleteSqlStringList.size() > 0) {
                saveAndRemoveList.addAll(deleteSqlStringList);
            }
            if (saveAndRemoveList.size() > 0) {
                // 去掉重复的 SQL
                LOGGER.info("执行迁移的sql脚本为:sqlString:"+getSqlStrings(saveAndRemoveList));
                List<String> uniqeSqlList = saveAndRemoveList.stream().distinct().collect(Collectors.toList());
                //插入备份表 & 删除来源表数据
                srcDataSourceProxy.executeAsBatch(uniqeSqlList, srcConn);
            }
        } catch (Exception e) {
            // 包装异常信息，把 SQL 加进去，便于调试
            throw new SQLException("执行SQL异常" + e.getMessage() + " "
                    + getSqlStrings(saveAndRemoveList));
        }
    }

    /**
     * 根据表达式获来源表取待更新的值Map
     *
     * @return
     */
    private HashMap<String, Object> getUpdateSyncMap(Map<String, Object> srcRow,boolean isSuccess) throws Exception {
        HashMap<String, Object> updateSyncMap = new HashMap<>();

        if (this.getSrcUpdateSync() == null) {
            return updateSyncMap;
        }

        boolean containsColon = false;
        for (UpdateSyncItem item : this.getSrcUpdateSync()) {
            Object updateValue = item.getUpdateValue();
            if(!StringUtils.isEmpty(item.getUpdateValue()) && item.getUpdateValue().contains(MesExtConstWords.COLON)){
                containsColon = true;
                updateValue = isSuccess ? item.getUpdateValue().split(MesExtConstWords.COLON)[0]:item.getUpdateValue().split(MesExtConstWords.COLON)[1];
            }

            // 如果包含表达式，则计算后赋值
            if (item.getUpdateValue().contains("#")) {
                updateValue = wmsSAPDbExpression.parse(item.getUpdateValue(), srcRow);
            }

            updateSyncMap.put(item.getColumnName(), updateValue);
        }
        //如果未配置失败的标识报错
        if(!isSuccess && !containsColon){
            LOGGER.error("WmsSAPDbWriter.saveSrcData 失败标识未配置");
            throw new Exception("WmsSAPDbWriter.saveSrcData 失败标识未配置");
        }

        return updateSyncMap;
    }


    /**
     * 生成保存一行数据到目标表的SQL
     *
     * @param row
     * @throws Exception
     */
    private String packRowSQL(DataRow row, Map<String, Integer> sqlCacheMap) throws Exception {
        if (row == null || row.getColumns().size() == 0) {
            LOGGER.error("no column in row!");
            throw new IllegalArgumentException("no column in row!");
        }

        // 获取主键列表
        List<String> destPks = getPkList(row);

        // 获取主键查询 SQL & KeyMap
        SqlAndKeyMap sqlAndKeyMap = new SqlAndKeyMap(row, destPks).invoke();
        String sqlString = sqlAndKeyMap.getSqlString();
        Map<String, Object> pkMap = sqlAndKeyMap.getPkMap();

        // 按照主键从业务表查询数据
        List<Map<String, Object>> resutlMap = destDataSourceProxy.queryMapList(sqlString, destConn);

        // 判断按主键查询的 SQL 有没有出现过
        Boolean dataDupInBatch = sqlCacheMap.containsKey(sqlString);

        // 没出现过，加入缓存中
        if (!dataDupInBatch) {
            sqlCacheMap.put(sqlString, 1);
        }

        // 最终要生成的插入或更新SQL
        String sqlResult;

        // 目标数据存在或者在本次处理的数据中重复则更新，不存在则新增
        if ((resutlMap != null && resutlMap.size() > 0) || dataDupInBatch) {
            HashMap<String, Object> rowMap = (HashMap<String, Object>) row.getItemMap();

            // id 不用更新
            if (rowMap.containsKey("id")) {
                rowMap.remove("id");
            }

            // 过滤标记为 NotTrans 数据项，这些数据不更新到数据库
            Iterator<Map.Entry<String, Object>> iterator = rowMap.entrySet().iterator();
            Object value;
            while (iterator.hasNext()) {
                value = iterator.next().getValue();
                if (value != null && value.toString().endsWith(WmsSAPDbExpression.NOT_TRANS)) {
                    iterator.remove();
                }
            }

            sqlResult = destDataSourceProxy.packUpdateSQL(row.getTable().getTableName(), (HashMap<String, Object>) pkMap,rowMap);
        } else {
            HashMap<String, Object> rowMap = (HashMap<String, Object>) row.getItemMap();
            // 带 NotTrans 的数据先把 :NotTrans 或者 NotTrans 标记去掉
            for (Map.Entry<String, Object> entry : rowMap.entrySet()) {
                if (entry.getValue() != null && entry.getValue().toString().endsWith(WmsSAPDbExpression.NOT_TRANS)) {
                    String transValue = entry.getValue().toString();
                    transValue = transValue.replace(":" + WmsSAPDbExpression.NOT_TRANS, "");
                    transValue = transValue.replace(WmsSAPDbExpression.NOT_TRANS, "");
                    LOGGER.debug("NotTrans Item: {} Insert before: {} after: {}", entry.getKey(), entry.getValue(), transValue);
                    rowMap.put(entry.getKey(), transValue);
                }
            }

            sqlResult = destDataSourceProxy.packInsertSQL(row.getTable().getTableName(),
                    (HashMap<String, Object>) row.getItemMap());
        }

        return sqlResult;
    }

    /**
     * 获取主键列表
     *
     * @param row
     * @return
     */
    private List<String> getPkList(DataRow row) {
        // 先查找外面配置的主键
        List<String> destPks = this.getDestPkProperty();

        // 一表对多表，主键设置在字段对照表里
        if (destPks == null || destPks.isEmpty()) {
            destPks = this.getDestPkByBeanName(row.getTable().getTableName());
        }

        // 没有找到主键时抛出异常
        if (destPks == null || destPks.isEmpty()) {
            LOGGER.error("no pk in table: " + row.getTable().getTableName());
            throw new IllegalArgumentException("no pk in table: " + row.getTable().getTableName());
        }

        return destPks;
    }


    /**
     * 内部类，用于拼接主键查询 SQL & KeyMap
     */
    private class SqlAndKeyMap {
        private DataRow row;
        private List<String> destPks;
        private String sqlString;
        private Map<String, Object> pkMap;

        public SqlAndKeyMap(DataRow row, List<String> destPks) {
            this.row = row;
            this.destPks = destPks;
        }

        public String getSqlString() {
            return sqlString;
        }

        public Map<String, Object> getPkMap() {
            return pkMap;
        }

        public SqlAndKeyMap invoke() {
            sqlString = "select 1 from " + row.getTable().getTableName()
                    + " where 1=1 ";

            // 目标表的主键列表
            pkMap = new HashMap<>();

            for (String column : destPks) {
                String value = WmsSAPDbWriter.this.getSafetyString(row.getValue(column));
                // 处理掉 NotTrans 字段
                if (value != null && value.contains(WmsSAPDbExpression.NOT_TRANS)) {
                    value = value.replace(":" + WmsSAPDbExpression.NOT_TRANS, "");
                    value = value.replace(WmsSAPDbExpression.NOT_TRANS, "");
                }
                sqlString += "and " + column + "='" + value + "' ";
                pkMap.put(column, row.getValue(column));
            }

            return this;
        }
    }

    /**
     * 重写判断对象相等的方法
     *
     * @param obj
     * @return
     */
    @Override
    public boolean equals(Object obj) {
        return super.equals(obj);
    }

    /**
     * 重写 hashCode 方法
     *
     * @return
     */
    @Override
    public int hashCode() {
        return super.hashCode();
    }
}
