package cn.estsh.i3plus.ext.mes.apiservice.schedulejob;

import cn.estsh.i3plus.ext.mes.api.base.IMesPullingOrderInfoService;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.mes.apiservice.schedulejob.BaseMesScheduleJob;
import cn.estsh.i3plus.platform.common.tool.JsonUtilTool;
import cn.estsh.i3plus.pojo.model.wms.WmsJobParamModel;
import cn.estsh.impp.framework.boot.init.ApplicationProperties;
import io.swagger.annotations.ApiOperation;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.JobExecutionContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * @Description : 根据拉动组和生产工单产生拉动单JOB
 * @Reference :
 * @Author : gsz
 * @CreateDate 2024/9/4 10:01
 * @Modify:
 **/
// 禁止 JOB 并发执行
@DisallowConcurrentExecution
@ApiOperation("根据拉动组和生产工单产生拉动单JOB")
public class MesCreatePullOrderByPullGroupAndWorkOrderJob extends BaseMesScheduleJob {

    public static final Logger LOGGER = LoggerFactory.getLogger(MesCreatePullOrderByPullGroupAndWorkOrderJob.class);

    private static final long serialVersionUID = -321526601595924818L;

    @Autowired
    private IMesPullingOrderInfoService pullingOrderInfoService;

    @Value("${server.port}")
    private String serverPort;
    @Value("${impp.server.ip}")
    private String ip;

    public MesCreatePullOrderByPullGroupAndWorkOrderJob() {
        super(MesCreatePullOrderByPullGroupAndWorkOrderJob.class, "根据拉动组和生产工单产生拉动单JOB");
        this.setMultiInstance(true);
    }

    @Override
    public void executeMesJob(JobExecutionContext context, ApplicationProperties applicationProperties) {

        List<WmsJobParamModel> wmsJobParamModelList = JsonUtilTool.toList(this.getJobParam(), WmsJobParamModel.class);
        if (!CollectionUtils.isEmpty(wmsJobParamModelList)) {
            for (WmsJobParamModel wmsJobParamModel : wmsJobParamModelList) {
                try {
                    long startTime = System.currentTimeMillis();
                    LOGGER.info("工厂{}根据拉动组和生产工单产生拉动单JOB --- start", wmsJobParamModel.getOrganizeCode());
                    pullingOrderInfoService.doCreatePullOrderByPullGroupAndWorkOrder(wmsJobParamModel.getOrganizeCode(), wmsJobParamModel.getPageSize(), MesExtConstWords.JOB);
                    long endTime = System.currentTimeMillis();
                    LOGGER.info("工厂{}根据拉动组和生产工单产生拉动单JOB --- end --- 耗时: {} ms", wmsJobParamModel.getOrganizeCode(), endTime - startTime);
                } catch (Exception e) {
                    LOGGER.info("工厂{}根据拉动组和生产工单产生拉动单JOB 执行失败{}", wmsJobParamModel.getOrganizeCode(), e);
                }
            }
        }
    }

}
