package cn.estsh.i3plus.ext.mes.apiservice.schedulejob;


import cn.estsh.i3plus.ext.mes.api.base.IMesProductionRecordService;
import cn.estsh.i3plus.mes.apiservice.schedulejob.BaseMesScheduleJob;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.report.MesProductAccountModel;
import cn.estsh.i3plus.pojo.mes.model.report.MesProductAccountResModel;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.impp.framework.boot.init.ApplicationProperties;
import cn.hutool.json.JSONObject;
import com.google.common.base.Objects;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.beanutils.BeanUtils;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.JobExecutionContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author castle, 暂时没有客供品移库
 * @version 1.0
 * @date 2021/2/2 16:44
 **/
@Slf4j
@DisallowConcurrentExecution
@Component
@ApiOperation("生成对账数据job")
public class MesGenerateAccountJob extends BaseMesScheduleJob {

    @Autowired
    private MesProductRecordLogRepository mesProductRecordLogRepository;

    @Autowired
    private MesProductionRecordRepository mesProductionRecordRepository;

    @Autowired
    private MesEquipmentLogDetailRepository mesEquipmentLogDetailRepository;

    @Autowired
    private MesEquipmentRepository mesEquipmentRepository;

    @Autowired
    private MesMouldMultiCavityRepository mesMouldMultiCavityRepository;

    @Autowired
    private IMesProductionRecordService mesProductionRecordService;

    public MesGenerateAccountJob() {
        super(MesGenerateAccountJob.class, "生成对账数据job");
    }

    @Override
    @Transactional
    public void executeMesJob(JobExecutionContext context, ApplicationProperties applicationProperties) {
        JSONObject jsonObject = new JSONObject();

        jsonObject.putIfAbsent("organizeCode", "CK01");
        jsonObject.putIfAbsent("userName", "job");

        String organizeCode = jsonObject.getStr("organizeCode");
        String userName = jsonObject.getStr("userName");
        if (null == organizeCode) {
            log.error("请添加需要对账的工厂代码!");
            return;
        }
        if (null == userName) {
            userName = "JOB";
        }
        /**
         * 760B_A58_ReMaoHan  只有finish信号
         * 760BFoaming
         * 760B_LaserCutting
         * ZhenKong
         */
        List<String> equimentList = Stream.of("760B_A58_ReMaoHan","760BFoaming","760B_LaserCutting","ZhenKong").collect(Collectors.toList());
        MesProductAccountModel productAccountModel = new MesProductAccountModel();
        productAccountModel.setStartTime("2024-09-12 08:00:00");
        productAccountModel.setEndTime("2024-09-13 08:00:00");
        productAccountModel.setEquipmentCode("HT2400APrinter_MixUA");
        productAccountModel.setPartNo("Y0556325-01");
        List<MesProductAccountResModel> mesProductAccountResModels = mesProductionRecordService.queryProductAccount(productAccountModel);
        if (!CollectionUtils.isEmpty(mesProductAccountResModels)) {
            mesProductAccountResModels = mesProductAccountResModels.stream().filter(mesProductAccountResModel -> mesProductAccountResModel.getDetailCount().compareTo(mesProductAccountResModel.getProductCount())> 0) .collect(Collectors.toList());
        }
        if (!CollectionUtils.isEmpty(mesProductAccountResModels)) {
            for (MesProductAccountResModel mesProductAccountResModel : mesProductAccountResModels) {
                DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
                DdlPreparedPack.getStringEqualPack(mesProductAccountResModel.getEquipmentCode(), "equipmentCode", packBean);

                DdlPreparedPack.getStringSmallerNotEqualPack(productAccountModel.getEndTime(), "createDatetime", packBean);
                DdlPreparedPack.getStringBiggerPack(productAccountModel.getStartTime(), "createDatetime", packBean);
                DdlPreparedPack.getStringEqualPack(mesProductAccountResModel.getPartNo(), "partNo", packBean);

                DdlPreparedPack.getOrderBy("createDatetime", CommonEnumUtil.ASC_OR_DESC.ASC.getValue(), packBean);
                List<MesProductRecordLog> recordLogs = mesProductRecordLogRepository.findByHqlWhere(packBean);
                if (mesProductAccountResModel.getEquipmentCode().contains("HT")) {
                    // 1 判断2个 shotcount之间有没有加工记录，如果没有，就需要看 设备是不是192，然后看是否是网络原因

                    String reason = handleHT(recordLogs);
                    mesProductAccountResModel.setErrorReason(reason);
                } else if (equimentList.contains(mesProductAccountResModel.getEquipmentCode())) {
                    // 1 判断2个 shotcount之间有没有加工记录，如果没有，就需要看 设备是不是192，然后看是否是网络原因
                    String reason =handleNotHT(recordLogs);
                    mesProductAccountResModel.setErrorReason(reason);
                } else {
                    String reason =handleNotHTAndHasRelease(recordLogs);
                    mesProductAccountResModel.setErrorReason(reason);
                }
            }
        }
    }

    private String handleNotHT(List<MesProductRecordLog> recordLogs) {
        int result = 0;
        String startTime = "";
        String endTime = "";
        int errorCount = 0;
        String reason = "";
        String reasonOffline = "【%s】到【%s】,客户端下线,导致【%s】个完成信号丢失";
        String reasonQuality = "【%s】到【%s】设备信号质量有问题，导致【%s】个完成信号丢失";
        String reasonBypass = "【%s】到【%s】设备信号Bypass,【%s】个完成信号丢失";
        boolean bypassFlag = false;

        List<String> finishList = Stream.of("Product_finish","PART_OK","Part_Finish","Product_OK").collect(Collectors.toList());

        for (MesProductRecordLog recordLog : recordLogs) {
            if (Objects.equal(recordLog.getCategoryLevelTwo(), "BYPASS")) {
                if (Objects.equal(recordLog.getEquipVariableValue(), "True")) {
                    if (StringUtils.isEmpty(startTime)) {
                        startTime = recordLog.getCreateDatetime();
                    }
                    endTime = recordLog.getCreateDatetime();
                    // 每次都取反，第一次是打开，那第二次就是关闭了
                    bypassFlag = true;
                    continue;
                } else {
                    if (bypassFlag) {
                        bypassFlag = false;
                        errorCount = 0;
                        continue;
                    }
                }
            }
            if (bypassFlag && recordLog.getType().equals("record"))  {
                errorCount ++;
                reason = String.format(reasonBypass,startTime, endTime, errorCount +"");
                continue;
            }


            if (result == 0) {
                if (!finishList.contains(recordLog.getCategoryLevelTwo()) || !Objects.equal(recordLog.getEquipVariableValue(), "True")) {
                    continue;
                }
                result ++;
            } else {
                if (finishList.contains(recordLog.getCategoryLevelTwo()) && Objects.equal(recordLog.getEquipVariableValue(), "True")) {
                    if (StringUtils.isEmpty(startTime)) {
                        startTime = recordLog.getCreateDatetime();
                    }
                    errorCount ++;
                    endTime = recordLog.getCreateDatetime();
                    if (Objects.equal(recordLog.getQuality(), "192")) {
                        reason += String.format(reasonOffline,startTime, endTime, errorCount +"");
                    } else {
                        reason += String.format(reasonQuality,startTime, endTime, errorCount +"");
                    }
                }
                if (recordLog.getType().equals("record")) {
                    result --;
                }
            }
        }
        return reason;
    }

    private String handleNotHTAndHasRelease(List<MesProductRecordLog> recordLogs) {
        int result = 0;
        String startTime = "";
        String endTime = "";
        int errorCount = 0;
        String reason = "";
        String reasonOffline = "【%s】到【%s】,客户端下线,导致【%s】个完成信号丢失";
        String reasonQuality = "【%s】到【%s】设备信号质量有问题，导致【%s】个完成信号丢失";
        String reasonBypass = "【%s】到【%s】设备信号Bypass,【%s】个完成信号丢失";
        String releaseBypass = "【%s】到【%s】设备信号没有个release就给了完成新号,导致【%s】个完成信号丢失";



        boolean bypassFlag = false;
        boolean releaseFlag = false;

        List<String> finishList = Stream.of("Product_finish","PART_OK","Part_Finish","Product_OK").collect(Collectors.toList());

        for (MesProductRecordLog recordLog : recordLogs) {
            if (Objects.equal(recordLog.getCategoryLevelTwo(), "BYPASS") && Objects.equal(recordLog.getEquipVariableValue(), "True")) {
                if (StringUtils.isEmpty(startTime)) {
                    startTime = recordLog.getCreateDatetime();
                }
                endTime = recordLog.getCreateDatetime();
                // 每次都取反，第一次是打开，那第二次就是关闭了
                bypassFlag = !bypassFlag;
                if (!bypassFlag) {
                    errorCount = 0;
                }
                continue;
            }
            if (bypassFlag && recordLog.getType().equals("record"))  {
                errorCount ++;
                reason = String.format(reasonBypass,startTime, endTime, errorCount +"");
                continue;
            }

            if (result == 0) {
                if (Objects.equal(recordLog.getCategoryLevelTwo(), "Work_Allow") && Objects.equal(recordLog.getEquipVariableValue(), "True")) {
                    releaseFlag = true;
                    continue;
                }
                if (!finishList.contains(recordLog.getCategoryLevelTwo()) || !Objects.equal(recordLog.getEquipVariableValue(), "True")) {
                    continue;
                }
                result ++;

            } else {
                if (finishList.contains(recordLog.getCategoryLevelTwo()) && Objects.equal(recordLog.getEquipVariableValue(), "True")) {
                    if (StringUtils.isEmpty(startTime)) {
                        startTime = recordLog.getCreateDatetime();
                    }
                    errorCount ++;
                    endTime = recordLog.getCreateDatetime();
                    if (!releaseFlag) {
                        reason = String.format(releaseBypass,startTime, endTime, errorCount +"");
                    } else {
                        if (Objects.equal(recordLog.getQuality(), "192")) {
                            reason = String.format(reasonOffline,startTime, endTime, errorCount +"");
                        } else {
                            reason = String.format(reasonQuality,startTime, endTime, errorCount +"");
                        }
                    }
                }
                if (recordLog.getType().equals("record")) {
                    result --;
                }
            }
        }
        return reason;
    }

    private String handleHT(List<MesProductRecordLog> recordLogs) {
        int result = 0;
        String startTime = "";
        String endTime = "";
        int errorCount = 0;
        String reason = "";
        String reasonOffline = "【%s】到【%s】,客户端下线,导致【%s】个完成信号丢失";
        String reasonQuality = "【%s】到【%s】设备信号质量有问题，导致【%s】个完成信号丢失";

        for (MesProductRecordLog recordLog : recordLogs) {
            if (result == 0) {
                if (!Objects.equal(recordLog.getCategoryLevelTwo(),"shot_counter") ) {
                    continue;
                }
                if (recordLog.getEquipVariableValue().compareTo("0") > 0) {
                    result ++;
                }
            } else {
              if (Objects.equal(recordLog.getCategoryLevelTwo(),"shot_counter") && recordLog.getEquipVariableValue().compareTo("0") > 0) {
                  if (StringUtils.isEmpty(startTime)) {
                      startTime = recordLog.getCreateDatetime();
                  }
                  errorCount ++;
                  endTime = recordLog.getCreateDatetime();
                  if (Objects.equal(recordLog.getQuality(), "192")) {
                      reason = String.format(reasonOffline,startTime, endTime, errorCount +"");
                  } else {
                      reason = String.format(reasonQuality,startTime, endTime, errorCount +"");
                  }
              }
              if (recordLog.getType().equals("record")) {
                    result --;
              }
            }
        }
        return reason;
    }
}

