package cn.estsh.i3plus.ext.mes.apiservice.schedulejob;

import cn.estsh.i3plus.ext.mes.api.base.IMesQmsSuspiciousService;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.mes.apiservice.schedulejob.BaseMesScheduleJob;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.nc.MesPartInspection;
import cn.estsh.i3plus.pojo.mes.repository.MesPartInspectionRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.init.ApplicationProperties;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.JobExecutionContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.CollectionUtils;

import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description : MES可疑品汇总
 * @Reference :
 * @Author : gsz
 * @CreateDate 2024/7/3 15:58
 * @Modify:
 **/
@DisallowConcurrentExecution
@ApiOperation("MES可疑品汇总")
public class MesQmsSuspiciousJob extends BaseMesScheduleJob {

    public static final Logger LOGGER = LoggerFactory.getLogger(MesQmsSuspiciousJob.class);

    private static final long serialVersionUID = 5078389675215031222L;

    @Autowired
    private MesPartInspectionRepository partInspectionRepository;

    @Autowired
    private IMesQmsSuspiciousService qmsSuspiciousService;

    @Value("${server.port}")
    private String serverPort;
    @Value("${impp.server.ip}")
    private String ip;

    public MesQmsSuspiciousJob() {
        super(MesQmsSuspiciousJob.class, "MES可疑品汇总");
        this.setMultiInstance(true);
    }

    @Override
    public void executeMesJob(JobExecutionContext context, ApplicationProperties applicationProperties) {
        try {
            String jobParam = this.getJobParam();

            if (StringUtils.isBlank(jobParam)) {
                throw new IllegalArgumentException("MES可疑品汇总job参数为空，请检查参数");
            }
            String[] organizeCodeList = jobParam.split(",");
            LOGGER.info("MES可疑品汇总 -----start");

            for (String organizeCode : organizeCodeList) {
                //获取前一天的所有ncStatus=40报废状态下可疑品缺陷
                DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
                //按天执行前一天的所有可疑记录
                //按小时 条码+物料出现过一次 就排除当前时间段
//                DdlPreparedPack.getNumEqualPack(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), "systemSyncStatus", packBean);
//                DdlPreparedPack.timeBuilder(
//                        new SimpleDateFormat("yyyy-MM-dd").format(TimeTool.getDateBefore(new Date(), 1)) + " 00:00:00",
//                        TimeTool.getToday() + " 00:00:00", "createDatetime", packBean, true);
                // 获取当前时间前1小时的时间
//                LocalDateTime now = LocalDateTime.now();
//                LocalDateTime oneHourAgo = now.minusHours(1);
//                DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
//                String formattedNow = now.format(formatter);
//                String formattedOneHourAgo = oneHourAgo.format(formatter);
//                DdlPreparedPack.timeBuilder(formattedOneHourAgo, formattedNow, "createDatetime", packBean, true);
                DdlPreparedPack.getNumEqualPack(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), "qmsSync", packBean);

                List<MesPartInspection> mesPartInspectionList = partInspectionRepository.findByHqlWhere(packBean);
                if (!CollectionUtils.isEmpty(mesPartInspectionList)) {
                    //过滤掉已经汇总的
                    List<MesPartInspection> partInspectionList = new ArrayList<>();
                    //排序取单件逻辑未同步的一条最新的
                    Map<String, List<MesPartInspection>> partSnMap = mesPartInspectionList.stream()
                            .filter(p -> p.getSourceType() == MesExtEnumUtil.PART_INSPECTION_SOURCE_TYPE.SINGLE.getValue())
                            .collect(Collectors.groupingBy(p -> p.getSn() + "-" + p.getPartNo()));
                    for (Map.Entry<String, List<MesPartInspection>> stringListEntry : partSnMap.entrySet()) {
                        List<MesPartInspection> value = stringListEntry.getValue();
                        MesPartInspection next = value.stream().sorted(Comparator.comparing(MesPartInspection::getCreateDatetime).reversed()).iterator().next();

                        partInspectionList.add(next);
                    }
                    //批次逻辑的全部汇总数量 42692 MES:可疑品汇总逻辑更新
                    Map<String, List<MesPartInspection>> partMap = mesPartInspectionList.stream()
                            .filter(p -> p.getSourceType() == MesExtEnumUtil.PART_INSPECTION_SOURCE_TYPE.LOT.getValue())
                            .collect(Collectors.groupingBy(MesPartInspection::getPartNo));
                    for (Map.Entry<String, List<MesPartInspection>> stringListEntry : partMap.entrySet()) {
                        partInspectionList.addAll(stringListEntry.getValue());
                    }
                    LOGGER.info("MES可疑品汇总 ----- partInspectionList{}", partInspectionList.size());
                    qmsSuspiciousService.doQmsSuspiciousByPartInspection(partInspectionList, organizeCode);

                    for (MesPartInspection mesPartInspection : mesPartInspectionList) {
                        mesPartInspection.setQmsSync(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
                        //42959 QMS可疑品汇总job 会把电子化检验终检人和终检时间覆盖掉
//                        ConvertBean.serviceModelUpdate(mesPartInspection, MesExtConstWords.JOB);
                    }
                    partInspectionRepository.saveAll(mesPartInspectionList);
                }
            }
            LOGGER.info("MES可疑品汇总 ----- end");

        } catch (Exception e) {
            LOGGER.error("MES可疑品汇总 e:{}", e.toString());
        }
    }

}
