package cn.estsh.i3plus.ext.mes.apiservice.schedulejob;


import cn.estsh.i3plus.ext.mes.api.base.IMesEquipmentSpotCheckService;
import cn.estsh.i3plus.ext.mes.api.busi.IMesSpotCheckOrderService;
import cn.estsh.i3plus.mes.apiservice.schedulejob.BaseMesScheduleJob;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentSpotCheck;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.init.ApplicationProperties;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.JobExecutionContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * @Description : 定时生成点检单
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/7/15 17:08
 * @Modify:
 **/
@Slf4j
@DisallowConcurrentExecution
@Component
@ApiOperation("定时生成点检单")
public class MesSpotCheckOrderGenerateJob extends BaseMesScheduleJob {

    @Autowired
    private IMesEquipmentSpotCheckService mesEquipmentSpotCheckService;

    @Autowired
    private IMesSpotCheckOrderService mesSpotCheckOrderService;

    public MesSpotCheckOrderGenerateJob() {
        super(MesSpotCheckOrderGenerateJob.class, "定时生成点检单");
    }

    @Override
    public void executeMesJob(JobExecutionContext context, ApplicationProperties applicationProperties) {
        String jobParam = this.getJobParam();

        if (StringUtils.isBlank(jobParam)) {
            throw new IllegalArgumentException("job参数为空，请检查参数");
        }
        for (String organizeCode : jobParam.split(",")) {
            try {
                long startTime = System.currentTimeMillis();
                MesEquipmentSpotCheck mesEquipmentSpotCheck = new MesEquipmentSpotCheck();
                mesEquipmentSpotCheck.setOrganizeCode(organizeCode);
                mesEquipmentSpotCheck.setTriggerMode(MesExtEnumUtil.EQUIPMENT_CHECK_INSPECTION_TRIGGER_METHOD.TIME_TRIGGERED.getValue());
                List<MesEquipmentSpotCheck> spotCheckList = mesEquipmentSpotCheckService.findMesEquipmentSpotCheck(mesEquipmentSpotCheck);
                if (CollectionUtils.isEmpty(spotCheckList)) {
                    log.info("不存在触发方式为【时间触发】的设备点检配置");
                    continue;
                }
                mesSpotCheckOrderService.insertMesSpotCheckOrder(spotCheckList, "JOB");
                long endTime = System.currentTimeMillis();
                log.info("工厂{}定时生成点检单 --- END --- 耗时: {} ms", organizeCode, endTime - startTime);
            } catch (Exception e) {
                log.info("工厂{}定时生成点检单 执行失败{}", organizeCode, e);
            }
        }
    }

}
