package cn.estsh.i3plus.ext.mes.apiservice.schedulejob;

import cn.estsh.i3plus.ext.mes.api.base.IMesWorkOrderService;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.mes.apiservice.schedulejob.BaseMesScheduleJob;
import cn.estsh.i3plus.platform.common.tool.JsonUtilTool;
import cn.estsh.i3plus.pojo.model.wms.WmsJobParamModel;
import cn.estsh.impp.framework.boot.init.ApplicationProperties;
import io.swagger.annotations.ApiOperation;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.JobExecutionContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * @Description : MES工单推送给WMS
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/11/13 16:40
 * @Modify:
 **/
// 禁止 JOB 并发执行
@DisallowConcurrentExecution
@ApiOperation("MES工单推送给WMS JOB")
public class MesWorkOrderSendToWmsJob extends BaseMesScheduleJob {

    public static final Logger LOGGER = LoggerFactory.getLogger(MesWorkOrderSendToWmsJob.class);

    private static final long serialVersionUID = -321526601595924818L;

    @Autowired
    private IMesWorkOrderService mesWorkOrderService;

    public MesWorkOrderSendToWmsJob() {
        super(MesWorkOrderSendToWmsJob.class, "MES工单推送给WMS JOB");
        this.setMultiInstance(true);
    }

    @Override
    public void executeMesJob(JobExecutionContext context, ApplicationProperties applicationProperties) {

        List<WmsJobParamModel> wmsJobParamModelList = JsonUtilTool.toList(this.getJobParam(), WmsJobParamModel.class);
        if (!CollectionUtils.isEmpty(wmsJobParamModelList)) {
            for (WmsJobParamModel wmsJobParamModel : wmsJobParamModelList) {
                try {
                    long startTime = System.currentTimeMillis();
                    LOGGER.info("工厂{}MES工单推送给WMSJOB --- start", wmsJobParamModel.getOrganizeCode());
                    mesWorkOrderService.doSendMesWorkOrderToWms(wmsJobParamModel.getOrganizeCode(), wmsJobParamModel.getPageSize(), MesExtConstWords.JOB);
                    long endTime = System.currentTimeMillis();
                    LOGGER.info("工厂{}MES工单推送给WMSJOB --- end --- 耗时: {} ms", wmsJobParamModel.getOrganizeCode(), endTime - startTime);
                } catch (Exception e) {
                    LOGGER.info("工厂{}MES工单推送给WMSJOB 执行失败{}", wmsJobParamModel.getOrganizeCode(), e);
                }
            }
        }
    }

}
