package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesBallTrackRuleService;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesBallTrackRule;
import cn.estsh.i3plus.pojo.mes.repository.MesEquipmentRepository;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @Description :
 * @Reference :
 * @Author : PZJ
 * @CreateDate : 2024/9/24 15:37
 * @Modify:
 **/
@Service
@Slf4j
public class MesBallTrackRuleServiceImpl extends BaseMesService<MesBallTrackRule> implements IMesBallTrackRuleService {


    @Autowired
    private MesEquipmentRepository mesEquipmentRepository;
    @Override
    protected void setPackQueryBean(MesBallTrackRule bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getEquipmentCode(), "equipmentCode", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getBallTrackType(), "ballTrackType", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getBallTrackName(), "ballTrackName", packBean);
    }

    @Override
    protected void onInsertBean(MesBallTrackRule item) {
        ValidatorBean.checkNotNull(item.getBallTrackName(), "轨道名称不能为空");
        ValidatorBean.checkNotNull(item.getEquipmentCode(), "设备代码不能为空");
        ValidatorBean.checkNotNull(item.getBallTrackType(), "轨道类型不能为空");
        ValidatorBean.checkNotNull(item.getEquipVariableValue(), "轨道变量值不能为空");
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getEquipmentCode(), "equipmentCode", ddlPackBean);
        if (!mesEquipmentRepository.isExitByHql(ddlPackBean)){
            MesException.throwBusiException("设备代码不存在");
        }
    }

    @Override
    protected void onUpdateBean(MesBallTrackRule item) {
        ValidatorBean.checkNotNull(item.getId(), "id不能为空");
        ValidatorBean.checkNotNull(item.getBallTrackName(), "轨道名称不能为空");
        ValidatorBean.checkNotNull(item.getBallTrackType(), "轨道类型不能为空");
        ValidatorBean.checkNotNull(item.getEquipmentCode(), "设备代码不能为空");
        ValidatorBean.checkNotNull(item.getEquipVariableValue(), "轨道变量值不能为空");
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getEquipmentCode(), "equipmentCode", ddlPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", ddlPackBean);
        if (!mesEquipmentRepository.isExitByHql(ddlPackBean)){
            MesException.throwBusiException("设备代码不存在");
        }
    }
}
