package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesBomService;
import cn.estsh.i3plus.ext.mes.api.base.IMesConfigService;
import cn.estsh.i3plus.ext.mes.apiservice.dao.IMesBomDao;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.ext.mes.pojo.model.MesBomTreeModel;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.tool.MathOperation;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.MesConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesBom;
import cn.estsh.i3plus.pojo.mes.bean.MesConfig;
import cn.estsh.i3plus.pojo.mes.bean.MesPart;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class MesBomService extends BaseMesService<MesBom> implements IMesBomService {

    @Autowired
    private IMesBomDao mesBomDao;

    @Autowired
    private IMesConfigService mesConfigService;

    protected void setPackQueryBean(MesBom bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getBomVersion(), MesExtConstWords.BOM_VERSION, packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartNo(), MesExtConstWords.PART_NO, packBean);
    }

    @Override
    public List<String> findBomVersionByPartNo(MesBom mesBom) {
        return mesBomDao.findBomVersionByPartNo(mesBom.getPartNo(), mesBom.getOrganizeCode());
    }

    @Override
    public List<String> findBomCodeByPartNoAndBomVersion(MesBom mesBom) {
        return mesBomDao.findBomCodeByPartNoAndBomVersion(mesBom.getPartNo(), mesBom.getBomVersion(), mesBom.getOrganizeCode());
    }

    @Override
    public List queryMesDataTreeByFirstFloor(MesBom bom) {

        List resultList = new ArrayList();
        ValidatorBean.checkNotNull(bom.getPartNo(), "零件号不能为空");
        MesBomTreeModel organize = new MesBomTreeModel();

        organize.setParentId(CommonEnumUtil.PARENT.DEFAULT.getValue());
        organize.setName("LML");
//        organize.setName(AuthUtil.getOrganize().getName());

        List<MesBom> bomList = baseRDao.findByProperty(new String[]{MesConstWords.IS_VALID, MesConstWords.IS_DELETED, MesConstWords.ORGANIZE_CODE, MesExtConstWords.PART_NO},
                new Object[]{CommonEnumUtil.IS_VAILD.VAILD.getValue(), CommonEnumUtil.IS_DEAL.NO.getValue(), bom.getOrganizeCode(), bom.getPartNo()});

        if (!CollectionUtils.isEmpty(bomList)) {
            organize.setChildTreeList(bomList);
            resultList.add(organize);
        }

        return resultList;
    }

    /**
     * 根据零件号查询所有
     *
     * @param bom 组织代码
     * @return
     */
    @Override
    public ListPager<MesBom> queryMesDataTree(MesBom bom, Pager pager) {
        List resultList = new ArrayList();
        MesBomTreeModel organize = new MesBomTreeModel();

        organize.setParentId(CommonEnumUtil.PARENT.DEFAULT.getValue());
        organize.setName(AuthUtil.getOrganize().getName());
        //organize.setName("zxw");
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(bom.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(bom.getPartNo(), MesExtConstWords.PART_NO, packBean);
        DdlPreparedPack.getStringEqualPack(bom.getBomVersion(), MesExtConstWords.BOM_VERSION, packBean);
        DdlPreparedPack.getStringEqualPack(bom.getBomCode(), MesExtConstWords.BOM_CODE, packBean);
        DdlPreparedPack.getStringSmallerPack(bom.getEffStartTime(), MesExtConstWords.EFF_START_TIME, packBean);



        String sql = " and (model.effEndTime = '' or model.effEndTime is null or model.effEndTime >= '" + bom.getEffStartTime()  + "' )" ;
        packBean.setWhereAppend(packBean.getWhereAppend() +  sql);
        packBean.setOrderByStr(" order by createDatetime desc ");

        pager = PagerHelper.getPager(pager, baseRDao.findByHqlWhereCount(packBean));

        List<MesBom> bomList = baseRDao.findByHqlWherePage(packBean, pager);

        recursionBomList(bomList,bom.getEffStartTime());
        organize.setChildTreeList(bomList);
        resultList.add(organize);

        return new ListPager<>(resultList, pager);
    }

    /**
     * 根据零件号查询所有
     *
     * @param bom 组织代码
     * @return
     */
    @Override
    public ListPager<MesBom> queryMesDataTreeAll(MesBom bom, Pager pager) {
        MesBomTreeModel organize = new MesBomTreeModel();

        organize.setParentId(CommonEnumUtil.PARENT.DEFAULT.getValue());
        organize.setName(AuthUtil.getOrganize().getName());
        //organize.setName("zxw");
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(bom.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(bom.getPartNo(), MesExtConstWords.PART_NO, packBean);
        DdlPreparedPack.getStringEqualPack(bom.getBomVersion(), MesExtConstWords.BOM_VERSION, packBean);
        DdlPreparedPack.getStringEqualPack(bom.getBomCode(), MesExtConstWords.BOM_CODE, packBean);
        DdlPreparedPack.getStringSmallerPack(bom.getEffStartTime(), MesExtConstWords.EFF_START_TIME, packBean);
        packBean.setOrderByStr(" order by createDatetime desc ");
        pager = PagerHelper.getPager(pager, baseRDao.findByHqlWhereCount(packBean));

        List<MesBom> bomList = baseRDao.findByHqlWherePage(packBean, pager);

        List<MesBom> list = recursionBomListAll(bomList,bom.getEffStartTime(), null);;


        return new ListPager<>(list, pager);
    }

    //报表使用的方法
    public List<MesBom> findMesBomByPartNoAndBomVersion(String partNo, String organizeCode, String bomVersion, String bomCode, String effStartTime) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partNo, MesExtConstWords.PART_NO, ddlPackBean);
        DdlPreparedPack.getStringEqualPack(bomVersion, MesExtConstWords.BOM_VERSION, ddlPackBean);
        DdlPreparedPack.getStringEqualPack(bomCode, MesExtConstWords.BOM_CODE, ddlPackBean);
        DdlPreparedPack.getStringSmallerPack(effStartTime, MesExtConstWords.EFF_START_TIME, ddlPackBean);
        DdlPreparedPack.getOrderBy(null, null, ddlPackBean);
        List<MesBom> mesBoms = baseRDao.findByHqlWhere(ddlPackBean);
        if (CollectionUtils.isEmpty(mesBoms)) {
            MesException.throwMesBusiException("物料【%s】生产版本【%s】BOM代码【%s】生效时间小于【%s】对应bom信息不存在", partNo, bomVersion, bomCode, effStartTime);
        }
        return mesBoms;
    }

    //生产汇报使用的方法
    public List<MesBom> findMesBomByPartNoAndBomVersion(String partNo, String organizeCode, String bomVersion) {

        //MES汇报查询BOM是否携带结束查询条件
        MesConfig config = mesConfigService.getCfgValueByCode(organizeCode, MesExtConstWords.MES_REPORT_FIND_BOM_WITH_EFFENDTIME);
        Boolean isWithEffEndTime = (null != config && !StringUtils.isEmpty(config.getCfgValue()) && config.getCfgValue().equals(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValueStr())) ? true : false;

        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partNo, MesExtConstWords.PART_NO, ddlPackBean);
        if (!StringUtils.isEmpty(bomVersion)) {
            DdlPreparedPack.getStringEqualPack(bomVersion, MesExtConstWords.BOM_VERSION, ddlPackBean);
        }
        DdlPreparedPack.getStringSmallerPack(TimeTool.getNowTime(true), MesExtConstWords.EFF_START_TIME, ddlPackBean);
        if (isWithEffEndTime) DdlPreparedPack.getStringBiggerPack(TimeTool.getNowTime(true), MesExtConstWords.EFF_END_TIME, ddlPackBean);
        DdlPreparedPack.getOrderBy(MesExtConstWords.EFF_START_TIME, CommonEnumUtil.ASC_OR_DESC.DESC.getValue(), ddlPackBean);
        List<MesBom> mesBoms = baseRDao.findByHqlTopWhere(ddlPackBean,1);
        if (CollectionUtils.isEmpty(mesBoms)) {
            MesException.throwMesBusiException("物料【%s】生产版本【%s】对应bom信息不存在", partNo, bomVersion);
        }
        MesBom mesBom = mesBoms.get(0);
        String bomCode = mesBom.getBomCode();
        String effStartTime = mesBom.getEffStartTime();
        DdlPackBean ddlPackBean1= DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partNo, MesExtConstWords.PART_NO, ddlPackBean1);
        DdlPreparedPack.getStringEqualPack(bomCode, MesExtConstWords.BOM_CODE, ddlPackBean1);
        DdlPreparedPack.getStringSmallerPack(effStartTime, MesExtConstWords.EFF_START_TIME, ddlPackBean1);
        return baseRDao.findByHqlWhere(ddlPackBean1);
    }

    @Override
    public List<MesBom> findBomList(String organizeCode, MesPart mesPart, String bomVersion) {
        List<MesBom> bomData = new ArrayList<>();
        // 递归展开 BOM
        expendBomChild(organizeCode, mesPart.getPartNo(), bomVersion, bomData);

        if (bomData.size() == 0) {
            MesException.throwMesBusiException("零件号 %s 没找找到 ERP 同步的原始 BOM 数据，请检查接口是否正常！", mesPart.getPartNo());
        }

        Iterator<MesBom> iterator = bomData.iterator();
        while (iterator.hasNext()) {
            MesBom item = iterator.next();
            // 删除虚零件
            if (Objects.equals("X", item.getPartType())) {
                iterator.remove();
            } else {
                item.setPartNo(mesPart.getPartNo());
                item.setPartName(mesPart.getPartName());
            }
        }

        Map<String, MesBom> summaryBomMap = new HashMap<>();

        for (MesBom item : bomData) {
            String key = String.format("%s:%s", item.getPartNo().toUpperCase().trim(),
                    item.getItemPartNo().toUpperCase().trim());
            if (!summaryBomMap.containsKey(key)) {
                summaryBomMap.put(key, item);
            } else {
                MesBom tempItem = summaryBomMap.get(key);
                // 子零件重复的累加数量
                tempItem.setItemQty(MathOperation.add(tempItem.getItemQty(), item.getItemQty()));
            }
        }
        // 返回汇总数据
        return new ArrayList<>(summaryBomMap.values());
    }

    @Override
    public MesBom findBom(String organizeCode, String bomVersion, String partNo) {
        DdlPackBean bomPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partNo, MesExtConstWords.PART_NO, bomPackBean);
        if (bomVersion != null){
            DdlPreparedPack.getStringEqualPack(bomVersion, MesExtConstWords.BOM_VERSION, bomPackBean);
        }
        DdlPreparedPack.getOrderBy(MesExtConstWords.EFF_START_TIME, CommonEnumUtil.ASC_OR_DESC.DESC.getValue(), bomPackBean);
        List<MesBom> bomList = this.baseRDao.findByHqlTopWhere(bomPackBean,1);
        return bomList.isEmpty() ? null : bomList.get(0);
    }

    @Override
    public MesBom findBom(String organizeCode, String partNo) {
        //MES汇报查询BOM是否携带结束查询条件
        MesConfig config = mesConfigService.getCfgValueByCode(organizeCode, MesExtConstWords.MES_REPORT_FIND_BOM_WITH_EFFENDTIME);
        Boolean isWithEffEndTime = (null != config && !StringUtils.isEmpty(config.getCfgValue()) && config.getCfgValue().equals(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValueStr())) ? true : false;

        DdlPackBean bomPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partNo, MesExtConstWords.PART_NO, bomPackBean);
        DdlPreparedPack.getStringSmallerPack(TimeTool.getNowTime(true), MesExtConstWords.EFF_START_TIME, bomPackBean);
        if (isWithEffEndTime){
            DdlPreparedPack.getStringBiggerPack(TimeTool.getNowTime(true), MesExtConstWords.EFF_END_TIME, bomPackBean);
        }
        DdlPreparedPack.getOrderBy(MesExtConstWords.EFF_START_TIME, CommonEnumUtil.ASC_OR_DESC.DESC.getValue(), bomPackBean);
        List<MesBom> bomList = this.baseRDao.findByHqlTopWhere(bomPackBean,1);
        return bomList.isEmpty() ? null : bomList.get(0);
    }

    @Override
    public List<MesBom> queryMesBomReverse(MesBom mesBom) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(mesBom.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesBom.getPartNo(), "itemPartNo", packBean);
        DdlPreparedPack.getStringSmallerPack(TimeTool.getNowTime(true), MesExtConstWords.EFF_START_TIME, packBean);
        DdlPreparedPack.getOrderBy(MesExtConstWords.EFF_START_TIME, CommonEnumUtil.ASC_OR_DESC.DESC.getValue(), packBean);
        List<MesBom> mesBomList = baseRDao.findByHqlWhere(packBean);
        if(CollectionUtils.isEmpty(mesBomList)){
            return new ArrayList<>();
        }
        List<MesBom> resultMesBomList = new ArrayList<>();
        Map<String, List<MesBom>> mesBomMap = mesBomList.stream().collect(Collectors.groupingBy(bom -> bom.getPartNo() + bom.getItemPartNo()));
        for (String partNoKey : mesBomMap.keySet()) {
            MesBom bom = mesBomMap.get(partNoKey).get(MesExtConstWords.ZERO);
            resultMesBomList.add(bom);
            //按照层数递归上一次
            bomReverse(resultMesBomList,bom.getPartNo(),bom.getOrganizeCode(),mesBom.getNum()-1);
        }
        return resultMesBomList;
    }

    private void bomReverse(List<MesBom> resultMesBomList,String partNo,String organizeCode, Integer num) {
        if(num == MesExtConstWords.ZERO) return;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partNo, "itemPartNo", packBean);
        DdlPreparedPack.getStringEqualPack("X", "partType", packBean);
        DdlPreparedPack.getStringSmallerPack(TimeTool.getNowTime(true), MesExtConstWords.EFF_START_TIME, packBean);
        DdlPreparedPack.getOrderBy(MesExtConstWords.EFF_START_TIME, CommonEnumUtil.ASC_OR_DESC.DESC.getValue(), packBean);
        List<MesBom> mesBomList = baseRDao.findByHqlWhere(packBean);
        if(CollectionUtils.isEmpty(mesBomList)) return;
        Map<String, List<MesBom>> mesBomMap = mesBomList.stream().collect(Collectors.groupingBy(bom -> bom.getPartNo() + bom.getItemPartNo()));
        for (String partNoKey : mesBomMap.keySet()) {
            MesBom bom = mesBomMap.get(partNoKey).get(MesExtConstWords.ZERO);
            resultMesBomList.add(bom);
            //按照层数递归上一次
            bomReverse(resultMesBomList,bom.getPartNo(),bom.getOrganizeCode(),num-1);
        }
    }

    /**
     * 递归展开 BOM
     *
     * @param organizeCode 工厂代码
     * @param partNo       父零件号
     * @param bomData      存放 BOM 数据的集合
     */
    private void expendBomChild(String organizeCode, String partNo, String bomVersion, List<MesBom> bomData) {
        List<MesBom> bomDetails = this.findMesBomByPartNoAndBomVersion(partNo, organizeCode, bomVersion);

        for (MesBom item : bomDetails) {
            MesBom newItem = new MesBom();
            BeanUtils.copyProperties(item, newItem);
            // 处理父零件用量不为 1 的情况
            if (newItem.getQty() != 1.0 && newItem.getQty() > 0.0) {
                Double rateQty = MathOperation.div(newItem.getItemQty(), newItem.getQty(), 8);
                newItem.setQty(1.0);
                newItem.setItemQty(rateQty);
            }
            // 加入到 BOM 清单中
            bomData.add(newItem);
            // 如果有子阶，继续递归
            if (Objects.equals("X", item.getPartType())) {
                expendBomChild(organizeCode, newItem.getItemPartNo(), null, bomData);
            }
        }
    }

    /**
     * 递归查询bom
     *
     * @return
     */

    private List<MesBom> recursionBomList(List<MesBom> bomList,String effStartTime) {
        if (!CollectionUtils.isEmpty(bomList)) {
            for (MesBom bom : bomList) {
                if (Objects.equals("X", bom.getPartType())) {
                    List<MesBom> resultList = findMesBomByPartNoAndBomVersion(bom.getItemPartNo(), bom.getOrganizeCode(), bom.getBomVersion(), bom.getBomCode(), effStartTime);
                    bom.setChildTreeList(resultList);
                    recursionBomList(resultList,effStartTime);
                }
            }
        }
        return bomList;

    }
    /**
     * 递归查询bom
     *
     * @return
     */

    private List<MesBom> recursionBomListAll(List<MesBom> bomList,String effStartTime,List<MesBom> allList) {
        if (CollectionUtils.isEmpty(allList)) {
            allList = new ArrayList<>();
            allList.addAll(bomList);
        }

        if (!CollectionUtils.isEmpty(bomList)) {
            for (MesBom bom : bomList) {
                if (Objects.equals("X", bom.getPartType())) {
                    List<MesBom> resultList = findMesBomByPartNoAndBomVersion(bom.getItemPartNo(), bom.getOrganizeCode(), bom.getBomVersion(), bom.getBomCode(), effStartTime);
                    allList.addAll(resultList);
                    recursionBomListAll(resultList,effStartTime, allList);
                }
            }
        }
        return allList;

    }

}
