package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesConfigService;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesConfig;
import cn.estsh.i3plus.pojo.mes.repository.MesConfigRepository;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Description:
 * @CreateDate: 2021/9/9 下午3:25
 * @Author: simon.song
 */
@Service
@Slf4j
public class MesConfigService implements IMesConfigService {
    @Autowired
    private MesConfigRepository configRepository;

    @Override
    public String getCfgValue(String org, String key) {
        MesConfig config = configRepository.getByProperty(
                new String[]{"cfgKey", "organizeCode", "isValid", "isDeleted"},
                new Object[]{key, org, CommonEnumUtil.IS_VAILD.VAILD.getValue(), CommonEnumUtil.IS_DEAL.NO.getValue()});

        if (config == null) {
            throw ImppExceptionBuilder.newInstance().setErrorCode(cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                    .setErrorDetail(String.format("配置表数据不存在，请维护key【%s】", key))
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .build();
        }
        return config.getCfgValue();
    }

    @Override
    public MesConfig getCfgValueByCode(String cfgCode) {
        return this.configRepository.getByProperty(new String[]{"isDeleted", "isValid", "cfgCode"},
                new Object[]{CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), cfgCode});
    }

    @Override
    public MesConfig getCfgValueByCode(String organizeCode,String cfgCode) {
        return this.configRepository.getByProperty(new String[]{"organizeCode","isDeleted", "isValid", "cfgCode"},
                new Object[]{organizeCode,CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), cfgCode});
    }

    @Override
    public List<MesConfig> getConfigList(String organizeCode, String cfgCode, String cfgKey, String cfgType) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(cfgCode, MesExtConstWords.CFG_CODE, packBean);
        DdlPreparedPack.getStringEqualPack(cfgKey, MesExtConstWords.CFG_KEY, packBean);
        DdlPreparedPack.getStringEqualPack(cfgType, MesExtConstWords.CFG_TYPE, packBean);
        return configRepository.findByHqlWhere(packBean);
    }

    @Override
    public Boolean checkCfgValueIsContainItem(String organizeCode, String cfgCode, String cfgKey, String cfgType, String splitStr, String item) {
        String cfgValue = getAndMergeCfgValue(organizeCode, cfgCode, cfgKey, cfgType, splitStr);
        if (StringUtils.isEmpty(cfgValue)) return false;
        List<String> cfgValueList = Arrays.asList(cfgValue.split(splitStr));
        return (!CollectionUtils.isEmpty(cfgValueList) && cfgValueList.contains(item)) ? true : false;
    }

    private String getAndMergeCfgValue(String organizeCode, String cfgCode, String cfgKey, String cfgType, String splitStr) {
        List<MesConfig> configList = getConfigList(organizeCode, cfgCode, cfgKey, cfgType);
        if (CollectionUtils.isEmpty(configList)) return null;
        return configList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getCfgValue()))).map(MesConfig::getCfgValue).collect(Collectors.joining(splitStr));
    }

    @Override
    public Boolean checkShardingTablesObjectCfg(String organizeCode, String cfgKey, String item) {
        return checkCfgValueIsContainItem(organizeCode, MesExtConstWords.MES_SHARDING_TABLES_OBJECT_CFG, cfgKey, CommonEnumUtil.SOFT_TYPE.MES.name(), MesExtConstWords.COMMA, item);
    }

    @Override
    public List<String> findCfgValueContainItem(String organizeCode, String cfgCode, String cfgKey, String cfgType, String splitStr, List<String> itemList) {
        String cfgValue = getAndMergeCfgValue(organizeCode, cfgCode, cfgKey, cfgType, splitStr);
        if (StringUtils.isEmpty(cfgValue)) return null;
        List<String> cfgValueList = Arrays.asList(cfgValue.split(splitStr));
        if (CollectionUtils.isEmpty(cfgValueList)) return null;
        return itemList.stream().filter(cfgValueList::contains).collect(Collectors.toList());
    }

    @Override
    public List<String> findShardingTablesObjectCfg(String organizeCode, String cfgKey, List<String> itemList) {
        if(CollectionUtils.isEmpty(itemList)) return null;
        return findCfgValueContainItem(organizeCode, MesExtConstWords.MES_SHARDING_TABLES_OBJECT_CFG, cfgKey, CommonEnumUtil.SOFT_TYPE.MES.name(), MesExtConstWords.COMMA, itemList);
    }

    @Override
    public void update(MesConfig mesConfig,String userName) {
        ConvertBean.serviceModelUpdate(mesConfig,userName);
        configRepository.update(mesConfig);
    }

    @Override
    public Boolean checkCfgValue(String organizeCode, String configCode, String value) {
        MesConfig config = getCfgValueByCode(organizeCode, configCode);
        return !Objects.isNull(config) && !StringUtil.isEmpty(config.getCfgValue()) && !StringUtil.isEmpty(value) && config.getCfgValue().equals(value);
    }

}
