package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesCraftRouteService;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesCraftRoute;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

/**
 * @Description : 工艺路线
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/5/12 15:13
 * @Modify:
 **/
@Service
@Slf4j
public class MesCraftRouteServiceImpl extends BaseMesService<MesCraftRoute> implements IMesCraftRouteService {

    @Override
    protected void setPackQueryBean(MesCraftRoute bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getCraftRouteCode(), "craftRouteCode", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getCraftRouteName(), "craftRouteName", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getCraftRouteType(), "craftRouteType", packBean);
    }

    @Override
    protected void onInsertBean(MesCraftRoute item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getCraftRouteCode(), "工艺路线代码不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getCraftRouteCode(), "craftRouteCode", roadPackBean);

        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("工艺路线代码【%s】已经存在，请检查数据", item.getCraftRouteCode())
                    .build();
        }
    }

    @Override
    protected void onUpdateBean(MesCraftRoute item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getCraftRouteCode(), "工艺路线代码不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getCraftRouteCode(), "craftRouteCode", roadPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", roadPackBean);
        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("工艺路线代码【%s】已经存在，请检查数据", item.getCraftRouteCode())
                    .build();
        }
    }

    @Override
    public MesCraftRoute getMesCraftRouteByCode(String code, String organizeCode) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(code, "craftRouteCode", ddlPackBean);
        MesCraftRoute mesCraftRoute = baseRDao.getByProperty(ddlPackBean);
        if (null == mesCraftRoute) {
            MesException.throwMesBusiException("工艺路线代码【%s】不存在，请检查数据", code);
        }
        return mesCraftRoute;
    }
}
