package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesCustomerCarModelDetailService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesCustomerCarModel;
import cn.estsh.i3plus.pojo.mes.bean.MesCustomerCarModelDetail;
import cn.estsh.i3plus.pojo.mes.bean.MesPartShippingGroup;
import cn.estsh.i3plus.pojo.mes.repository.MesCustPartInvoicedRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesCustomerCarModelRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesCustomerPartRepository;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Slf4j
public class MesCustomerCarModelDetailService extends BaseMesService<MesCustomerCarModelDetail> implements IMesCustomerCarModelDetailService {
    @Autowired
    private MesCustPartInvoicedRepository mesCustPartInvoicedRao;

    @Autowired
    private MesCustomerPartRepository customerPartRao;

    @Autowired
    private MesCustomerCarModelRepository customerCarModelRao;

    private boolean check(String organizeCode,String custPartNo,String carModelCode){
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(carModelCode,"carModelCode", ddlPackBean);
        List<MesCustomerCarModel> carModelList = customerCarModelRao.findByHqlTopWhere(ddlPackBean, 1);
        if (carModelList == null || carModelList.isEmpty()) {
            return false;
        }
        String custCode = carModelList.get(0).getCustCode();
        DdlPackBean ddlPackBean1 = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(custCode,"custCode",ddlPackBean1);
        DdlPreparedPack.getStringEqualPack(custPartNo,"custPartNo",ddlPackBean1);

        int count = mesCustPartInvoicedRao.findByHqlWhereCount(ddlPackBean1);
        if (count > 0) {
            return true;
        }else {
            DdlPackBean custPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(custCode,"custCode",custPackBean);
            DdlPreparedPack.getStringEqualPack(custPartNo,"custPartNo",custPackBean);
            int count1 = customerPartRao.findByHqlWhereCount(custPackBean);
            return count1 > 0;
        }
    }
    protected void setPackQueryBean(MesCustomerCarModelDetail bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getCarModelCode(),"carModelCode",packBean);
    }

    protected void onInsertBean(MesCustomerCarModelDetail item) {
        boolean check = check(AuthUtil.getOrganizeCode(), item.getCustPartNo(), item.getCarModelCode());
        if (!check) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("待开票表及客户零件表不存在客户零件号【%s】，请检查数据",
                            item.getCustPartNo())
                    .build();
        }
        // 数据校验

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getCustPartNo(), "custPartNo", roadPackBean);

        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("客户零件号【%s】已经存在，请检查数据", item.getCustPartNo())
                    .build();
        }

    }

    protected void onUpdateBean(MesCustomerCarModelDetail item) {
        // 数据校验

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getCustPartNo(), "custPartNo", roadPackBean);

        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", roadPackBean);
        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("客户零件号【%s】已经存在，请检查数据", item.getCustPartNo())
                    .build();
        }
    }
}
