package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesCustomerMessagePointService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesCustomerMessagePoint;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Slf4j
public class MesCustomerMessagePointService extends BaseMesService<MesCustomerMessagePoint> implements IMesCustomerMessagePointService {
    protected void onInsertBean(MesCustomerMessagePoint item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getMessageNo(), "信息点代码不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getMessageNo(), "messageNo", roadPackBean);
        DdlPreparedPack.getNumEqualPack(item.getMessageOrder(), "messageOrder", roadPackBean);

        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("信息点代码【%s】顺序号【%s】已经存在，请检查数据", item.getMessageNo(),item.getMessageOrder())
                    .build();
        }

    }

    protected void onUpdateBean(MesCustomerMessagePoint item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getMessageNo(), "信息点代码不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getMessageNo(), "messageNo", roadPackBean);
        DdlPreparedPack.getNumEqualPack(item.getMessageOrder(), "messageOrder", roadPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", roadPackBean);
        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("信息点代码【%s】顺序号【%s】已经存在，请检查数据", item.getMessageNo(),item.getMessageOrder())
                    .build();
        }
    }

    protected void setPackQueryBean(MesCustomerMessagePoint bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getCustOrganizeCode(), "custOrganizeCode", packBean);
    }

    @Override
    public MesCustomerMessagePoint getPointByCode(String pointCode, String organizeCode) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(pointCode, "messageNo", ddlPackBean);
        List<MesCustomerMessagePoint> messagePoints = baseRDao.findByHqlWhere(ddlPackBean);
        return messagePoints.isEmpty() ? null: messagePoints.get(0);
    }

    @Override
    public List<MesCustomerMessagePoint> getByCustOrganizeCode(String custOrganizeCode) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(AuthUtil.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(custOrganizeCode, "custOrganizeCode", ddlPackBean);
        return baseRDao.findByHqlWhere(ddlPackBean);
    }
}
