package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesCustomerPartTypeService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesCustomerPartType;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
/**
 * @Description : 客户零件类型
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/11/5 17:25
 * @Modify:
 **/
@Service
@Slf4j
public class MesCustomerPartTypeServiceImpl extends BaseMesService<MesCustomerPartType> implements IMesCustomerPartTypeService {

    @Override
    protected void onInsertBean(MesCustomerPartType item) {
        checkData(item);
    }

    @Override
    protected void onUpdateBean(MesCustomerPartType item) {
        checkData(item);
    }

    @Override
    protected void setPackQueryBean(MesCustomerPartType bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringLikerPack(bean.getPartTypeCode(), "partTypeCode", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartTypeName(), "partTypeName", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getCustPartNo(), "custPartNo", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getCustPartName(), "custPartName", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getCustCode(), "custCode", packBean);
    }

    private void checkData(MesCustomerPartType item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getPartTypeCode(), "零件类型代码不能为空");
        ValidatorBean.checkNotNull(item.getCustPartNo(), "客户物料号不能为空");

        DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPartTypeCode(), "partTypeCode", seriesPackBean);
        DdlPreparedPack.getStringEqualPack(item.getCustPartNo(), "custPartNo", seriesPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", seriesPackBean);
        boolean flg = baseRDao.isExitByHql(seriesPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("类型【%s】客户零件【%s】数据已存在，请检查数据", item.getPartTypeCode(), item.getCustPartNo())
                    .build();
        }
    }
}
