package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesEquitmentChannelService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentChannel;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Slf4j
public class MesEquipmentChannelService extends BaseMesService<MesEquipmentChannel> implements IMesEquitmentChannelService {


    @Override
    protected void setPackQueryBean(MesEquipmentChannel bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getEquipmentCode(),"equipmentCode",packBean);
        DdlPreparedPack.getStringEqualPack(bean.getChannel(),"channel",packBean);

    }


    @Override
    public MesEquipmentChannel insert(MesEquipmentChannel bean) {
        //插入前构造并校验数据
        onInsertBean(bean);

        bean = baseRDao.insert(bean);
        return bean;
    }

    @Override
    public MesEquipmentChannel update(MesEquipmentChannel bean) {
        //插入前构造并校验数据
        onUpdateBean(bean);

        baseRDao.update(bean);
        return bean;
    }


    protected void onInsertBean(MesEquipmentChannel item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getEquipmentCode(), "设备代码不能为空");
        ValidatorBean.checkNotNull(item.getChannel(), "设备通道不能为空");


        DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getEquipmentCode(), "equipmentCode", seriesPackBean);
        DdlPreparedPack.getStringEqualPack(item.getChannel(), "channel", seriesPackBean);

        boolean flg = baseRDao.isExitByHql(seriesPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】设备代码和【%s】通道已经存在，请检查数据", item.getEquipmentCode(), item.getChannel())
                    .build();
        }

    }

    protected void onUpdateBean(MesEquipmentChannel item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getEquipmentCode(), "设备代码不能为空");
        ValidatorBean.checkNotNull(item.getChannel(), "设备通道不能为空");


        DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getEquipmentCode(), "equipmentCode", seriesPackBean);
        DdlPreparedPack.getStringEqualPack(item.getChannel(), "channel", seriesPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", seriesPackBean);

        boolean flg = baseRDao.isExitByHql(seriesPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】设备代码和【%s】通道已经存在，请检查数据", item.getEquipmentCode(), item.getChannel())
                    .build();
        }
    }
    // 校验导入数据
    @Override
    public void validateImport(List<MesEquipmentChannel> beanList) {

        for (MesEquipmentChannel item : beanList) {
            ValidatorBean.checkNotNull(item.getEquipmentCode(), "设备代码不能为空");
            ValidatorBean.checkNotNull(item.getChannel(), "设备通道不能为空");


            DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(item.getEquipmentCode(), "equipmentCode", seriesPackBean);
            DdlPreparedPack.getStringEqualPack(item.getChannel(), "channel", seriesPackBean);

            boolean flg = baseRDao.isExitByHql(seriesPackBean);
            if (flg) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("【%s】设备代码和【%s】通道已经存在，请检查数据", item.getEquipmentCode(), item.getChannel())
                        .build();
            }
        }
    }
}
