package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesEquipmentSpotCheckPartService;
import cn.estsh.i3plus.ext.mes.apiservice.config.AuthUtilExt;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentSpotCheck;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentSpotCheckPart;
import cn.estsh.i3plus.pojo.mes.bean.MesPart;
import cn.estsh.i3plus.pojo.mes.repository.MesEquipmentSpotCheckRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesPartRepository;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Slf4j
public class MesEquipmentSpotCheckPartService extends BaseMesService<MesEquipmentSpotCheckPart> implements IMesEquipmentSpotCheckPartService {

    @Autowired
    private MesEquipmentSpotCheckRepository equipmentSpotCheckRepository;
    @Autowired
    private MesPartRepository mesPartRepository;

    @Override
    protected void setPackQueryBean(MesEquipmentSpotCheckPart bean, DdlPackBean packBean) {
        DdlPreparedPack.getNumEqualPack(bean.getPid(), "pid", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartNo(), "partNo", packBean);
    }

    @Override
    public void validateImport(List<MesEquipmentSpotCheckPart> beanList) {
        String organizeCode = AuthUtil.getOrganizeCode();
        for (MesEquipmentSpotCheckPart item : beanList) {
            // 数据校验
            if (StringUtil.isEmpty(item.getPid())) {
                ValidatorBean.checkNotNull(item.getSpotCheckCode(), "点检项目代码不能为空");

                DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(organizeCode);
                DdlPreparedPack.getStringEqualPack(item.getSpotCheckCode(), "spotCheckCode", seriesPackBean);
                MesEquipmentSpotCheck itemFlg = equipmentSpotCheckRepository.getByProperty(seriesPackBean);
                if (StringUtil.isEmpty(itemFlg)) {
                    MesException.throwMesBusiException("点检项目代码【%s】不存在，请检查数据", item.getSpotCheckCode());
                }
                item.setPid(itemFlg.getId());

                DdlPackBean partPackBean = DdlPackBean.getDdlPackBean(organizeCode);
                DdlPreparedPack.getStringEqualPack(item.getPartNo(),"partNo", partPackBean);
                MesPart mesPart = mesPartRepository.getByProperty(partPackBean);
                if (mesPart == null) {
                    MesException.throwMesBusiException("零件号【%s】不存在", item.getPartNo());
                }
                item.setPartName(mesPart.getPartName());

            } else {
                ValidatorBean.checkNotNull(item.getPid(), "点检项目id不能为空");

                DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
                DdlPreparedPack.getNumEqualPack(item.getPid(), "id", seriesPackBean);

                if (!equipmentSpotCheckRepository.isExitByHql(seriesPackBean)) {
                    MesException.throwMesBusiException("点检项目id【%s】不存在，请检查数据", item.getPid());
                }
            }
            item.setId(null);
            item.setSystemSyncStatus(CommonEnumUtil.FALSE);
            item.setSystemSyncDatetime("");
            ConvertBean.serviceModelInitialize(item, AuthUtilExt.getUserName());
        }
    }

    @Override
    public List<MesEquipmentSpotCheckPart> validateReturnImport(List<MesEquipmentSpotCheckPart> beanList) {

        for (MesEquipmentSpotCheckPart item : beanList) {
            // 数据校验
            if (StringUtil.isEmpty(item.getPid())) {
                ValidatorBean.checkNotNull(item.getSpotCheckCode(), "点检项目代码不能为空");

                DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
                DdlPreparedPack.getStringEqualPack(item.getSpotCheckCode(), "spotCheckCode", seriesPackBean);
                MesEquipmentSpotCheck itemFlg = equipmentSpotCheckRepository.getByProperty(seriesPackBean);
                if (StringUtil.isEmpty(itemFlg)) {
                    MesException.throwMesBusiException("点检项目代码【%s】不存在，请检查数据", item.getSpotCheckCode());
                }
                item.setPid(itemFlg.getId());
            } else {
                ValidatorBean.checkNotNull(item.getPid(), "点检项目id不能为空");

                DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
                DdlPreparedPack.getNumEqualPack(item.getPid(), "id", seriesPackBean);

                if (!equipmentSpotCheckRepository.isExitByHql(seriesPackBean)) {
                    MesException.throwMesBusiException("点检项目id【%s】不存在，请检查数据", item.getPid());
                }
            }
        }
        return beanList;
    }
}
