package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesEquipmentSpotCheckService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipment;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentSpotCheck;
import cn.estsh.i3plus.pojo.mes.repository.MesEquipmentRepository;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Slf4j
public class MesEquipmentSpotCheckService extends BaseMesService<MesEquipmentSpotCheck> implements IMesEquipmentSpotCheckService {

    @Autowired
    private MesEquipmentRepository equipmentRepository;

    protected void setPackQueryBean(MesEquipmentSpotCheck bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getSpotCheckCode(), "spotCheckCode", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getSpotCheckName(), "spotCheckName", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getWorkCellCode(), "workCellCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getEquipCode(), "equipCode", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getSpotCheckOrderType(), "spotCheckOrderType", packBean);
        //43172 点检主数据主表导入和新增时，基于设备代码排序
        DdlPreparedPack.getOrderByPack(
                new Object[]{CommonEnumUtil.ASC_OR_DESC.ASC.getValue(), CommonEnumUtil.ASC_OR_DESC.DESC.getValue()},
                new String[]{"equipCode", "createDatetime"}, packBean);

    }

    protected void onInsertBean(MesEquipmentSpotCheck item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getSpotCheckCode(), "点检项目代码不能为空");

        DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getSpotCheckCode(), "spotCheckCode", seriesPackBean);
        boolean flg = baseRDao.isExitByHql(seriesPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】点检项目代码已经存在，请检查数据", item.getSpotCheckCode())
                    .build();
        }

    }

    protected void onUpdateBean(MesEquipmentSpotCheck item) {

        // 数据校验
        ValidatorBean.checkNotNull(item.getSpotCheckCode(), "点检项目代码不能为空");

        DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getSpotCheckCode(), "spotCheckCode", seriesPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", seriesPackBean);
        boolean flg = baseRDao.isExitByHql(seriesPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】点检项目代码已经存在，请检查数据", item.getSpotCheckCode())
                    .build();
        }
    }

    @Override
    public void validateImport(List<MesEquipmentSpotCheck> beanList) {

        for (MesEquipmentSpotCheck item : beanList) {
            // 数据校验
            ValidatorBean.checkNotNull(item.getSpotCheckCode(), "点检项目代码不能为空");

            DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(item.getSpotCheckCode(), "spotCheckCode", seriesPackBean);
            boolean flg = baseRDao.isExitByHql(seriesPackBean);
            if (flg) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("【%s】点检项目代码已经存在，请检查数据", item.getSpotCheckCode())
                        .build();
            }
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(item.getEquipName(),"equipmentName", ddlPackBean);

            MesEquipment mesEquipment = equipmentRepository.getByProperty(ddlPackBean);
            if (mesEquipment == null) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("【%s】设备名称不存在，请检查数据", item.getEquipName())
                        .build();
            }
            item.setEquipCode(mesEquipment.getEquipmentCode());
        }
    }

    @Override
    public List<MesEquipmentSpotCheck> findMesEquipmentSpotCheck(MesEquipmentSpotCheck mesEquipmentSpotCheck) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(mesEquipmentSpotCheck.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(mesEquipmentSpotCheck.getTriggerMode(), "triggerMode", packBean);
        return baseRDao.findByHqlWhere(packBean);
    }
}
