package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesInterfaceDataMapperService;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.dbinterface.MesInterfaceDataMapper;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

/**
 * @Description : 接口数据映射
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/8/22 10:04
 * @Modify:
 **/
@Service
@Slf4j
public class MesInterfaceDataMapperServiceImpl extends BaseMesService<MesInterfaceDataMapper> implements IMesInterfaceDataMapperService {

    @Override
    protected void onInsertBean(MesInterfaceDataMapper item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getSrcTableName(), "来源表名不能为空");
        ValidatorBean.checkNotNull(item.getSrcWhere(), "来源表的查询条件不能为空");
        ValidatorBean.checkNotNull(item.getDestBeanName(), "目标实体对象的名字不能为空");

        DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getSrcTableName(), "srcTableName", seriesPackBean);
        DdlPreparedPack.getStringEqualPack(item.getSrcWhere(), "srcWhere", seriesPackBean);
        DdlPreparedPack.getStringEqualPack(item.getDestBeanName(), "destBeanName", seriesPackBean);
        if (baseRDao.isExitByHql(seriesPackBean)) {
            MesException.throwMesBusiException("来源表名为【%s】、查询条件为【%s】、目标实体对象为【%s】接口映射信息已存在",
                    item.getSrcTableName(), item.getSrcWhere(), item.getDestBeanName());
        }

    }

    @Override
    protected void onUpdateBean(MesInterfaceDataMapper item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getSrcTableName(), "来源表名不能为空");
        ValidatorBean.checkNotNull(item.getSrcWhere(), "来源表的查询条件不能为空");
        ValidatorBean.checkNotNull(item.getDestBeanName(), "目标实体对象的名字不能为空");

        DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getSrcTableName(), "srcTableName", seriesPackBean);
        DdlPreparedPack.getStringEqualPack(item.getSrcWhere(), "srcWhere", seriesPackBean);
        DdlPreparedPack.getStringEqualPack(item.getDestBeanName(), "destBeanName", seriesPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", seriesPackBean);
        if (baseRDao.isExitByHql(seriesPackBean)) {
            MesException.throwMesBusiException("来源表名为【%s】、查询条件为【%s】、目标实体对象为【%s】接口映射信息已存在",
                    item.getSrcTableName(), item.getSrcWhere(), item.getDestBeanName());
        }
    }

    @Override
    protected void setPackQueryBean(MesInterfaceDataMapper bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringLikerPack(bean.getSrcTableName(), "srcTableName", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getDestBeanName(), "destBeanName", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getBusiRoute(), "busiRoute", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getGroupName(), "groupName", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getInterfaceName(), "interfaceName", packBean);
    }

}
