package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesLabelTemplateService;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.util.MesPcnConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesLabelTemplate;
import cn.estsh.i3plus.pojo.mes.bean.MesLabelTemplateFieldInfo;
import cn.estsh.i3plus.pojo.mes.bean.MesLabelTemplateFieldValue;
import cn.estsh.i3plus.pojo.mes.bean.MesLabelTemplateParam;
import cn.estsh.i3plus.pojo.mes.repository.MesLabelTemplateFieldInfoRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesLabelTemplateFieldValueRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesLabelTemplateParamRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesPartRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.util.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

/**
 * @Description : 打印模板
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/7/10 13:43
 * @Modify:
 **/
@Service
@Slf4j
public class MesLabelTemplateServiceImpl extends BaseMesService<MesLabelTemplate> implements IMesLabelTemplateService {

    @Autowired
    private MesLabelTemplateParamRepository mesLabelTemplateParamRDao;
    @Autowired
    private MesLabelTemplateFieldInfoRepository labelTemplateFieldInfoRDao;
    @Autowired
    private MesPartRepository mesPartRepository;
    @Autowired
    private MesLabelTemplateFieldValueRepository labelTemplateFieldValueRDao;

    @Override
    protected void setPackQueryBean(MesLabelTemplate bean, DdlPackBean packBean) {

        DdlPreparedPack.getStringLikerPack(bean.getTemplateCode(), "templateCode", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getTemplateName(), "templateName", packBean);

    }

    @Override
    public ListPager queryMesLabelTemplateAndParamListByPager(MesLabelTemplate labelTemplate, Pager pager) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(labelTemplate.getOrganizeCode());
        DdlPreparedPack.getStringLikerPack(labelTemplate.getTemplateCode(), "templateCode", packBean);
        pager = PagerHelper.getPager(pager, baseRDao.findByHqlWhereCount(packBean));

        List<MesLabelTemplate> resultList = baseRDao.findByHqlWherePage(packBean, pager);
        if (!CollectionUtils.isEmpty(resultList)){
            for (MesLabelTemplate mesLabelTemplate : resultList) {
                // 根据模板代码查询模板参数信息
                List<MesLabelTemplateParam> templateParamList = mesLabelTemplateParamRDao.findByProperty(
                        new String[]{"templateCode", MesPcnConstWords.IS_VALID, MesPcnConstWords.IS_DELETED, MesExtConstWords.ORGANIZE_CODE},
                        new Object[]{mesLabelTemplate.getTemplateCode(), CommonEnumUtil.IS_VAILD.VAILD.getValue(),
                                CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(),labelTemplate.getOrganizeCode()});

                if (!CollectionUtils.isEmpty(templateParamList)) {
                    mesLabelTemplate.setLabelTemplateParamList(templateParamList);
                }
            }
        }
        return new ListPager<>(resultList, pager);
    }

    @Override
    public MesLabelTemplate getLabelTemplate(String templateCode, String organizeCode) {
        if(StringUtil.isEmpty(templateCode) || StringUtil.isEmpty(organizeCode)) return null;
        DdlPackBean templatePackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(templateCode, MesExtConstWords.TEMPLATE_CODE, templatePackBean);
        MesLabelTemplate mesLabelTemplate = baseRDao.getByProperty(templatePackBean);
        if (mesLabelTemplate != null) {
            //级联获取labelTemplateParam
            DdlPackBean templateParamPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(mesLabelTemplate.getTemplateCode(), "templateCode", templateParamPackBean);
            List<MesLabelTemplateParam> params = mesLabelTemplateParamRDao.findByHqlWhere(templateParamPackBean);
            mesLabelTemplate.setLabelTemplateParamList(params);
        }
        return mesLabelTemplate;
    }

    @Override
    public List<MesLabelTemplateFieldInfo> queryMesLabelTemplateParamField(MesLabelTemplateFieldInfo request) {
        List<MesLabelTemplateFieldInfo> result = new ArrayList<>();
        // 根据模板代码查询模板参数信息
        List<MesLabelTemplateParam> templateParamList = mesLabelTemplateParamRDao.findByProperty(
                new String[]{"templateCode", MesPcnConstWords.IS_VALID, MesPcnConstWords.IS_DELETED, MesExtConstWords.ORGANIZE_CODE},
                new Object[]{request.getTemplateCode(), CommonEnumUtil.IS_VAILD.VAILD.getValue(),
                        CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(),request.getOrganizeCode()});

        if (CollectionUtils.isEmpty(templateParamList)) {
            return result;
        }

        Map<String, MesLabelTemplateParam> templateParamMap = new TreeMap<>();
        for (MesLabelTemplateParam param : templateParamList) {
            templateParamMap.putIfAbsent(param.getTemplateParam(), param);
        }
        List<MesLabelTemplateFieldInfo> templateFieldInfos = labelTemplateFieldInfoRDao.findByProperty(
                new String[]{"templateCode", MesPcnConstWords.IS_VALID, MesPcnConstWords.IS_DELETED, MesExtConstWords.ORGANIZE_CODE},
                new Object[]{request.getTemplateCode(), CommonEnumUtil.IS_VAILD.VAILD.getValue(),
                        CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(),request.getOrganizeCode()});
        if (!CollectionUtils.isEmpty(templateFieldInfos)) {
            for (MesLabelTemplateFieldInfo templateFieldInfo : templateFieldInfos) {
                if (StringUtil.isEmpty(templateFieldInfo.getTemplateParam())) {
                    continue;
                }
                if (templateParamMap.remove(templateFieldInfo.getTemplateParam()) != null) {
                    result.add(templateFieldInfo);
                }
            }
        }
        for (Map.Entry<String, MesLabelTemplateParam> entry : templateParamMap.entrySet()) {
            MesLabelTemplateParam param = entry.getValue();
            MesLabelTemplateFieldInfo fieldInfo = new MesLabelTemplateFieldInfo();
            fieldInfo.setTemplateId(param.getTemplateId());
            fieldInfo.setTemplateParam(param.getTemplateParam());
            fieldInfo.setTemplateCode(param.getTemplateCode());
            fieldInfo.setOrganizeCode(param.getOrganizeCode());
            result.add(fieldInfo);
        }
        return result;
    }

    @Override
    public void saveMesLabelTemplateParamField(List<MesLabelTemplateFieldInfo> templateFieldInfos) {
        if (CollectionUtils.isEmpty(templateFieldInfos)) {
            return;
        }

        MesLabelTemplateFieldInfo request = templateFieldInfos.get(0);
        final String username = request.getCreateUser();
        if (StringUtil.isEmpty(username)) {
            MesException.throwMesBusiException("用户名不能为空");
        }
        Map<String, MesLabelTemplateFieldInfo> templateFieldInfoMap = new TreeMap<>();
        for (MesLabelTemplateFieldInfo info : templateFieldInfos) {
            templateFieldInfoMap.putIfAbsent(info.getTemplateParam(), info);
        }

        List<MesLabelTemplateFieldInfo> saveFieldInfos = new ArrayList<>();
        List<MesLabelTemplateFieldInfo> oldFieldInfos = labelTemplateFieldInfoRDao.findByProperty(
                new String[]{"templateCode", MesExtConstWords.ORGANIZE_CODE},
                new Object[]{request.getTemplateCode(), request.getOrganizeCode()});
        if (!CollectionUtils.isEmpty(oldFieldInfos)) {
            for (MesLabelTemplateFieldInfo oldFieldInfo : oldFieldInfos) {
                if (StringUtil.isEmpty(oldFieldInfo.getTemplateParam())) {
                    oldFieldInfo.setIsValid(CommonEnumUtil.IS_VAILD.INVAILD.getValue());
                    oldFieldInfo.setRemark("模板参数字段名为空");
                    saveFieldInfos.add(oldFieldInfo);
                    continue;
                }
                MesLabelTemplateFieldInfo templateFieldInfo = templateFieldInfoMap.remove(oldFieldInfo.getTemplateParam());
                if (templateFieldInfo != null) {
                    oldFieldInfo.setTemplateId(templateFieldInfo.getTemplateId());
                    oldFieldInfo.setTemplateFieldType(templateFieldInfo.getTemplateFieldType());
                    oldFieldInfo.setTemplateFieldText(templateFieldInfo.getTemplateFieldText());
                    oldFieldInfo.setIsValid(CommonEnumUtil.IS_VAILD.VAILD.getValue());
                    oldFieldInfo.setIsDeleted(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
                } else {
                    oldFieldInfo.setIsValid(CommonEnumUtil.IS_VAILD.INVAILD.getValue());
                    oldFieldInfo.setRemark("模板参数字段被移除");
                }
                ConvertBean.serviceModelUpdate(oldFieldInfo, username);
                saveFieldInfos.add(oldFieldInfo);
            }
        }

        for (Map.Entry<String, MesLabelTemplateFieldInfo> entry : templateFieldInfoMap.entrySet()) {
            MesLabelTemplateFieldInfo fieldInfo = entry.getValue();
            ConvertBean.serviceModelInitialize(fieldInfo, username);
            saveFieldInfos.add(fieldInfo);
        }

        labelTemplateFieldInfoRDao.saveAll(saveFieldInfos);
    }

    @Override
    public List<MesLabelTemplateFieldValue> queryMesLabelTemplateParamPartValue(String partNo, String labelTemplate, String organizeCode) {
        if (StringUtil.isEmpty(labelTemplate)) {
            MesException.throwMesBusiException("零件号【%s】未维护\"零件条码匹配模板\"字段值", partNo);
        }

        List<MesLabelTemplateFieldValue> result = new ArrayList<>();
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partNo, "partNo", ddlPackBean);

        List<MesLabelTemplateFieldInfo> templateFieldInfos = labelTemplateFieldInfoRDao.findByProperty(
                new String[]{"templateCode", "templateFieldType", MesPcnConstWords.IS_VALID, MesPcnConstWords.IS_DELETED, MesExtConstWords.ORGANIZE_CODE},
                new Object[]{labelTemplate, MesExtEnumUtil.LABEL_TEMPLATE_FIELD_TYPE.CUSTOM.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(),
                        CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), organizeCode});
        Map<String, MesLabelTemplateFieldInfo> templateFieldInfoMap = new TreeMap<>();
        for (MesLabelTemplateFieldInfo info : templateFieldInfos) {
            templateFieldInfoMap.putIfAbsent(info.getTemplateParam(), info);
        }

        List<MesLabelTemplateFieldValue> fieldValueList = labelTemplateFieldValueRDao.findByProperty(
                new String[]{"partNo", "templateCode", MesPcnConstWords.IS_VALID, MesPcnConstWords.IS_DELETED, MesExtConstWords.ORGANIZE_CODE},
                new Object[]{partNo, labelTemplate, CommonEnumUtil.IS_VAILD.VAILD.getValue(),
                        CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), organizeCode});
        if (!CollectionUtils.isEmpty(fieldValueList)) {
            for (MesLabelTemplateFieldValue fieldValue : fieldValueList) {
                if (StringUtil.isEmpty(fieldValue.getTemplateParam())) {
                    continue;
                }
                MesLabelTemplateFieldInfo templateFieldInfo = templateFieldInfoMap.remove(fieldValue.getTemplateParam());
                if (templateFieldInfo != null) {
                    MesLabelTemplateFieldValue field = new MesLabelTemplateFieldValue();
                    field.setPartNo(partNo);
                    field.setTemplateCode(templateFieldInfo.getTemplateCode());
                    field.setTemplateParam(templateFieldInfo.getTemplateParam());
                    field.setTemplateFieldText(templateFieldInfo.getTemplateFieldText());
                    field.setFieldValue(fieldValue.getFieldValue());
                    field.setOrganizeCode(organizeCode);
                    result.add(field);
                }
            }
        }

        for (Map.Entry<String, MesLabelTemplateFieldInfo> entry : templateFieldInfoMap.entrySet()) {
            MesLabelTemplateFieldInfo fieldInfo = entry.getValue();
            MesLabelTemplateFieldValue fieldValue = new MesLabelTemplateFieldValue();
            fieldValue.setPartNo(partNo);
            fieldValue.setTemplateCode(fieldInfo.getTemplateCode());
            fieldValue.setTemplateParam(fieldInfo.getTemplateParam());
            fieldValue.setTemplateFieldText(fieldInfo.getTemplateFieldText());
            fieldValue.setOrganizeCode(organizeCode);
            result.add(fieldValue);
        }
        return result;
    }

    @Override
    public void saveMesLabelTemplateParamPartValue(List<MesLabelTemplateFieldValue> templateFieldValues) {
        if (CollectionUtils.isEmpty(templateFieldValues)) {
            return;
        }
        MesLabelTemplateFieldValue request = templateFieldValues.get(0);
        final String username = request.getCreateUser();
        if (StringUtil.isEmpty(username)) {
            MesException.throwMesBusiException("用户名不能为空");
        }
        Map<String, MesLabelTemplateFieldValue> templateFieldInfoMap = new TreeMap<>();
        for (MesLabelTemplateFieldValue value : templateFieldValues) {
            templateFieldInfoMap.putIfAbsent(value.getTemplateParam(), value);
        }

        List<MesLabelTemplateFieldValue> saveFieldValues = new ArrayList<>();
        List<MesLabelTemplateFieldValue> oldFieldValues = labelTemplateFieldValueRDao.findByProperty(
                new String[]{"partNo", "templateCode", MesExtConstWords.ORGANIZE_CODE},
                new Object[]{request.getPartNo(), request.getTemplateCode(), request.getOrganizeCode()});
        if (!CollectionUtils.isEmpty(oldFieldValues)) {
            for (MesLabelTemplateFieldValue oldFieldValue : oldFieldValues) {
                if (StringUtil.isEmpty(oldFieldValue.getTemplateParam())) {
                    oldFieldValue.setIsValid(CommonEnumUtil.IS_VAILD.INVAILD.getValue());
                    oldFieldValue.setRemark("模板参数字段名为空");
                    saveFieldValues.add(oldFieldValue);
                    continue;
                }
                MesLabelTemplateFieldValue templateFieldInfo = templateFieldInfoMap.remove(oldFieldValue.getTemplateParam());
                if (templateFieldInfo != null) {
                    oldFieldValue.setFieldValue(templateFieldInfo.getFieldValue());
                    oldFieldValue.setIsValid(CommonEnumUtil.IS_VAILD.VAILD.getValue());
                    oldFieldValue.setIsDeleted(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
                } else {
                    oldFieldValue.setIsValid(CommonEnumUtil.IS_VAILD.INVAILD.getValue());
                    oldFieldValue.setRemark("模板参数字段被移除");
                }
                ConvertBean.serviceModelUpdate(oldFieldValue, username);
                saveFieldValues.add(oldFieldValue);
            }
        }

        for (Map.Entry<String, MesLabelTemplateFieldValue> entry : templateFieldInfoMap.entrySet()) {
            MesLabelTemplateFieldValue fieldInfo = entry.getValue();
            ConvertBean.serviceModelInitialize(fieldInfo, username);
            saveFieldValues.add(fieldInfo);
        }

        labelTemplateFieldValueRDao.saveAll(saveFieldValues);
    }
}
