package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesOrgService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesOrg;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

/**
 * @Description:  责任人区域关系
 * @Author: gsz
 * @Date: 2024/5/27 19:10
 * @Modify:
 */
@Service
@Slf4j
public class MesOrgServiceImpl extends BaseMesService<MesOrg> implements IMesOrgService {

    @Override
    protected void setPackQueryBean(MesOrg bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getInventoryLocationCode(), "inventoryLocationCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getResponsibleParty(), "responsibleParty", packBean);

    }

    @Override
    protected void onInsertBean(MesOrg item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getOrganizeCode(), "工厂号不能为空");
        ValidatorBean.checkNotNull(item.getInventoryLocationCode(), "inventoryLocationCode 不能为空");
        ValidatorBean.checkNotNull(item.getResponsibleParty(), "responsibleParty 不能为空");

        //唯一性校验
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getInventoryLocationCode(), "inventoryLocationCode", packBean);
        DdlPreparedPack.getStringEqualPack(item.getResponsibleParty(), "responsibleParty", packBean);
        boolean flg = baseRDao.isExitByHql(packBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("区域代码【%s】责任方【%s】已经存在，请检查数据",
                            item.getInventoryLocationCode(), item.getResponsibleParty() )
                    .build();
        }
    }


}
