package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesPartProduceTypeService;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesPartProduceType;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

/**
 * @Description : 产品生产类型
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/6/3 14:41
 * @Modify:
 **/
@Service
@Slf4j
public class MesPartProduceTypeServiceImpl extends BaseMesService<MesPartProduceType> implements IMesPartProduceTypeService {

    @Override
    protected void setPackQueryBean(MesPartProduceType bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getPptCode(), "pptCode", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPptName(), "pptName", packBean);
    }

    @Override
    protected void onInsertBean(MesPartProduceType item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getPptCode(), "产品生产类型不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPptCode(), "pptCode", roadPackBean);

        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            MesException.throwMesBusiException("产品生产类型【%s】已经存在，请检查数据", item.getPptCode());
        }
    }

    @Override
    protected void onUpdateBean(MesPartProduceType item) {

        // 数据校验
        ValidatorBean.checkNotNull(item.getPptCode(), "产品生产类型不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPptCode(), "pptCode", roadPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", roadPackBean);
        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            MesException.throwMesBusiException("产品生产类型【%s】已经存在，请检查数据", item.getPptCode());
        }
    }
}
