package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesPartSapService;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesPartSap;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;

@Service
@Slf4j
public class MesPartSapService extends BaseMesService<MesPartSap> implements IMesPartSapService {

    @Override
    protected void setPackQueryBean(MesPartSap bean, DdlPackBean packBean) {
        DdlPreparedPack.getNumEqualPack(bean.getIsTestPiece(), "isTestPiece", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartNo(), "partNo", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartName(), "partName", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartStatus(), "partStatus", packBean);
    }

    @Override
    protected void onInsertBean(MesPartSap item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getPartNo(), "物料号不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPartNo(), "partNo", roadPackBean);

        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("物料号【%s】已经存在，请检查数据", item.getPartNo())
                    .build();
        }
    }

    @Override
    protected void onUpdateBean(MesPartSap item) {

        // 数据校验
        ValidatorBean.checkNotNull(item.getPartNo(), "物料号不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPartNo(), "partNo", roadPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", roadPackBean);
        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("物料号【%s】已经存在，请检查数据", item.getPartNo())
                    .build();
        }
    }

    @Override
    public MesPartSap getMesPartSapByPartNo(String partNo, String organizeCode) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partNo, "partNo", ddlPackBean);
        MesPartSap mesPart = baseRDao.getByProperty(ddlPackBean);
        if (null == mesPart) {
            MesException.throwMesBusiException("物料【%s】信息不存在", partNo);
        }
        return mesPart;
    }

    @Override
    public List<MesPartSap> findMesPartSapByPartNoList(List<String> partNoList, String organizeCode) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(partNoList, "partNo", ddlPackBean);
        List<MesPartSap> mesPartSapList = baseRDao.findByHqlWhere(ddlPackBean);
        if (CollectionUtils.isEmpty(mesPartSapList)) {
            MesException.throwMesBusiException("物料【%s】信息不存在", partNoList);
        }
        return mesPartSapList;
    }
}
