package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesPartService;
import cn.estsh.i3plus.ext.mes.apiservice.utils.BeanMapUtilsExt;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesPart;
import cn.estsh.i3plus.pojo.mes.bean.MesPartType;
import cn.estsh.i3plus.pojo.mes.repository.MesPartRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesPartTypeRepository;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@Slf4j
public class MesPartService extends BaseMesService<MesPart> implements IMesPartService {

    @Autowired
    private MesPartRepository mesPartRepository;

    @Autowired
    private MesPartTypeRepository mesPartTypeRepository;

    protected void setPackQueryBean(MesPart bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringLikerPack(bean.getPartNo(), "partNo", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartName(), "partName", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartTypeCode(), "partTypeCode", packBean);

    }

    @Override
    public void setBeanModel(List<MesPart> list) {
        if(CollectionUtils.isEmpty(list)){
            return;
        }
        List<String> typeCodeList = list.stream().map(MesPart::getPartTypeCode).filter(t -> !StringUtils.isEmpty(t)).distinct().collect(Collectors.toList());
        Map<String, MesPartType> beanMap = CollectionUtils.isEmpty(typeCodeList) ? null : BeanMapUtilsExt.getBeanMap(mesPartTypeRepository, "partTypeCode", list.iterator().next().getOrganizeCode(), typeCodeList);
        if(!CollectionUtils.isEmpty(beanMap)){
            list.forEach(t->{
                if (beanMap.containsKey(t.getPartTypeCode())) {
                    t.setPartTypeName(beanMap.get(t.getPartTypeCode()).getPartTypeName());
                }
            });
        }
    }

    @Override
    public MesPart getPartByPartNo(String partNo, String organizeCode) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partNo, "partNo", ddlPackBean);
        MesPart mesPart = baseRDao.getByProperty(ddlPackBean);
        if (null == mesPart) {
            MesException.throwMesBusiException("物料【%s】信息不存在", partNo);
        }
        return mesPart;
    }

    @Override
    public List<MesPart> findMesPartByPartNoList(List<String> partNoList, String organizeCode) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(partNoList, "partNo", ddlPackBean);
        List<MesPart> mesPartList = baseRDao.findByHqlWhere(ddlPackBean);
        if (CollectionUtils.isEmpty(mesPartList)) {
            MesException.throwMesBusiException("物料【%s】信息不存在", partNoList);
        }
        return mesPartList;
    }

    @Override
    public ListPager<MesPart> queryMesPartByPagerExt(MesPart mesPart, Pager pager) {
        DdlPackBean partQueryPackBean = DdlPackBean.getDdlPackBean(mesPart.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(mesPart.getIsValid(), "isValid", partQueryPackBean);
        DdlPreparedPack.getStringLikerPack(mesPart.getPartNo(), "partNo", partQueryPackBean);
        DdlPreparedPack.getStringLikerPack(mesPart.getPartName(), "partName", partQueryPackBean);
        pager = PagerHelper.getPager(pager, mesPartRepository.findByHqlWhereCount(partQueryPackBean));
        List<MesPart> mesPartList = mesPartRepository.findByHqlWherePage(partQueryPackBean, pager);
        return new ListPager<>(mesPartList, pager);
    }

    protected void onInsertBean(MesPart item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getPartNo(), "物料号不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPartNo(), "partNo", roadPackBean);

        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("物料号【%s】已经存在，请检查数据", item.getPartNo())
                    .build();
        }
    }

    protected void onUpdateBean(MesPart item) {

        // 数据校验
        ValidatorBean.checkNotNull(item.getPartNo(), "物料号不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPartNo(), "partNo", roadPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", roadPackBean);
        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("物料号【%s】已经存在，请检查数据", item.getPartNo())
                    .build();
        }
    }
}
