package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesPartShippingGroupDetailService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesPartProdGroup;
import cn.estsh.i3plus.pojo.mes.bean.MesPartShippingGroup;
import cn.estsh.i3plus.pojo.mes.bean.MesPartShippingGroupDetail;
import cn.estsh.i3plus.pojo.mes.repository.MesCustPartInvoicedRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesCustomerPartRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesPartShippingGroupRepository;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Slf4j
public class MesPartShippingGroupDetailService extends BaseMesService<MesPartShippingGroupDetail> implements IMesPartShippingGroupDetailService {
    @Autowired
    private MesCustPartInvoicedRepository mesCustPartInvoicedRao;

    @Autowired
    private MesCustomerPartRepository customerPartRao;
    @Autowired
    private MesPartShippingGroupRepository shippingGroupRao;

    private boolean check(String organizeCode,String custPartNo,String shippingGroupCode){
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(shippingGroupCode,"partProdGroupCode", ddlPackBean);
        List<MesPartShippingGroup> shippingGroupList = shippingGroupRao.findByHqlTopWhere(ddlPackBean, 1);
        if (shippingGroupList == null || shippingGroupList.isEmpty()) {
            return false;
        }
        String custCode = shippingGroupList.get(0).getCustCode();
        DdlPackBean ddlPackBean1 = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(custCode,"custCode",ddlPackBean1);
        DdlPreparedPack.getStringEqualPack(custPartNo,"custPartNo",ddlPackBean1);

        int count = mesCustPartInvoicedRao.findByHqlWhereCount(ddlPackBean1);
        if (count > 0) {
            return true;
        }else {
            DdlPackBean custPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(custCode,"custCode",custPackBean);
            DdlPreparedPack.getStringEqualPack(custPartNo,"custPartNo",custPackBean);
            int count1 = customerPartRao.findByHqlWhereCount(custPackBean);
            return count1 > 0;
        }
    }

    @Override
    protected void onInsertBean(MesPartShippingGroupDetail item) {
        boolean check = check(AuthUtil.getOrganizeCode(), item.getCustPartNo(), item.getShippingGroupCode());
        if (!check) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("待开票表及客户零件表不存在客户零件号【%s】，请检查数据",
                            item.getCustPartNo())
                    .build();
        }
    }

    protected void setPackQueryBean(MesPartShippingGroupDetail bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getShippingGroupCode(), "shippingGroupCode", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartNo(), "partNo", packBean);
    }
    @Override
    public void validateImport(List<MesPartShippingGroupDetail> beanList) {

        for (MesPartShippingGroupDetail item : beanList) {
            // 数据校验
            ValidatorBean.checkNotNull(item.getShippingGroupCode(), "发运组代码不能为空");
            ValidatorBean.checkNotNull(item.getCustPartNo(), "客户零件号不能为空");


            DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(item.getShippingGroupCode(), "shippingGroupCode", seriesPackBean);
            DdlPreparedPack.getStringEqualPack(item.getCustPartNo(), "custPartNo", seriesPackBean);
            boolean flg = baseRDao.isExitByHql(seriesPackBean);
            if (flg) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("【%s】发运组代码已经存在，请检查数据", item.getShippingGroupCode(), item.getShippingGroupCode())
                        .build();
            }
        }
    }
}
