package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesPartTransferService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.repository.MesCustPartInvoicedRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesCustomerPartRepository;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
@Slf4j
public class MesPartTransferService extends BaseMesService<MesPartTransfer> implements IMesPartTransferService {


    @Override
    protected void setPackQueryBean(MesPartTransfer bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringLikerPack(bean.getCustCode(), "custCode", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getCustOrganizeCode(), "custOrganizeCode", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getMessageNo(), "messageNo", packBean);
    }
    @Override
    protected void onInsertBean(MesPartTransfer item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getOrganizeCode(), "工厂号不能为空");
        ValidatorBean.checkNotNull(item.getCustCode(), "custCode不能为空" );
        ValidatorBean.checkNotNull(item.getCustOrganizeCode(), "custOrganizeCode不能为空" );
        ValidatorBean.checkNotNull(item.getMessageNo(), "messageNo不能为空");

        //唯一性校验
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getCustCode(), "custCode", packBean);
        DdlPreparedPack.getStringEqualPack(item.getCustOrganizeCode(), "custOrganizeCode", packBean);
        DdlPreparedPack.getStringEqualPack(item.getMessageNo(), "messageNo", packBean);

        boolean flg = baseRDao.isExitByHql(packBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("客户零件转化配置客户代码【%s】信息点代码【%s】已经存在，请检查数据",
                            item.getCustCode(),item.getMessageNo() )
                    .build();
        }
    }


    @Override
    protected void onUpdateBean(MesPartTransfer item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getOrganizeCode(), "工厂号不能为空");
        ValidatorBean.checkNotNull(item.getCustCode(), "custCode不能为空" );
        ValidatorBean.checkNotNull(item.getCustOrganizeCode(), "custOrganizeCode不能为空" );
        ValidatorBean.checkNotNull(item.getMessageNo(), "messageNo不能为空");

        //唯一性校验
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getCustCode(), "custCode", packBean);
        DdlPreparedPack.getStringEqualPack(item.getCustOrganizeCode(), "custOrganizeCode", packBean);
        DdlPreparedPack.getStringEqualPack(item.getMessageNo(), "messageNo", packBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", packBean);
        boolean flg = baseRDao.isExitByHql(packBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("客户零件转化配置客户代码【%s】信息点代码【%s】已经存在，请检查数据",
                            item.getCustCode(),item.getMessageNo() )
                    .build();
        }
    }
}
