package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesPrinterConfigureService;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesPrinterConfigure;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

/**
 * @Description : 打印机配置表
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/6/13 11:19
 * @Modify:
 **/
@Service
@Slf4j
public class MesPrinterConfigureServiceImpl extends BaseMesService<MesPrinterConfigure> implements IMesPrinterConfigureService {

    @Override
    protected void setPackQueryBean(MesPrinterConfigure bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getPrinterCode(), "printerCode", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPrinterName(), "printerName", packBean);
    }

    @Override
    protected void onInsertBean(MesPrinterConfigure item) {
        // 数据校验
        checkNotNull(item);

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPrinterCode(), "printerCode", packBean);

        if (baseRDao.isExitByHql(packBean)) {
            MesException.throwMesBusiException("打印机编号【%s】数据已存在", item.getPrinterCode());
        }
    }

    @Override
    protected void onUpdateBean(MesPrinterConfigure item) {
        // 数据校验
        checkNotNull(item);

        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPrinterCode(), "printerCode", ddlPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", ddlPackBean);

        if (baseRDao.isExitByHql(ddlPackBean)) {
            MesException.throwMesBusiException("打印机编号【%s】数据已存在", item.getPrinterCode());
        }
    }

    private void checkNotNull(MesPrinterConfigure item) {
        ValidatorBean.checkNotNull(item.getPrinterCode(), "打印机编号不能为空");
        ValidatorBean.checkNotNull(item.getPrinterName(), "打印机名称不能为空");
    }
}
