package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesEquitmentService;
import cn.estsh.i3plus.ext.mes.api.base.IMesPartSapService;
import cn.estsh.i3plus.ext.mes.api.base.IMesProdRuleNosortCfgService;
import cn.estsh.i3plus.ext.mes.apiservice.dao.IMesProdRuleNosortCfgDao;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipment;
import cn.estsh.i3plus.pojo.mes.bean.MesPartSap;
import cn.estsh.i3plus.pojo.mes.bean.MesProdRuleNosortCfg;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@Slf4j
public class MesProdRuleNosortCfgService extends BaseMesService<MesProdRuleNosortCfg> implements IMesProdRuleNosortCfgService {

    @Autowired
    private IMesProdRuleNosortCfgDao mesProdRuleNosortCfgDao;

    @Autowired
    private IMesPartSapService partSapService;

    @Autowired
    private IMesEquitmentService equitmentService;

    @Override
    protected void setPackQueryBean(MesProdRuleNosortCfg bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringLikerPack(bean.getInPartNo(), "inPartNo", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getInPartNoRule(), "inPartNoRule", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getOutPartNo(), "outPartNo", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getEquipmentCode(), "equipmentCode", packBean);

    }

    @Override
    protected void setBeanModel(List<MesProdRuleNosortCfg> prodRuleNosortCfgList) {
        if (CollectionUtils.isEmpty(prodRuleNosortCfgList)) return;
        List<MesEquipment> equipmentList = equitmentService.findAll(DdlPackBean.getDdlPackBean(prodRuleNosortCfgList.get(0).getOrganizeCode()));
        Map<String, List<MesEquipment>> equipmentCodeMap = CollectionUtils.isEmpty(equipmentList) ? null :
                equipmentList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getEquipmentCode()))).collect(Collectors.groupingBy(MesEquipment::getEquipmentCode));
        if (CollectionUtils.isEmpty(equipmentCodeMap)) return;
        prodRuleNosortCfgList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getEquipmentCode()))).forEach(o -> {
            if (equipmentCodeMap.containsKey(o.getEquipmentCode())) o.setEquipmentName(equipmentCodeMap.get(o.getEquipmentCode()).get(0).getEquipmentName());
        });
    }

    @Override
    public MesProdRuleNosortCfg insert(MesProdRuleNosortCfg bean) {
        //插入前构造并校验数据
        onInsertBean(bean);

        bean = baseRDao.insert(bean);
        return bean;
    }

    protected void onInsertBean(MesProdRuleNosortCfg item) {
        ValidatorBean.checkNotNull(item.getOutPartNo(), "产成零件号不能为空");
        ValidatorBean.checkNotNull(item.getEquipmentCode(), "设备不能为空");

        if (!StringUtils.isEmpty(item.getInPartNo())) {
            MesPartSap inPart = partSapService.getMesPartSapByPartNo(item.getInPartNo(), item.getOrganizeCode());
            if (inPart == null) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("进料零件号不存在，请检查数据")
                        .build();
            }
        }
        if (!StringUtils.isEmpty(item.getOutPartNo())) {
            MesPartSap outPart = partSapService.getMesPartSapByPartNo(item.getOutPartNo(),item.getOrganizeCode());
            if (outPart == null) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("产成零件号不存在，请检查数据")
                        .build();
            }
        }
    }

    protected void onUpdateBean(MesProdRuleNosortCfg item) { onInsertBean(item); }

    @Override
    public MesProdRuleNosortCfg findMesProdRuleNosortCfgByWorkCenterCode(String workCenterCode, String partNo, String organizeCode) {
        List<MesProdRuleNosortCfg> prodRuleNosortCfgList = mesProdRuleNosortCfgDao.findMesProdRuleNosortCfgByWorkCenterCode(workCenterCode, partNo, organizeCode);
        if(CollectionUtils.isEmpty(prodRuleNosortCfgList)){
            MesException.throwMesBusiException("产线【%s】产出零件【%s】对应的非排序产品加工规则配置不存在",workCenterCode,partNo);
        }
        return prodRuleNosortCfgList.iterator().next();
    }

    @Override
    public void validateImport(List<MesProdRuleNosortCfg> beanList) {

        for (MesProdRuleNosortCfg item : beanList) {
            // 数据校验
            ValidatorBean.checkNotNull(item.getOutPartNo(), "产成零件号不能为空");
            ValidatorBean.checkNotNull(item.getEquipmentCode(), "设备不能为空");
        }
    }
}
