package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesProductVersionService;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesProductVersion;
import cn.estsh.i3plus.pojo.mes.repository.MesProductVersionRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Slf4j
public class MesProductVersionService extends BaseMesService<MesProductVersion> implements IMesProductVersionService {

    @Autowired
    private MesProductVersionRepository mesProductVersionRepository;

    @Override
    protected void setPackQueryBean(MesProductVersion bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringLikerPack(bean.getPartNo(),"partNo",packBean);
        DdlPreparedPack.getStringEqualPack(bean.getWorkCenterCode(),"workCenterCode",packBean);
        DdlPreparedPack.getStringEqualPack(bean.getProductVersion(),"productVersion",packBean);
        DdlPreparedPack.getStringEqualPack(bean.getShipInventoryPoint(),"shipInventoryPoint",packBean);
        DdlPreparedPack.getStringEqualPack(bean.getReceiveInventoryPoint(),"receiveInventoryPoint",packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartName(),"partName",packBean);
    }

    @Override
    public ListPager<MesProductVersion> queryMesProductVersionByPagerExt(MesProductVersion mesProductVersion, Pager pager) {
        DdlPackBean productVersionQueryPackBean = DdlPackBean.getDdlPackBean(mesProductVersion.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(mesProductVersion.getIsValid(), "isValid", productVersionQueryPackBean);
        DdlPreparedPack.getStringLikerPack(mesProductVersion.getProductVersion(), "productVersion", productVersionQueryPackBean);
        DdlPreparedPack.getStringLikerPack(mesProductVersion.getPartNo(), "partNo", productVersionQueryPackBean);
        DdlPreparedPack.getStringLikerPack(mesProductVersion.getWorkCenterCode(), "workCenterCode", productVersionQueryPackBean);
        DdlPreparedPack.getStringLikerPack(mesProductVersion.getShipInventoryPoint(), "shipInventoryPoint", productVersionQueryPackBean);
        DdlPreparedPack.getStringLikerPack(mesProductVersion.getReceiveInventoryPoint(), "receiveInventoryPoint", productVersionQueryPackBean);
        pager = PagerHelper.getPager(pager, mesProductVersionRepository.findByHqlWhereCount(productVersionQueryPackBean));
        List<MesProductVersion> mesProductVersions = mesProductVersionRepository.findByHqlWherePage(productVersionQueryPackBean, pager);
        return new ListPager<>(mesProductVersions, pager);
    }

    @Override
    public MesProductVersion getMesProductVersionByPartNoAndProductVersion(String organizeCode, String partNo, String productVersion) {
        if (StringUtil.isEmpty(organizeCode) || StringUtil.isEmpty(partNo)) return null;
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partNo, "partNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(productVersion, "productVersion", ddlPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, ddlPackBean);
        return baseRDao.getByProperty(ddlPackBean);
    }
}
