package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesProductionRecordService;
import cn.estsh.i3plus.ext.mes.apiservice.dao.IMesProdRuleNosortCfgDao;
import cn.estsh.i3plus.ext.mes.apiservice.utils.BeanUtil;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.ext.mes.pojo.model.MesLoadingListDetailModel;
import cn.estsh.i3plus.ext.mes.pojo.model.MesShippingOrderManagementDetailModel;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesProdRuleNosortCfg;
import cn.estsh.i3plus.pojo.mes.bean.MesProduceSn;
import cn.estsh.i3plus.pojo.mes.bean.MesProductRecordLog;
import cn.estsh.i3plus.pojo.mes.bean.MesProductionRecord;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesLoadingListDetail;
import cn.estsh.i3plus.pojo.mes.model.MesProductionRecordModel;
import cn.estsh.i3plus.pojo.mes.model.report.MesProductAccountModel;
import cn.estsh.i3plus.pojo.mes.model.report.MesProductAccountResModel;
import cn.estsh.i3plus.pojo.mes.repository.MesProduceSnRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesProductRecordLogRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import com.google.common.base.Objects;
import com.google.common.collect.Lists;
import com.xxl.job.core.util.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.hibernate.SQLQuery;
import org.hibernate.query.internal.NativeQueryImpl;
import org.hibernate.transform.Transformers;
import org.mvel2.util.Make;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Description : 设备加工记录表
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/6/28 9:27
 * @Modify:
 **/
@Service
@Slf4j
public class MesProductionRecordServiceImpl extends BaseMesService<MesProductionRecord> implements IMesProductionRecordService {
    @Autowired
    private EntityManager entityManager;

    @Autowired
    private IMesProdRuleNosortCfgDao mesProdRuleNosortCfgDao;

    @Autowired
    private MesProduceSnRepository mesProduceSnRepository;

    @Autowired
    private MesProductRecordLogRepository mesProductRecordLogRepository;

    @Override
    public List<MesProductionRecord> findMesProductionRecord(String organizeCode, String sn) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(sn, "productSn", packBean);
        DdlPreparedPack.getOrderBy("createDatetime", CommonEnumUtil.ASC_OR_DESC.ASC.getValue(), packBean);
        List<MesProductionRecord> produceSnList = baseRDao.findByHqlWhere(packBean);
        if (CollectionUtils.isEmpty(produceSnList)) {
            MesException.throwMesBusiException("条码【%s】加工记录信息不存在", sn);
        }
        return produceSnList;
    }

    @Override
    public ListPager queryRecordGroupByPartNoWorkCellCode(MesProductionRecord mesProductionRecord, Pager pager) {

        return queryERPMaterialStockHqlPager(mesProductionRecord, pager);
    }

    @Override
    public ListPager<MesProductionRecordModel> queryMesProductionOffLine(MesProductionRecordModel mesProductionRecordModel, Pager pager) {
        List<MesProdRuleNosortCfg> mesProdRuleNosortCfgs = mesProdRuleNosortCfgDao.findMesProdRuleNosortCfgByWorkCellCode(mesProductionRecordModel.getOnLineWorkCellCode(), mesProductionRecordModel.getPartNo(), mesProductionRecordModel.getOrganizeCode());
        if(CollectionUtils.isEmpty(mesProdRuleNosortCfgs) || StringUtil.isEmpty(mesProdRuleNosortCfgs.iterator().next().getOutPartNo())) return new ListPager<>(new ArrayList<>(),pager);
        //筛选下线工位的当前零件的产成条码
        List<MesProduceSn> mesProduceSnList = getMesProduceSnList(mesProductionRecordModel);
        if(CollectionUtils.isEmpty(mesProduceSnList)) return new ListPager<>(new ArrayList<>(),pager);
        //在上线工位消耗进料条码且没有报工
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(mesProductionRecordModel.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesProdRuleNosortCfgs.iterator().next().getOutPartNo(),"partNo",packBean);
        DdlPreparedPack.getStringEqualPack(mesProductionRecordModel.getOnLineWorkCellCode(),"workCellCode",packBean);
        DdlPreparedPack.getInPackList(mesProduceSnList.stream().map(MesProduceSn::getProductSn).distinct().collect(Collectors.toList()),"productSn",packBean);
        List<MesProductionRecord> productionRecordList = baseRDao.findByHqlWhere(packBean);
        Map<String,MesProductionRecord> productionRecordMap = new HashMap<>();
        if(!CollectionUtils.isEmpty(productionRecordList)){
            for (MesProductionRecord productionRecord : productionRecordList) {
                productionRecordMap.put(productionRecord.getProductSn(),productionRecord);
            }
        }
        //数据组装
        List<MesProductionRecordModel> mesProductionRecordModelList = new ArrayList<>();
        MesProductionRecordModel model;
        for (MesProduceSn mesProduceSn : mesProduceSnList) {
            model = new MesProductionRecordModel();
            BeanUtils.copyProperties(mesProduceSn,model);
            if(!productionRecordMap.containsKey(mesProduceSn.getProductSn()) || (productionRecordMap.containsKey(mesProduceSn.getProductSn()) && MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_10.getValue() == productionRecordMap.get(mesProduceSn.getProductSn()).getReportStatus())){
                model.setOffLineWorkCellCode(mesProduceSn.getWorkCellCode());
                mesProductionRecordModelList.add(model);
            }
        }
        pager = PagerHelper.getPager(pager, mesProductionRecordModelList.size());
        return new ListPager<>(mesProductionRecordModelList.subList(pager.getStartRow(),
                Math.min(pager.getPageSize() * pager.getCurrentPage(), mesProductionRecordModelList.size())), pager);
    }



    @Override
    public ListPager<MesProductAccountResModel> queryProductAccount(MesProductAccountModel productAccountModel, Pager pager) {
        List<MesProductAccountResModel> resList = new ArrayList<>();

        String sqlStr = "select mpr.part_no,mpr.equipment_code,mpr.equipment_name,mpr.work_cell_code,mpr.work_center_code," +
                "mpr.product_count,mpr.detail_count,error_reason from (\n" +
                "\n" +
                "SELECT\n" +
                "\tmpr.part_no ,\n" +
                "\tmpr.equipment_code ,\n" +
                "\tme.equipment_name ,\n" +
                "\tme.work_cell_code ,\n" +
                "\tme.work_center_code ,\n" +
                "\tmpr.product_count ,\n" +
                "\tmpr.detail_count , \n" +
                "\t null error_reason\n" +
                "\t\n" +
                "FROM\n" +
                "\t(\n" +
                "\tSELECT\n" +
                "\t\tequipment_code,\n" +
                "\t\tpart_no,\n" +
                "\t\tsum( CASE WHEN type = 'record' THEN 1 ELSE 0 END ) product_count,\n" +
                "\t\tsum( CASE WHEN type = 'logdetail' THEN 1 ELSE 0 END ) detail_count \n" +
                "\tFROM\n" +
                "\t\tmes_product_record_log \n" +
                "\tWHERE\n" +
                "\t\tequipment_code LIKE '%HT%' \n";

        if (!StringUtils.isEmpty(productAccountModel.getEquipmentCode())) {
            sqlStr += " and  equipment_code = :equipmentCode ";
        } if (!StringUtils.isEmpty(productAccountModel.getPartNo())) {
            sqlStr += " and  part_no=:partNo ";
        } if (!StringUtils.isEmpty(productAccountModel.getStartTime())) {
            sqlStr += " and create_date_time >= :startTime ";
        } if (!StringUtils.isEmpty(productAccountModel.getEndTime())) {
            sqlStr += " and create_date_time < :endTime ";
        }


        sqlStr +=
                "\t\tAND part_no IS NOT NULL \n" +
                "\tGROUP BY\n" +
                "\t\tequipment_code,\n" +
                "\t\tpart_no \n" +
                "\t) mpr\n" +
                "\tLEFT JOIN mes_wc_equipment me ON mpr.equipment_code = me.equipment_code \n" +
                "\n" +
                "\n" +
                "UNION\n" +
                "SELECT\n" +
                "\tmprnc.out_part_no part_no,\n" +
                "\tmprnc.equipment_code ,\n" +
                "\tme.equipment_name ,\n" +
                "\tme.work_cell_code ,\n" +
                "\tme.work_center_code ,\n" +
                "\teqcount.product_count ,\n" +
                "\teqcount.detail_count , \n" +
                 "\t null error_reason \n" +
                "FROM\n" +
                "\tmes_prod_rule_nosort_cfg mprnc\n" +
                "\tLEFT JOIN mes_wc_equipment me ON mprnc.equipment_code = me.equipment_code and me.is_deleted = 2 and me.is_valid = 1 \n" +
                "\tLEFT JOIN (\n" +
                "\tSELECT\n" +
                "\t\tequipment_code,\n" +
                "\t\tpart_no,\n" +
                "\t\tsum( CASE WHEN type = 'record' THEN 1 ELSE 0 END ) product_count,\n" +
                "\t\tsum(\n" +
                "\t\tCASE\n" +
                "\t\t\t\t\n" +
                "\t\t\t\tWHEN type = 'logdetail' \n" +
                "\t\t\t\tAND CATEGORY_LEVEL_TWO IN ( 'Product_finish', 'PART_OK', 'Part_Finish', 'Product_OK' ) \n" +
                "\t\t\t\tAND EQUIP_VARIABLE_VALUE = 'True' THEN\n" +
                "\t\t\t\t\t1 ELSE 0 \n" +
                "\t\t\t\tEND \n" +
                "\t\t\t\t) detail_count \n" +
                "\t\t\tFROM\n" +
                "\t\t\t\tmes_product_record_log \n" +
                "\t\t\tWHERE  1 = 1 \n";
        if (!StringUtils.isEmpty(productAccountModel.getEquipmentCode())) {
            sqlStr += "\t\t\t and  equipment_code = :equipmentCode \n";
        } if (!StringUtils.isEmpty(productAccountModel.getPartNo())) {
            sqlStr += "\t\t\t and  part_no= :partNo \n";
        } if (!StringUtils.isEmpty(productAccountModel.getStartTime())) {
            sqlStr += "\t\t\t and create_date_time >= :startTime \n";
        } if (!StringUtils.isEmpty(productAccountModel.getEndTime())) {
            sqlStr += "\t\t\t and create_date_time < :endTime \n";
        }
        sqlStr +=
                "\t\t\t    and \n" +
                "\t\t\t\tpart_no IS NOT NULL \n" +
                "\t\t\tGROUP BY\n" +
                "\t\t\t\tequipment_code,\n" +
                "\t\t\t\tpart_no \n" +
                "\t\t\t) eqcount ON mprnc.equipment_code = eqcount.equipment_code  \n" +
                "\t AND mprnc.out_part_no = eqcount.part_no where 1=1 ";
        if (!StringUtils.isEmpty(productAccountModel.getEquipmentCode())) {
            sqlStr += "and mprnc.equipment_code= :equipmentCode ";
        }
        sqlStr += "\tand eqcount.product_count is not null and eqcount.detail_count is not null ) mpr \n" +
                "\t\n" +
                "\tleft join mes_equipment me\n" +
                "\ton mpr.equipment_code = me.equipment_code\n" +
                "\n" +
                "where mpr.product_count != 0 and mpr.detail_count != 0\n" +
                "and me.equipment_type = 10";


        Query query = entityManager.createNativeQuery(sqlStr);
        if (!StringUtils.isEmpty(productAccountModel.getEquipmentCode())) {
            query.setParameter("equipmentCode", productAccountModel.getEquipmentCode());
        }
        if (!StringUtils.isEmpty(productAccountModel.getPartNo())) {
            query.setParameter("partNo", productAccountModel.getPartNo());
        }
        if (!StringUtils.isEmpty(productAccountModel.getStartTime())) {
            query.setParameter("startTime", productAccountModel.getStartTime());
        }
        if (!StringUtils.isEmpty(productAccountModel.getEndTime())) {
            query.setParameter("endTime", productAccountModel.getEndTime());
        }

       /* query.unwrap(NativeQueryImpl.class).setResultTransformer(Transformers.aliasToBean(MesProductAccountResModel.class));
        List<MesProductAccountResModel> results = query.getResultList();*/
        List results = query.getResultList();
        for (Object result : results) {
            Object[] cells = (Object[]) result;
            MesProductAccountResModel model = new MesProductAccountResModel();
            model.setPartNo(StringUtils.isEmpty(cells[0]) ? "" : String.valueOf(cells[0]));
            model.setEquipmentCode(StringUtils.isEmpty(cells[1]) ? "" : String.valueOf(cells[1]));
            model.setWorkCellCode(StringUtils.isEmpty(cells[3]) ? "" : String.valueOf(cells[3]));
            model.setWorkCenterCode(StringUtils.isEmpty(cells[4]) ? "" : String.valueOf(cells[4]));
            model.setProductCount(StringUtils.isEmpty(cells[5]) ? "" : String.valueOf(cells[5]));
            model.setDetailCount(StringUtils.isEmpty(cells[6]) ? "" : String.valueOf(cells[6]));
            model.setErrorReason(StringUtils.isEmpty(cells[7]) ? "" : String.valueOf(cells[7]));

            resList.add(model);
        }

        //query.unwrap(NativeQueryImpl.class).setResultTransformer(Transformers.aliasToBean(MesProductAccountResModel.class));
        //List<MesProductAccountResModel> results = query.getResultList();



        return new ListPager<>(resList,pager);
    }

    @Override
    public List<MesProductAccountResModel> queryProductAccount(MesProductAccountModel productAccountModel) {
        List<MesProductAccountResModel> resList = new ArrayList<>();

        String sqlStr = "select mpr.part_no,mpr.equipment_code,mpr.equipment_name,mpr.work_cell_code,mpr.work_center_code," +
                "mpr.product_count,mpr.detail_count,error_reason from (\n" +
                "\n" +
                "SELECT\n" +
                "\tmpr.part_no ,\n" +
                "\tmpr.equipment_code ,\n" +
                "\tme.equipment_name ,\n" +
                "\tme.work_cell_code ,\n" +
                "\tme.work_center_code ,\n" +
                "\tmpr.product_count ,\n" +
                "\tmpr.detail_count , \n" +
                "\t null error_reason\n" +
                "\t\n" +
                "FROM\n" +
                "\t(\n" +
                "\tSELECT\n" +
                "\t\tequipment_code,\n" +
                "\t\tpart_no,\n" +
                "\t\tsum( CASE WHEN type = 'record' THEN 1 ELSE 0 END ) product_count,\n" +
                "\t\tsum( CASE WHEN type = 'logdetail'  and  CATEGORY_LEVEL_TWO ='shot_counter' and EQUIP_VARIABLE_VALUE > 0 THEN 1 ELSE 0 END ) detail_count \n" +
                "\tFROM\n" +
                "\t\tmes_product_record_log \n" +
                "\tWHERE\n" +
                "\t\tequipment_code LIKE '%HT%' \n";

        if (!StringUtils.isEmpty(productAccountModel.getEquipmentCode())) {
            sqlStr += " and  equipment_code = :equipmentCode ";
        } if (!StringUtils.isEmpty(productAccountModel.getPartNo())) {
            sqlStr += " and  part_no like '%' :partNo '%' ";
        } if (!StringUtils.isEmpty(productAccountModel.getStartTime())) {
            sqlStr += " and create_date_time >= :startTime ";
        } if (!StringUtils.isEmpty(productAccountModel.getEndTime())) {
            sqlStr += " and create_date_time < :endTime ";
        }


        sqlStr +=
                "\t\tAND part_no IS NOT NULL \n" +
                        "\tGROUP BY\n" +
                        "\t\tequipment_code,\n" +
                        "\t\tpart_no \n" +
                        "\t) mpr\n" +
                        "\tLEFT JOIN mes_wc_equipment me ON mpr.equipment_code = me.equipment_code \n" +
                        "\n" +
                        "\n" +
                        "UNION\n" +
                        "select \n" +
                        "eqcount.part_no,\n" +
                        "eqcount.equipment_code,\n" +
                        "me.equipment_name,\n" +
                        "me.work_cell_code,\n" +
                        "me.work_center_code,\n" +
                        "eqcount.product_count,\n" +
                        "eqcount.detail_count,\n" +
                        "eqcount.error_reason\n" +
                        "\n" +
                        "from (" +
                        "SELECT\n" +
                        "\teqcount.part_no,\n" +
                        "\teqcount.equipment_code ,\n" +
                        "\tme.equipment_name ,\n" +
                        "\tme.work_cell_code ,\n" +
                        "\tme.work_center_code ,\n" +
                        "\teqcount.product_count ,\n" +
                        "\teqcount.detail_count , \n" +
                        "\t null error_reason \n" +
                        "FROM\n" +
                        "\tmes_prod_rule_nosort_cfg mprnc\n" +
                        "\tRIGHT JOIN mes_wc_equipment me ON mprnc.equipment_code = me.equipment_code and me.is_deleted = 2 and me.is_valid = 1 \n" +
                        "\tRIGHT JOIN (\n" +
                        "\tSELECT\n" +
                        "\t\tequipment_code,\n" +
                        "\t\tpart_no,\n" +
                        "\t\tsum( CASE WHEN type = 'record' THEN 1 ELSE 0 END ) product_count,\n" +
                        "\t\tsum(\n" +
                        "\t\tCASE\n" +
                        "\t\t\t\t\n" +
                        "\t\t\t\tWHEN type = 'logdetail' \n" +
                        "AND ( (equipment_code != 'Geiss' and CATEGORY_LEVEL_TWO IN ( 'Product_finish', 'PART_OK', 'Part_Finish', 'Product_OK' ) )\n" +
                        "\t\t\t\tor  (equipment_code = 'Geiss' and CATEGORY_LEVEL_TWO = 'Product_finish') )\n" +
                        "\t\t\t\tAND EQUIP_VARIABLE_VALUE = 'True' THEN\n" +
                        "\t\t\t\t\t1 ELSE 0 \n" +
                        "\t\t\t\tEND" +
                        "\t\t\t\t) detail_count \n" +
                        "\t\t\tFROM\n" +
                        "\t\t\t\tmes_product_record_log \n" +
                        "\t\t\tWHERE  1 = 1  and equipment_code not like '%HT%' \n";
        if (!StringUtils.isEmpty(productAccountModel.getEquipmentCode())) {
            sqlStr += "\t\t\t and  equipment_code = :equipmentCode \n";
        } if (!StringUtils.isEmpty(productAccountModel.getPartNo())) {
            sqlStr += " and  part_no like '%' :partNo '%' ";

        } if (!StringUtils.isEmpty(productAccountModel.getStartTime())) {
            sqlStr += "\t\t\t and create_date_time >= :startTime \n";
        } if (!StringUtils.isEmpty(productAccountModel.getEndTime())) {
            sqlStr += "\t\t\t and create_date_time < :endTime \n";
        }
        sqlStr +=
                "\t\t\t    and \n" +
                        "\t\t\t\tpart_no IS NOT NULL \n" +
                        "\t\t\tGROUP BY\n" +
                        "\t\t\t\tequipment_code,\n" +
                        "\t\t\t\tpart_no \n" +
                        "\t\t\t) eqcount ON mprnc.equipment_code = eqcount.equipment_code  \n" +
                        "\t AND mprnc.out_part_no = eqcount.part_no where eqcount.equipment_code not LIKE '%HT%'  ";
        if (!StringUtils.isEmpty(productAccountModel.getEquipmentCode())) {
            sqlStr += "and eqcount.equipment_code= :equipmentCode ";
        }
        sqlStr += "\tand eqcount.product_count is not null and eqcount.detail_count is not null " +
                ") eqcount \t \n" +
                "left join mes_wc_equipment me ON eqcount.equipment_code = me.equipment_code and me.is_deleted = 2 and me.is_valid = 1 "+
                ") mpr \n" +
                "\t\n" +
                "\tleft join mes_equipment me\n" +
                "\ton mpr.equipment_code = me.equipment_code\n" +
                "\t " +
                "where (mpr.product_count != 0 or mpr.detail_count != 0)\n" +
                "and me.equipment_type = 10 order by me.equipment_code asc";


        Query query = entityManager.createNativeQuery(sqlStr);
        if (!StringUtils.isEmpty(productAccountModel.getEquipmentCode())) {
            query.setParameter("equipmentCode", productAccountModel.getEquipmentCode());
        }
        if (!StringUtils.isEmpty(productAccountModel.getPartNo())) {
            query.setParameter("partNo", productAccountModel.getPartNo());
        }
        if (!StringUtils.isEmpty(productAccountModel.getStartTime())) {
            query.setParameter("startTime", productAccountModel.getStartTime());
        }
        if (!StringUtils.isEmpty(productAccountModel.getEndTime())) {
            query.setParameter("endTime", productAccountModel.getEndTime());
        }

       /* query.unwrap(NativeQueryImpl.class).setResultTransformer(Transformers.aliasToBean(MesProductAccountResModel.class));
        List<MesProductAccountResModel> results = query.getResultList();*/
        List results = query.getResultList();
        for (Object result : results) {
            Object[] cells = (Object[]) result;
            MesProductAccountResModel model = new MesProductAccountResModel();
            model.setPartNo(StringUtils.isEmpty(cells[0]) ? "" : String.valueOf(cells[0]));
            model.setEquipmentCode(StringUtils.isEmpty(cells[1]) ? "" : String.valueOf(cells[1]));
            model.setEquipmentName(StringUtils.isEmpty(cells[2]) ? "" : String.valueOf(cells[2]));
            model.setWorkCellCode(StringUtils.isEmpty(cells[3]) ? "" : String.valueOf(cells[3]));
            model.setWorkCenterCode(StringUtils.isEmpty(cells[4]) ? "" : String.valueOf(cells[4]));
            model.setProductCount(StringUtils.isEmpty(cells[5]) ? "" : String.valueOf(cells[5]));
            model.setDetailCount(StringUtils.isEmpty(cells[6]) ? "" : String.valueOf(cells[6]));
            model.setErrorReason(StringUtils.isEmpty(cells[7]) ? "" : String.valueOf(cells[7]));

            resList.add(model);
        }

        //query.unwrap(NativeQueryImpl.class).setResultTransformer(Transformers.aliasToBean(MesProductAccountResModel.class));
        //List<MesProductAccountResModel> results = query.getResultList();



        return resList;
    }

    private List<MesProduceSn> getMesProduceSnList(MesProductionRecordModel mesProductionRecordModel) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(mesProductionRecordModel.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesProductionRecordModel.getPartNo(),"partNo",packBean);
        DdlPreparedPack.getStringEqualPack(mesProductionRecordModel.getOffLineWorkCellCode(),"workCellCode",packBean);
        DdlPreparedPack.getStringBiggerPack(mesProductionRecordModel.getCreateDateTimeStart(),"createDatetime",packBean);
        DdlPreparedPack.getStringSmallerPack(mesProductionRecordModel.getCreateDateTimeEnd(),"createDatetime",packBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.PRODUCE_SN_STATUS.OFFLINE.getValue(),"snStatus",packBean);
        return mesProduceSnRepository.findByHqlWhere(packBean);
    }

    private ListPager<MesProductionRecord> queryERPMaterialStockHqlPager(MesProductionRecord mesProductionRecord, Pager pager) {
        String org = mesProductionRecord.getOrganizeCode();
        String startTime = mesProductionRecord.getCompleteDateTimeStart();
        String endTime = mesProductionRecord.getCompleteDateTimeEnd();
        List<MesProductionRecord> mesProductionRecordList = new ArrayList<>();
        String countSql = "select count(1)  ";
        String commHql = " SELECT " +
                " DISTINCT mpr.id as id," +
                " mpr.organize_code as organizeCode," +
                " mpr.create_date_time AS createDatetime, " +
                " mpr.create_user AS createUser, " +
                " mpr.modify_date_time AS  modifyDatetime, " +
                " mpr.modify_user AS  modifyUser, " +
                " mpr.part_no as partNo," +
                " mpr.part_name as partName," +
                " mpr.work_center_code as workCenterCode," +
                " mpr.shift_code as shiftCode," +
                " mpr.work_cell_code as workCellCode," +
                " mpr.equipment_name as equipmentName," +
                " mpr.complete_date_time as completeDateTime," +
                " count( mpr.qty) as qty ";

        String sql = " from mes_production_record mpr" +
                " where 1=1  " +
                " and mpr.organize_code =:organizeCode " +
                " and mpr.report_type !='10' " +
                " and mpr.is_deleted ='2' " +
                " and mpr.is_valid ='1' ";
        if (!StringUtil.isEmpty(startTime)) {
            sql += " and mpr.complete_date_time >=:startTime ";
        }
        if (!StringUtil.isEmpty(endTime)) {
            sql += " and mpr.complete_date_time <:endTime ";
        }

        if (!StringUtil.isEmpty(mesProductionRecord.getPartNo())) {
            sql += " and mpr.part_no ='" + mesProductionRecord.getPartNo() + "' ";
        }

        if (!StringUtil.isEmpty(mesProductionRecord.getWorkCenterCode())) {
            sql += " and mpr.work_center_code like '%" + mesProductionRecord.getWorkCenterCode() + "%' ";
        }

        if (!StringUtil.isEmpty(mesProductionRecord.getShiftCode())) {
            sql += " and mpr.shift_code ='" + mesProductionRecord.getShiftCode() + "' ";
        }

        if (!StringUtil.isEmpty(mesProductionRecord.getWorkCellCode())) {
            sql += " and mpr.work_cell_code like '%" + mesProductionRecord.getWorkCellCode() + "%' ";
        }

        if (!StringUtil.isEmpty(mesProductionRecord.getEquipmentName())) {
            sql += " and mpr.equipment_name like '%" + mesProductionRecord.getEquipmentName() + "%' ";
        }

        sql += " GROUP BY mpr.part_no,mpr.work_cell_code ORDER BY mpr.complete_date_time desc ";

        Query queryObject = entityManager.createNativeQuery(commHql + sql);
        queryObject.setParameter("organizeCode", org);
        if (!StringUtil.isEmpty(startTime)) {
            queryObject.setParameter("startTime", startTime);
        }
        if (!StringUtil.isEmpty(endTime)) {
            queryObject.setParameter("endTime", endTime);
        }

        Query queryCountObject = entityManager.createNativeQuery(countSql + sql);
        queryCountObject.setParameter("organizeCode", org);
        if (!StringUtil.isEmpty(startTime)) {
            queryCountObject.setParameter("startTime", startTime);
        }
        if (!StringUtil.isEmpty(endTime)) {
            queryCountObject.setParameter("endTime", endTime);
        }

        queryObject.unwrap(SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP)
                .setMaxResults(pager.getPageSize()).setFirstResult(pager.getStartRow()).getResultList();
        queryCountObject.unwrap(SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);

        List resultList = queryCountObject.getResultList();
        int countSize = resultList.size();
        if (countSize <= 0) {
            return new ListPager(mesProductionRecordList,pager);
        }

        pager = PagerHelper.getPager(pager, countSize);

        List<HashMap<String, Object>> hashMaps = queryObject.getResultList();

        for (HashMap<String, Object> hashMap : hashMaps) {
            try {
                mesProductionRecordList.add((MesProductionRecord) BeanUtil.populateBean(hashMap, MesProductionRecord.class));
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return new ListPager(mesProductionRecordList, pager);

    }

    @Override
    public List<MesProductAccountResModel> transferAccount(List<MesProductAccountResModel> mesProductAccountResModels, MesProductAccountModel productAccountModel, Pager pager) {
        /**
         * 760B_A58_ReMaoHan  只有finish信号
         * 760BFoaming
         * 760B_LaserCutting
         * ZhenKong
         */
        List<String> equimentList = Stream.of("760B_A58_ReMaoHan","760BFoaming","760B_LaserCutting","ZhenKong").collect(Collectors.toList());
        if (CollectionUtils.isEmpty(mesProductAccountResModels)) {
            return null;
        }
        List<MesProductAccountResModel> partitionList = partition(mesProductAccountResModels, pager.getPageSize(), pager.getCurrentPage());
        for (MesProductAccountResModel mesProductAccountResModel : partitionList) {
            boolean isErrorCount = false;
            int size = mesProductAccountResModel.getPartNo().split(",").length == 1 ? mesProductAccountResModel.getPartNo().split(";").length : mesProductAccountResModel.getPartNo().split(",").length;

            int productCount = Integer.valueOf(mesProductAccountResModel.getProductCount());
            int detailCount = Integer.valueOf(mesProductAccountResModel.getDetailCount());
            if (detailCount * size > productCount) {
                isErrorCount = true;
            }
            if (!isErrorCount) {
                continue;
            }
            DdlPackBean packBean = DdlPackBean.getDdlPackBean("CK01");
            DdlPreparedPack.getStringEqualPack(mesProductAccountResModel.getEquipmentCode(), "equipmentCode", packBean);

            DdlPreparedPack.getStringSmallerNotEqualPack(productAccountModel.getEndTime(), "createDatetime", packBean);
            DdlPreparedPack.getStringBiggerPack(productAccountModel.getStartTime(), "createDatetime", packBean);
            DdlPreparedPack.getStringEqualPack(mesProductAccountResModel.getPartNo(), "partNo", packBean);

            DdlPreparedPack.getOrderBy("createDatetime", CommonEnumUtil.ASC_OR_DESC.ASC.getValue(), packBean);
            List<MesProductRecordLog> recordLogs = mesProductRecordLogRepository.findByHqlWhere(packBean);

            //
            DdlPackBean packOffilinBean = DdlPackBean.getDdlPackBean("CK01");
            DdlPreparedPack.getStringEqualPack(mesProductAccountResModel.getEquipmentCode(), "equipmentCode", packOffilinBean);

            Date yesterday = TimeTool.getDateBefore(DateUtil.parseDateTime(productAccountModel.getStartTime()), 1);
            String yesterdayStr = DateUtil.formatDateTime(yesterday);
            DdlPreparedPack.getStringSmallerNotEqualPack(productAccountModel.getStartTime(), "createDatetime", packOffilinBean);
            DdlPreparedPack.getStringBiggerPack(yesterdayStr, "createDatetime", packOffilinBean);
            DdlPreparedPack.getStringEqualPack(mesProductAccountResModel.getPartNo(), "partNo", packOffilinBean);
            DdlPreparedPack.getStringLikerPack("客户端", "message", packOffilinBean);

            DdlPreparedPack.getOrderBy("createDatetime", CommonEnumUtil.ASC_OR_DESC.ASC.getValue(), packOffilinBean);
            List<MesProductRecordLog> offlineRecordLogs = mesProductRecordLogRepository.findByHqlWhere(packOffilinBean);

            recordLogs.addAll(offlineRecordLogs);
            recordLogs = recordLogs.stream().sorted(Comparator.comparing(MesProductRecordLog::getCreateDatetime)).collect(Collectors.toList());
            if (mesProductAccountResModel.getEquipmentCode().contains("HT")) {
                // 1 判断2个 shotcount之间有没有加工记录，如果没有，就需要看 设备是不是192，然后看是否是网络原因
                String reason = handleHT(recordLogs);
                mesProductAccountResModel.setErrorReason(reason);
            } else if (equimentList.contains(mesProductAccountResModel.getEquipmentCode())) {
                // 1 判断2个 shotcount之间有没有加工记录，如果没有，就需要看 设备是不是192，然后看是否是网络原因
                String reason =handleNotHT(recordLogs);
                mesProductAccountResModel.setErrorReason(reason);
            } else {
                String reason =handleNotHTAndHasRelease(recordLogs);
                mesProductAccountResModel.setErrorReason(reason);
            }
        }

        return partitionList;
    }

    /*private String handleNotHT(List<MesProductRecordLog> recordLogs) {
        int result = 0;
        String startTime = "";
        String endTime = "";
        String reason = "";
        String offlineReason = "【%s】到【%s】,客户端下线,导致【%s】个完成信号丢失";
        String reasonOffline = "因其他原因：系统，网络，发版等，导致【%s】的完成信号丢失;";
        String reasonQuality = "【%s】设备信号质量为0，导致【%s】个完成信号丢失;";
        String reasonBypass = "【%s】的完成信号,设备打BYPASS,视为无效;";
        // BYPASS 默认fasle
        boolean bypassFlag = false;
        boolean offlineFlag = false;
        List<String> finishList = Stream.of("Product_finish","Part_Finish").collect(Collectors.toList());

        for (MesProductRecordLog recordLog : recordLogs) {
            int errorCount = 0;

            if (recordLog.getId() == 1838536402558042112l) {
                int a = 1;
            }
            if (Objects.equal(recordLog.getType(), "monitor") && recordLog.getMessage().contains("客户端下线")) {
                startTime = recordLog.getCreateDatetime();
                offlineFlag = true;
                continue;
            }
            if (Objects.equal(recordLog.getType(), "monitor") && recordLog.getMessage().contains("客户端上线") ) {
                endTime = recordLog.getCreateDatetime();
                offlineFlag = false;
                if (result > 0) {
                    reason += String.format(offlineReason, startTime, endTime, 1 + "");
                    result = 0;
                    errorCount = 0;
                }
                //result = 0;
                continue;
            }
            if (offlineFlag &&finishList.contains(recordLog.getCategoryLevelTwo()) && Objects.equal(recordLog.getEquipVariableValue(), "True")) {
                if (result > 0) {
                    //endTime = recordLog.getCreateDatetime();
                    errorCount ++;
                    reason += String.format(offlineReason, startTime, startTime, errorCount + "");
                    continue;
                }
                errorCount ++;
                result++;
                continue ;
            }

            if (Objects.equal(recordLog.getCategoryLevelTwo(), "BYPASS")) {
                if (Objects.equal(recordLog.getEquipVariableValue(), "True")) {
                    if (StringUtils.isEmpty(startTime)) {
                        startTime = recordLog.getCreateDatetime();
                    }
                    endTime = recordLog.getCreateDatetime();
                    // 每次都取反，第一次是打开，那第二次就是关闭了
                    bypassFlag = true;
                    continue;
                } else {
                    if (bypassFlag) {
                        bypassFlag = false;
                        errorCount = 0;
                        continue;
                    }
                }
            }
            if (bypassFlag && recordLog.getType().equals("record"))  {
                errorCount ++;
                reason += String.format(reasonBypass,startTime, errorCount +"");
                continue;
            }


            if (result == 0) {
                if (!finishList.contains(recordLog.getCategoryLevelTwo()) || !Objects.equal(recordLog.getEquipVariableValue(), "True")) {
                    continue;
                }
                startTime = recordLog.getCreateDatetime();
                result ++;
            } else {
                if (finishList.contains(recordLog.getCategoryLevelTwo()) && Objects.equal(recordLog.getEquipVariableValue(), "True")) {
                    errorCount ++;
                    if (Objects.equal(recordLog.getQuality(), "192")) {
                        reason += String.format(reasonOffline,startTime, errorCount +"");
                    } else {
                        reason += String.format(reasonQuality,startTime, errorCount +"");
                    }
                    startTime = recordLog.getCreateDatetime();

                }
                if (recordLog.getType().equals("record")) {
                    result --;
                }
            }
        }
        // 如果走到最后，还有预编译的错误，则直接加上
        if (!StringUtils.isEmpty(preReason)) {
            reason += preReason;
        }
        return reason;
    }*/


    private String handleNotHT(List<MesProductRecordLog> recordLogs) {
        int result = 0;
        // 定义有问题的开始时间
        String startTime = "";
        // 定义有问题的结束时间
        String endTime = "";
        // 错误个数
        // 异常原因
        String reason = "";
        String reasonOffline = "因其他原因：系统，网络，发版等，导致【%s】的完成信号丢失;";

        String offlineReason = "【%s】到【%s】,客户端下线,导致【%s】个完成信号丢失;";
        String reasonQuality = "【%s】设备信号质量为0，视为无效;";
        String reasonBypass = "【%s】的完成信号,设备打BYPASS,视为无效;";

        // BYPASS 默认fasle
        boolean bypassFlag = false;

        // BYPASS 默认fasle
        boolean offlineFlag = false;
        //List<String> finishList = Stream.of("Product_finish","PART_OK","Part_Finish","Product_OK").collect(Collectors.toList());
        List<String> finishList = Stream.of("Product_finish","Part_Finish").collect(Collectors.toList());
        String preReason = "";
        int errorCount = 0;

        for (MesProductRecordLog recordLog : recordLogs) {
            if (recordLog.getId() ==1846373460483817472l) {
                int a = 1;
            }
            if (Objects.equal(recordLog.getType(), "monitor") && recordLog.getMessage().contains("客户端下线")) {
                startTime = recordLog.getCreateDatetime();
                offlineFlag = true;
                continue;
            }
            if (Objects.equal(recordLog.getType(), "monitor") && recordLog.getMessage().contains("客户端上线") ) {
                if (!offlineFlag)  {
                    continue;
                }
                endTime = recordLog.getCreateDatetime();
                offlineFlag = false;

                if (errorCount > 0) {
                    reason += String.format(offlineReason, startTime, endTime, errorCount + "");
                    preReason = "";
                    result = 0;
                    errorCount = 0;
                } else {
                    if (result > 0) {
                        reason += String.format(offlineReason, startTime, endTime, 1 + "");
                        preReason = "";
                        result = 0;
                        errorCount = 0;
                    }
                }
                //result = 0;
                continue;
            }
            // 如果是bypass 遇到的所有加工完成指令 都计数并且原因是bypass导致
            if (bypassFlag && finishList.contains(recordLog.getCategoryLevelTwo()) && Objects.equal(recordLog.getEquipVariableValue(), "True"))  {
                if (!StringUtils.isEmpty(preReason)) {
                    reason += preReason;
                }
                preReason = String.format(reasonBypass,startTime, 1 +"");
                //reason += String.format(reasonBypass,startTime, 1 +"");
                continue;
            }
            if (offlineFlag  && finishList.contains(recordLog.getCategoryLevelTwo()) && Objects.equal(recordLog.getEquipVariableValue(), "True")) {
                // 如果经过了1个finish，然后再经过客户端下线，再遇到finish则认为是客户端下线引起,如果没有需要预处理一个下线的原因
                errorCount ++;
                if (result > 0) {
                    //reason += preReason;
                    preReason = String.format(offlineReason, endTime, recordLog.getCreateDatetime(), errorCount + "");
                    endTime = recordLog.getCreateDatetime();
                    continue;
                }
                /*else {
                    endTime = recordLog.getCreateDatetime();
                    preReason = String.format(offlineReason, startTime,endTime, errorCount + "");
                }*/
                result++;
                continue ;
            }

            if (Objects.equal(recordLog.getCategoryLevelTwo(), "BYPASS") && Objects.equal(recordLog.getEquipVariableValue(), "True")) {
                // 如果有finish然后遇到bypass 需要预处理
                if (result > 0) {
                    reason += String.format(reasonBypass,startTime, 1 +"");
                    result = 0;
                }
                startTime = recordLog.getCreateDatetime();
                endTime = recordLog.getCreateDatetime();
                bypassFlag = true;
                continue;
            }

            // 遇到bypass = false 则重置错误次数
            if (bypassFlag && Objects.equal(recordLog.getCategoryLevelTwo(), "BYPASS") && Objects.equal(recordLog.getEquipVariableValue(), "False")) {
                endTime = recordLog.getCreateDatetime();
                errorCount = 0;
                bypassFlag = false;
                continue;
            }

            // BYPASS 逻辑结束  result = 1 代表一个成功新型号，遇到一个加工记录就会-1  变成0
            if (result == 0) {
                // 没有加工完成指令，或者没有release指令，但是有加工记录，则也需要重置
                if (recordLog.getType().equals("record")) {
                    bypassFlag = false;
                    preReason = "";
                }

                if (!finishList.contains(recordLog.getCategoryLevelTwo()) || !Objects.equal(recordLog.getEquipVariableValue(), "True")) {
                    continue;
                }
                if (!StringUtils.isEmpty(preReason)) {
                    reason +=  preReason;
                    preReason = "";
                }
                result ++;
                startTime = recordLog.getCreateDatetime();

            } else {
                if (finishList.contains(recordLog.getCategoryLevelTwo()) && Objects.equal(recordLog.getEquipVariableValue(), "True")) {
                    // 上一个加工记录信号还没走完，这边又继续发一个加工记录，且没有release 两个都要判定问题
                    if (!StringUtils.isEmpty(preReason)) {
                        reason +=  preReason;
                    }
                    if (Objects.equal(recordLog.getQuality(), "192")) {
                        reason += String.format(reasonOffline,startTime, 1 +"");
                    } else {
                        reason += String.format(reasonQuality,startTime, 1 +"");
                    }
                    // 每次preReason附完值都需要更新startTime，完成信号
                    startTime = recordLog.getCreateDatetime();

                }

                if (recordLog.getType().equals("record")) {
                    bypassFlag = false;
                    offlineFlag = false;
                    result --;
                    preReason = "";
                    startTime = "";
                    errorCount = 0;
                }
            }
        }
        // 如果走到最后，还有预编译的错误，则直接加上
        if (!StringUtils.isEmpty(preReason)) {
            reason += preReason;
        } else {
            if (result > 0) {
                reason += String.format(reasonOffline,startTime, 1 +"");
            }
        }

        return reason;
    }

    private String handleNotHTAndHasRelease(List<MesProductRecordLog> recordLogs) {
        int result = 0;
        // 定义有问题的开始时间
        String startTime = "";
        // 定义有问题的结束时间
        String endTime = "";
        // 错误个数
        // 异常原因
        String reason = "";
        String reasonOffline = "因其他原因：系统，网络，发版等，导致【%s】的完成信号丢失;";

        String offlineReason = "【%s】到【%s】,客户端下线,导致【%s】个完成信号丢失;";
            String reasonQuality = "【%s】设备信号质量为0，视为无效;";
        String reasonBypass = "【%s】的完成信号,设备打BYPASS,视为无效;";
        String releaseBypass = "【%s】的完成信号,未发送Release信号,视为无效;";


        // BYPASS 默认fasle
        boolean bypassFlag = false;
        // release 默认false
        boolean releaseFlag = false;

        // BYPASS 默认fasle
        boolean offlineFlag = false;
        //List<String> finishList = Stream.of("Product_finish","PART_OK","Part_Finish","Product_OK").collect(Collectors.toList());
        List<String> finishList = Stream.of("Product_finish","Part_Finish").collect(Collectors.toList());
        String preReason = "";
        int errorCount = 0;

        for (MesProductRecordLog recordLog : recordLogs) {
            if (recordLog.getId() ==1296257140498948096l) {
                int a = 1;
            }
            if (Objects.equal(recordLog.getType(), "monitor") && recordLog.getMessage().contains("客户端下线")) {
                startTime = recordLog.getCreateDatetime();
                offlineFlag = true;
                continue;
            }
            if (Objects.equal(recordLog.getType(), "monitor") && recordLog.getMessage().contains("客户端上线") ) {
                endTime = recordLog.getCreateDatetime();
                offlineFlag = false;
                if (result > 0) {
                    // 如果preReason已经有则代表已经判定过，可以直接获取
                    if (!StringUtils.isEmpty(preReason)) {
                        reason += preReason;
                        startTime = recordLog.getCreateDatetime();
                        preReason = "";
                    } else {
                        reason += String.format(offlineReason, startTime, endTime, 1 + "");
                    }
                    result = 0;
                    errorCount = 0;
                }
                //result = 0;
                continue;
            }
            // 如果是bypass 遇到的所有加工完成指令 都计数并且原因是bypass导致
            if (bypassFlag && finishList.contains(recordLog.getCategoryLevelTwo()) && Objects.equal(recordLog.getEquipVariableValue(), "True"))  {
                if (!StringUtils.isEmpty(preReason)) {
                    reason += preReason;
                }
                preReason = String.format(reasonBypass,startTime, 1 +"");
                //reason += String.format(reasonBypass,startTime, 1 +"");
                continue;
            }
            if (offlineFlag  && finishList.contains(recordLog.getCategoryLevelTwo()) && Objects.equal(recordLog.getEquipVariableValue(), "True")) {
                // 如果经过了1个finish，然后再经过客户端下线，再遇到finish则认为是客户端下线引起,如果没有需要预处理一个下线的原因
                errorCount ++;

                if (result > 0) {
                    //reason += preReason;
                    preReason = String.format(offlineReason, endTime, recordLog.getCreateDatetime(), errorCount + "");
                    endTime = recordLog.getCreateDatetime();
                    continue;
                }
                /*if (result > 0) {
                    reason += String.format(offlineReason, startTime, startTime, errorCount + "");
                    continue;
                } else {
                    endTime = recordLog.getCreateDatetime();
                    preReason = String.format(offlineReason, startTime,endTime, errorCount + "");
                }*/
                result++;
                continue ;
            }

            if (Objects.equal(recordLog.getCategoryLevelTwo(), "BYPASS") && Objects.equal(recordLog.getEquipVariableValue(), "True")) {
                if (offlineFlag) {
                    continue;
                }
                // 如果有finish然后遇到bypass 需要预处理
                if (result > 0) {
                    reason += String.format(reasonBypass,startTime, 1 +"");
                    result = 0;
                }
                startTime = recordLog.getCreateDatetime();
                endTime = recordLog.getCreateDatetime();
                bypassFlag = true;
                continue;
            }

            // 遇到bypass = false 则重置错误次数
            if (bypassFlag && Objects.equal(recordLog.getCategoryLevelTwo(), "BYPASS") && Objects.equal(recordLog.getEquipVariableValue(), "False")) {
                if (offlineFlag) {
                    continue;
                }
                endTime = recordLog.getCreateDatetime();
                errorCount = 0;
                bypassFlag = false;
                continue;
            }
            // 如果没有发送release 就出现了加工完成信号，则记录错误原因，且预错误，遇到有加工记录则取消
            /**
             *  没有release有完成信号 需要预处理，先得出一个结论:如果没有发送release指令，导致信号丢失
             * 后面如果没有加工记录，则预处理结果变成实际结果
             * 后面有加工记录则预处理结果清空
             */
            if (!releaseFlag) {
                if (finishList.contains(recordLog.getCategoryLevelTwo()) && Objects.equal(recordLog.getEquipVariableValue(), "True")) {
                    if (!StringUtils.isEmpty(preReason)) {
                        reason += preReason;
                    }
                    startTime = recordLog.getCreateDatetime();
                    errorCount ++;
                    preReason = String.format(releaseBypass, startTime, errorCount + "");
                    continue;

                }
            }
            // BYPASS 逻辑结束  result = 1 代表一个成功新型号，遇到一个加工记录就会-1  变成0
            if (result == 0) {
                // 没有加工完成指令，或者没有release指令，但是有加工记录，则也需要重置
                if (recordLog.getType().equals("record")) {
                    releaseFlag = false;
                    bypassFlag = false;
                    preReason = "";
                }

                if (Objects.equal(recordLog.getCategoryLevelTwo(), "Work_Allow") && Objects.equal(recordLog.getEquipVariableValue(), "True")) {
                    releaseFlag = true;
                    continue;
                }

                if (!finishList.contains(recordLog.getCategoryLevelTwo()) || !Objects.equal(recordLog.getEquipVariableValue(), "True")) {
                    continue;
                }
                if (!releaseFlag) {
                    reason += preReason;
                    startTime = recordLog.getCreateDatetime();
                    preReason = String.format(releaseBypass, startTime, 1 + "");
                    result ++;
                    continue;
                } if (!StringUtils.isEmpty(preReason)) {
                    reason +=  preReason;
                    preReason = "";
                }
                result ++;
                startTime = recordLog.getCreateDatetime();

            } else {
                if (finishList.contains(recordLog.getCategoryLevelTwo()) && Objects.equal(recordLog.getEquipVariableValue(), "True")) {
                    // 上一个加工记录信号还没走完，这边又继续发一个加工记录，且没有release 两个都要判定问题
                    if (!StringUtils.isEmpty(preReason)) {
                        reason +=  preReason;
                    }
                    if (!releaseFlag) {
                        startTime = recordLog.getCreateDatetime();
                        preReason = String.format(releaseBypass, startTime, 1 + "");
                    } else {
                        if (Objects.equal(recordLog.getQuality(), "192")) {
                            reason += String.format(reasonOffline,startTime, 1 +"");
                        } else {
                            reason += String.format(reasonQuality,startTime, 1 +"");
                        }
                        // 每次preReason附完值都需要更新startTime，完成信号
                        startTime = recordLog.getCreateDatetime();
                    }
                }

                if (recordLog.getType().equals("record")) {
                    releaseFlag = false;
                    bypassFlag = false;
                    offlineFlag = false;
                    result --;
                    preReason = "";
                    startTime = "";
                    errorCount = 0;
                }
            }
        }
        // 如果走到最后，还有预编译的错误，则直接加上
        if (!StringUtils.isEmpty(preReason)) {
            reason += preReason;
        }
        return reason;
    }

    private String handleHT(List<MesProductRecordLog> recordLogs) {
        int result = 0;
        String startTime = "";
        String endTime = "";
        String reason = "";
        String offlineReason = "【%s】到【%s】,客户端下线,导致【%s】个完成信号丢失";
        String reasonOffline = "因其他原因：系统，网络，发版等，导致【%s】的完成信号丢失;";
        String reasonQuality = "【%s】设备信号质量为0，导致完成信号丢失;";
        boolean offlineFlag = false;
        int errorCount = 0;

        for (MesProductRecordLog recordLog : recordLogs) {
            if (recordLog.getId() ==1284126441587343426l) {
                int i = 1;
            }
            if (Objects.equal(recordLog.getType(), "monitor") && recordLog.getMessage().contains("客户端下线")) {
                startTime = recordLog.getCreateDatetime();
                offlineFlag = true;
                continue;
            }
            if (Objects.equal(recordLog.getType(), "monitor") && recordLog.getMessage().contains("客户端上线") && offlineFlag) {
                endTime = recordLog.getCreateDatetime();
                offlineFlag = false;
                if (errorCount > 0) {
                    reason += String.format(offlineReason, startTime, endTime, errorCount + "");
                    errorCount = 0;
                }
                continue;
            }
            if (offlineFlag && Objects.equal(recordLog.getCategoryLevelTwo(),"shot_counter") && recordLog.getEquipVariableValue().compareTo("0") > 0) {
                errorCount ++;
                continue ;
            }

            if (result == 0) {
                if (!Objects.equal(recordLog.getCategoryLevelTwo(),"shot_counter") ) {
                    continue;
                }
                if (recordLog.getEquipVariableValue().compareTo("0") > 0) {
                    result ++;
                }
            } else {
                if (Objects.equal(recordLog.getCategoryLevelTwo(),"shot_counter") && recordLog.getEquipVariableValue().compareTo("0") > 0) {
                    startTime = recordLog.getCreateDatetime();

                    if (Objects.equal(recordLog.getQuality(), "192")) {
                        reason += String.format(reasonOffline,startTime, 1 +"");
                    } else {
                        reason += String.format(reasonQuality,startTime, 1 +"");
                    }
                }
                if (recordLog.getType().equals("record")) {
                    result --;
                }
            }
        }
        if (errorCount > 0 && offlineFlag) {
            reason += String.format(offlineReason, startTime, "最后", errorCount + "");
        }
        return reason;
    }

    private List<MesProductAccountResModel> partition(List<MesProductAccountResModel> list, int pageSize, int pageNum) {
        // 将 List 按照 PageSzie 拆分成多个List
        List<List<MesProductAccountResModel>> partition = Lists.partition(list, pageSize);
        // 总页数
        int pages = partition.size();
        pageNum  = pageNum -1;
        pageNum = pageNum <= 0 ? 0 : (pageNum <= (pages - 1) ? pageNum : (pages - 1));
        return partition.get(pageNum);
    }
}
