package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesCustomerPartService;
import cn.estsh.i3plus.ext.mes.api.base.IMesPullingOrderInfoService;
import cn.estsh.i3plus.ext.mes.api.base.IMesWorkOrderPartService;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.icloud.core.sdk.ICoreUtilCloud;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.util.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class MesPullingOrderInfoService extends BaseMesService<MesPullingOrderInfo> implements IMesPullingOrderInfoService {
    public static final Logger LOGGER = LoggerFactory.getLogger(MesPullingOrderInfoService.class);
    @Autowired
    private MesPullingOrderInfoRepository mesPullingOrderInfoRepository;
    @Autowired
    private MesPullingOrderPartInfoRepository mesPullingOrderPartInfoRepository;
    @Autowired
    private MesPartPullRepository mesPartPullRDao;
    @Autowired
    private MesPartPullDetailRepository mesPartPullDetailRDao;
    @Autowired
    private MesWorkOrderPartRepository workOrderPartRepository;
    @Autowired
    private MesWorkOrderRepository mesWorkOrderRepository;
    @Autowired
    private ICoreUtilCloud sysOrderNoRuleCloud;
    @Autowired
    private MesWorkCenterRepository mesWorkCenterRepository;
    @Autowired
    private MesPullingOrderExtendsRepository mesPullingOrderExtendsRepository;
    @Autowired
    private IMesPrintedSnLogRepository snLogRao;
    @Autowired
    private IMesCustomerPartService mesCustomerPartService;
    @Autowired
    private IMesWorkOrderPartService mesWorkOrderPartService;
    protected void setPackQueryBean(MesPullingOrderInfo bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getPullCode(), "pullCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getPullingOrderNo(), "pullingOrderNo", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getPullOrderType(), "pullOrderType", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getPrintStatus(), "printStatus", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getPullOrderStatus(), "pullOrderStatus", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getWorkOrderNo(), "workOrderNo", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartNo(), "partNo", packBean);
    }

    @Override
    public void validateImport(List<MesPullingOrderInfo> beanList) {

        for (MesPullingOrderInfo item : beanList) {
            // 数据校验
            ValidatorBean.checkNotNull(item.getPullCode(), "拉动组代码不能为空");
            ValidatorBean.checkNotNull(item.getWorkCenterCode(), "产线代码不能为空");


            DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(item.getPullCode(), "pullCode", seriesPackBean);
            DdlPreparedPack.getStringEqualPack(item.getWorkOrderNo(), "workOrderNo", seriesPackBean);
            DdlPreparedPack.getStringEqualPack(item.getWorkCenterCode(), "workCenterCode", seriesPackBean);
            boolean flg = baseRDao.isExitByHql(seriesPackBean);
            if (flg) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("工单【%s】拉动组代码【%s】已经存在，请检查数据", item.getWorkOrderNo(), item.getPullCode())
                        .build();
            }
        }
    }

    @Override
    public void doCreatePullOrderByPullGroupAndWorkOrder(String organizeCode,Integer pageSize, String userName) {
        //1.拉动组集合+拉动组明细集合pullingGroupList+ pullingpartlist
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        List<MesPartPull> pulliGroupList = mesPartPullRDao.findByHqlWhere(ddlPackBean);
        if (CollectionUtils.isEmpty(pulliGroupList)) {
            LOGGER.info("根据拉动组和生产工单产生拉动单JOB-->{}拉动组不存在", organizeCode);
            return;
        }
        Map<String, List<MesPartPull>> pullCodeMap = pulliGroupList.stream().collect(Collectors.groupingBy(MesPartPull::getPullCode));

        DdlPackBean ddlPackBeanDetail = DdlPackBean.getDdlPackBean(organizeCode);
        List<MesPartPullDetail> pullDetailList = mesPartPullDetailRDao.findByHqlWhere(ddlPackBeanDetail);
        if (CollectionUtils.isEmpty(pullDetailList)) {
            LOGGER.info("根据拉动组和生产工单产生拉动单JOB-->{}拉动组明细不存在", organizeCode);
            return;
        }
        Map<String, List<MesPartPullDetail>> partPullDetailListMap = pullDetailList.stream().collect(Collectors.groupingBy(MesPartPullDetail::getPartNo));
        //2.客户物料集合 [PART_NO] in 拉动组明细物料集合
        //3.产线数据
//        DdlPackBean workPackBean = DdlPackBean.getDdlPackBean(organizeCode);
//        DdlPreparedPack.getInPackList(workCenterList, "workCenterCode", workPackBean);
//        List<MesWorkCenter> mesWorkCenterList = mesWorkCenterRepository.findByHqlWhere(workPackBean);
//        if (CollectionUtils.isEmpty(mesWorkCenterList)) {
//            LOGGER.info("根据拉动组和生产工单产生拉动单JOB-->工厂号{},拉动组对应产线信息不存在-->{}", organizeCode, workCenterList);
//            return;
//        }
//        Map<String, List<MesWorkCenter>> workCenterListMap = mesWorkCenterList.stream().collect(Collectors.groupingBy(MesWorkCenter::getWorkCenterCode));
        List<String> workCenterList = pulliGroupList.stream().map(MesPartPull::getWorkCenterCode).distinct().collect(Collectors.toList());

        //4.生产工单 在加工状态 [PULLING_STATUS]  =0 未创建
        List<MesWorkOrder> workOrderList = getMesWorkOrderList(organizeCode, pageSize, workCenterList);
        if (CollectionUtils.isEmpty(workOrderList)) {
            LOGGER.info("根据拉动组和生产工单产生拉动单JOB-->工厂号{},产线信息没有【未创建】【在加工】状态的工单-->{}", organizeCode, workCenterList);
            return;
        }
        Map<String, List<MesWorkOrder>> workOrderListMap = workOrderList.stream().collect(Collectors.groupingBy(MesWorkOrder::getWorkOrderNo));

        List<MesWorkOrderPart> mesWorkOrderPartList = mesWorkOrderPartService.findMesWorkOrderPart(workOrderList.stream().map(MesWorkOrder::getWorkOrderNo).collect(Collectors.toList()),organizeCode);
        if (CollectionUtils.isEmpty(mesWorkOrderPartList)) {
            LOGGER.info("根据拉动组和生产工单产生拉动单JOB-->工厂号{},产线对应的工单对应工单零件不存在-->{}", organizeCode, workOrderList.stream().map(MesWorkOrder::getWorkOrderNo).collect(Collectors.toList()));
            return;
        }
        LOGGER.info("根据拉动组和生产工单产生拉动单JOB-->工厂号{},产线对应的工单对应工单零件mesWorkOrderPartList-->{}", organizeCode, mesWorkOrderPartList);
        //5.取工单零件和拉动组交集，根据拉动主表对应的类型 生成相应的拉动单和拉动单明细
        Map<String, List<MesWorkOrderPart>> workOrderNoListMap = mesWorkOrderPartList.stream().collect(Collectors.groupingBy(MesWorkOrderPart::getWorkOrderNo));
        //6、获取客户物料信息
        List<MesCustomerPart> mesCustomerPartByPartNoList = mesCustomerPartService.findMesCustomerPartByPartNo(organizeCode, partPullDetailListMap.keySet().stream().distinct().collect(Collectors.toList()));
        Map<String, List<MesCustomerPart>> customerPartMap = CollectionUtils.isEmpty(mesCustomerPartByPartNoList) ? null : mesCustomerPartByPartNoList.stream().collect(Collectors.groupingBy(MesCustomerPart::getErpPartNo));

        List<MesPullingOrderInfo> mesPullingOrderInfoList = new ArrayList<>();
        List<MesPullingOrderPartInfo> mesPullingOrderPartInfoList = new ArrayList<>();
        List<MesPullingOrderExtends> mesPullingOrderExtendsList = new ArrayList<>();
        List<String> workOrderNoList = new ArrayList<>();
        for (Map.Entry<String, List<MesWorkOrderPart>> stringListEntry : workOrderNoListMap.entrySet()) {
            //需要生成拉动单的工单号 对应工单
            List<MesWorkOrder> mesWorkOrders = workOrderListMap.get(stringListEntry.getKey());
            MesWorkOrder mesWorkOrder = mesWorkOrders.get(0);

            //工单对应的工单零件号
            List<MesWorkOrderPart> workOrderParts = stringListEntry.getValue();
            //待生成的拉动组明细
            List<MesPartPullDetail> pullDetailListNew = new ArrayList<>();
            for (MesWorkOrderPart workOrderPart : workOrderParts) {
                if (!CollectionUtils.isEmpty(partPullDetailListMap.get(workOrderPart.getPartNo()))) {
                    pullDetailListNew.addAll(partPullDetailListMap.get(workOrderPart.getPartNo()));
                }
            }
            if (CollectionUtils.isEmpty(pullDetailListNew)) {
                LOGGER.info("根据拉动组和生产工单产生拉动单JOB-->工厂号{},产线-->{}对应的工单号-->{}没有需要生成拉动单零件",
                        organizeCode, mesWorkOrder.getWorkCenterCode(), mesWorkOrder.getWorkOrderNo());
                continue;
            }
            Map<String, List<MesWorkOrderPart>> workOrderPartListMap = workOrderParts.stream().collect(Collectors.groupingBy(MesWorkOrderPart::getPartNo));
            //生成拉动单 如果是多个拉动组的话都要产生拉动单
            Map<String, List<MesPartPullDetail>> pullCodeListMap = pullDetailListNew.stream().collect(Collectors.groupingBy(MesPartPullDetail::getPullCode));
            for (Map.Entry<String, List<MesPartPullDetail>> stringListEntryDetail : pullCodeListMap.entrySet()) {
                String pullCode = stringListEntryDetail.getKey();
                List<MesPartPull> mesPartPulls = pullCodeMap.get(pullCode);
                MesPartPull mesPartPull = mesPartPulls.get(0);
                //拉动组明细拉动单号对应拉动组 如果拉动组产线跟工单产线不一致 不生成
                if (mesWorkOrder.getWorkCenterCode().equals(mesPartPull.getWorkCenterCode())){
                    List<MesPartPullDetail> mesPartPullDetails = stringListEntryDetail.getValue();
                    //拉动组号+年月日+工单后5位
                    String orderNo = mesPartPull.getPullCode()+ TimeTool.getNowTime("yyMMdd") +
                            mesWorkOrder.getWorkOrderNo().substring(mesWorkOrder.getWorkOrderNo().length()-5);
                    MesPullingOrderInfo pullingOrder = new MesPullingOrderInfo();
                    pullingOrder.setOrganizeCode(organizeCode);
                    pullingOrder.setPullingOrderNo(orderNo);
                    pullingOrder.setPullOrderType(mesPartPull.getPullOrderType());
                    pullingOrder.setPullGroupFid(mesPartPull.getId());
                    pullingOrder.setCustOrderNo(StringUtil.isEmpty(mesWorkOrder.getCustOrderNo()) ? "" : mesWorkOrder.getCustOrderNo());
                    pullingOrder.setWorkOrderNo(mesWorkOrder.getWorkOrderNo());
                    pullingOrder.setWorkOrderSeq(StringUtil.isEmpty(mesWorkOrder.getWorkOrderSeq()) ? "" : mesWorkOrder.getWorkOrderSeq());
                    pullingOrder.setIsPrint(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
                    pullingOrder.setPrintStatus(MesExtEnumUtil.PRINT_STATUS.UNPRINT.getValue());
                    pullingOrder.setPullOrderStatus(MesExtEnumUtil.PULL_ORDER_STATUS.PULL_ORDER_STATUS_10.getValue());
                    pullingOrder.setWorkCenterCode(mesPartPull.getWorkCenterCode());
                    pullingOrder.setPullCode(mesPartPull.getPullCode());
                    pullingOrder.setPartNo(mesWorkOrder.getPartNo());
                    pullingOrder.setPartName(mesWorkOrder.getPartName());
                    pullingOrder.setCustPartNo(mesWorkOrder.getCustPartNo());
                    pullingOrder.setCarModelCode(mesWorkOrder.getCarModelCode());
                    pullingOrder.setProductSeq(StringUtil.isEmpty(mesWorkOrder.getProductSeq()) ? 0L : mesWorkOrder.getProductSeq());
                    pullingOrder.setOrderFlag(StringUtil.isEmpty(mesWorkOrder.getOrderFlag())?"":mesWorkOrder.getOrderFlag().substring(0,1));

                    ConvertBean.serviceModelInitialize(pullingOrder, userName);
                    mesPullingOrderInfoList.add(pullingOrder);

                    for (MesPartPullDetail partInfo : mesPartPullDetails) {
                        MesPullingOrderPartInfo pullingOrderPartInfo = new MesPullingOrderPartInfo();
                        pullingOrderPartInfo.setOrganizeCode(organizeCode);
                        pullingOrderPartInfo.setPullCode(partInfo.getPullCode());
                        pullingOrderPartInfo.setPullingOrderNo(pullingOrder.getPullingOrderNo());
                        pullingOrderPartInfo.setWorkOrderNo(pullingOrder.getWorkOrderNo());
                        pullingOrderPartInfo.setWorkOrderSeq(pullingOrder.getWorkOrderSeq());
                        pullingOrderPartInfo.setCustOrderNo(pullingOrder.getCustOrderNo());
                        pullingOrderPartInfo.setPartNo(partInfo.getPartNo());
                        pullingOrderPartInfo.setPartName(partInfo.getPartName());
                        MesWorkOrderPart mesWorkOrderPart = workOrderPartListMap.get(partInfo.getPartNo()).get(0);
                        pullingOrderPartInfo.setPullQty(mesWorkOrderPart.getItemQty());
                        pullingOrderPartInfo.setCustOrderNo(pullingOrder.getCustOrderNo());
                        pullingOrderPartInfo.setWorkOrderSeq(pullingOrder.getWorkOrderSeq());
                        pullingOrderPartInfo.setWaterSeq(pullingOrder.getProductSeq());
                        pullingOrderPartInfo.setWorkCellCode(mesWorkOrderPart.getWorkCellCode());
                        pullingOrderPartInfo.setEquipment(partInfo.getEquipment());
                        pullingOrderPartInfo.setLocation(partInfo.getPullAddr());//todo
                        pullingOrderPartInfo.setStatus(MesExtEnumUtil.PULL_ORDER_PART_STATUS.UN_SENTED.getValue());///明细已送料为1，未送料为0
                        pullingOrderPartInfo.setCustPartNo(!Objects.isNull(customerPartMap) && customerPartMap.containsKey(partInfo.getPartNo()) ? customerPartMap.get(partInfo.getPartNo()).iterator().next().getCustPartNo() : "");
                        if (partInfo.getScanValidationType() == 10) {
                            pullingOrderPartInfo.setColor(MesExtEnumUtil.PART_PULL_DETAIL_COLOR.WHITE.getCode());
                        } else {
                            pullingOrderPartInfo.setColor(MesExtEnumUtil.PART_PULL_DETAIL_COLOR.RED.getCode());
                        }
                        pullingOrderPartInfo.setCarSeries(partInfo.getCarSeries());
                        pullingOrderPartInfo.setOrderFlag(pullingOrder.getOrderFlag());

                        ConvertBean.serviceModelInitialize(pullingOrderPartInfo, userName);
                        mesPullingOrderPartInfoList.add(pullingOrderPartInfo);
                    }
                    //7.拉动组+加工单号+生成拉动单状态的对应关系中间表，每次新增都要记录
                    MesPullingOrderExtends mesPullingOrderExtends= new MesPullingOrderExtends();
                    mesPullingOrderExtends.setOrganizeCode(organizeCode);
                    mesPullingOrderExtends.setPullCode(pullingOrder.getPullCode());
                    mesPullingOrderExtends.setPullingOrderNo(pullingOrder.getPullingOrderNo());
                    mesPullingOrderExtends.setWorkOrderNo(pullingOrder.getWorkOrderNo());
                    mesPullingOrderExtends.setPullOrderStatus(pullingOrder.getPullOrderStatus());
                    ConvertBean.serviceModelInitialize(mesPullingOrderExtends, userName);
                    mesPullingOrderExtendsList.add(mesPullingOrderExtends);
                }

            }
            //6.产生拉动单后更新工单拉动状态为已拉动
            workOrderNoList.add(mesWorkOrder.getWorkOrderNo());

        }
        //数据持久化
        if (!CollectionUtils.isEmpty(mesPullingOrderInfoList)) {
            mesPullingOrderInfoRepository.saveAll(mesPullingOrderInfoList);
        }
        if (!CollectionUtils.isEmpty(mesPullingOrderPartInfoList)) {
            mesPullingOrderPartInfoRepository.saveAll(mesPullingOrderPartInfoList);
        }
        if (!CollectionUtils.isEmpty(mesPullingOrderExtendsList)) {
            mesPullingOrderExtendsRepository.saveAll(mesPullingOrderExtendsList);
        }
        //更新拉动状态
        if (!CollectionUtils.isEmpty(workOrderNoList)) {
            updateMesWorkOrder(organizeCode, userName, workOrderNoList);
        }
    }

    private List<MesWorkOrder> getMesWorkOrderList(String organizeCode, Integer pageSize, List<String> workCenterList) {
        DdlPackBean packBeanWorkOrder = DdlPackBean.getDdlPackBean(organizeCode);
        if (workCenterList.size() == MesExtConstWords.ONE) {
            DdlPreparedPack.getStringEqualPack(workCenterList.get(MesExtConstWords.ZERO), MesExtConstWords.WORK_CENTER_CODE, packBeanWorkOrder);
        } else {
            DdlPreparedPack.getInPackList(workCenterList, MesExtConstWords.WORK_CENTER_CODE, packBeanWorkOrder);
        }
        DdlPreparedPack.getInPackList(Arrays.asList(MesExtEnumUtil.ORDER_STATUS.RELEASE.getValue(),MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue()), "workOrderStatus", packBeanWorkOrder);
        DdlPreparedPack.getNumberSmallerPack(MesExtEnumUtil.WORK_ORDER_PULLING_STATUS.SUBMIT.getValue(), "pullingStatus", packBeanWorkOrder);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.ASC.getValue()}, new String[]{"createDatetime"}, packBeanWorkOrder);
        return mesWorkOrderRepository.findByHqlTopWhere(packBeanWorkOrder, StringUtil.isEmpty(pageSize) ? MesExtConstWords.TEN : pageSize);
    }

    private void updateMesWorkOrder(String organizeCode, String userName, List<String> workOrderNoList) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        if (workOrderNoList.size() == MesExtConstWords.ONE) {
            DdlPreparedPack.getStringEqualPack(workOrderNoList.get(MesExtConstWords.ZERO), MesExtConstWords.WORK_ORDER_NO, packBean);
        } else {
            DdlPreparedPack.getInPackList(workOrderNoList, MesExtConstWords.WORK_ORDER_NO, packBean);
        }
        mesWorkOrderRepository.updateByPropertiesNoSync(new String[]{"modifyUser", "modifyDatetime", "pullingStatus"},
                new Object[]{userName, TimeTool.getNowTime(true), MesExtEnumUtil.WORK_ORDER_PULLING_STATUS.SUBMIT.getValue()}, packBean);
    }

    @Override
    public Map doSPSForEquipment(String assemblyLine, String pullingGroupCode, String organizeCode) {
        Map resultMap = new HashMap();
        //查询拉动单状态为已扫描=20的 拉动单
        DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(organizeCode);
//        DdlPreparedPack.getStringEqualPack(pullingGroupCode, "pullCode", seriesPackBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.PART_PULL_ORDER_TYPE.SPS_PULL.getValue(), "pullOrderType", seriesPackBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.PULL_ORDER_STATUS.PULL_ORDER_STATUS_20.getValue(), "pullOrderStatus", seriesPackBean);
//        DdlPreparedPack.getStringEqualPack(assemblyLine, "workCenterCode", seriesPackBean);
        seriesPackBean.setWhereAppend(seriesPackBean.getWhereAppend() + "  order by modifyDatetime desc");

        MesPullingOrderInfo mesPullingOrder = baseRDao.getByProperty(seriesPackBean);
        if (StringUtil.isEmpty(mesPullingOrder)) {
            resultMap.put("success", false);
            String message = "SPS拉动数据已扫描数据为空，请检查拉动单数据";
            resultMap.put("message", message);
            return resultMap;
        }
        //更新主表状态 源系统为25配料完成  现在-【已发送设备-30】
        mesPullingOrder.setPullOrderStatus(MesExtEnumUtil.PULL_ORDER_STATUS.PULL_ORDER_STATUS_30.getValue());
        ConvertBean.serviceModelUpdate(mesPullingOrder, "WS.SPS");
        baseRDao.update(mesPullingOrder);
        String message = "更新SPS状态成功，已发送SPS信息给设备! id:" + mesPullingOrder.getId() + "Code:" + mesPullingOrder.getPullingOrderNo() + "产线" + assemblyLine + "拉动组" + pullingGroupCode + "";
        //查询拉动组明细
        DdlPackBean partPackBean = DdlPackBean.getDdlPackBean(mesPullingOrder.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesPullingOrder.getPullingOrderNo(), "pullingOrderNo", partPackBean);
        List<MesPullingOrderPartInfo> pullingOrderPartInfos = mesPullingOrderPartInfoRepository.findByHqlWhere(partPackBean);
        if (!CollectionUtils.isEmpty(pullingOrderPartInfos)) {
            mesPullingOrder.setPartList(pullingOrderPartInfos);
        }
        resultMap.put("orderInfo", JSONObject.toJSONString(mesPullingOrder));
        resultMap.put("success", true);
        resultMap.put("message", message);


        return resultMap;
    }
    @Override
    public List<MesPullingOrderInfo> doMesPullingOrderInfoPrint(MesPullingOrderInfo bean, String userName) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(bean.getPullingOrderNo(), "pullingOrderNo", packBean);
//        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.PRINT_STATUS.UNPRINT.getValue(), "printStatus", packBean);
        List<MesPullingOrderInfo> pullingOrderInfos = mesPullingOrderInfoRepository.findByHqlWhere(packBean);
        if (!CollectionUtils.isEmpty(pullingOrderInfos)) {
            //把查出来的拉动单打印并修改打印状态为已打印
            List<MesPrintedSnLog> snLogList = new ArrayList<>();
            for (MesPullingOrderInfo pullingOrderInfo : pullingOrderInfos) {
                //查询拉动组明细
                DdlPackBean partPackBean = DdlPackBean.getDdlPackBean(pullingOrderInfo.getOrganizeCode());
                DdlPreparedPack.getStringEqualPack(pullingOrderInfo.getPullingOrderNo(), "pullingOrderNo", partPackBean);
                List<MesPullingOrderPartInfo> pullingOrderPartInfos = mesPullingOrderPartInfoRepository.findByHqlWhere(partPackBean);
                if (!org.springframework.util.CollectionUtils.isEmpty(pullingOrderPartInfos)) {
                    pullingOrderInfo.setPartCount(pullingOrderPartInfos.size());
                    pullingOrderInfo.setPullingOrderPartInfos(pullingOrderPartInfos);
                }

                pullingOrderInfo.setPrintTime(TimeTool.getNowTime(true));
                pullingOrderInfo.setPrintStatus(MesExtEnumUtil.PRINT_STATUS.PRINTED.getValue());
                pullingOrderInfo.setIsPrint(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
                ConvertBean.serviceModelUpdate(pullingOrderInfo, userName);

                //10-12 打印补打拉动单新增log表
                MesPrintedSnLog snLog = new MesPrintedSnLog();
                snLog.setBarcode(pullingOrderInfo.getPullingOrderNo());
                snLog.setCustPartNo(pullingOrderInfo.getCustPartNo());
                snLog.setWorkOrderNo(pullingOrderInfo.getWorkOrderNo());
                snLog.setPartNo(pullingOrderInfo.getPartNo());
                snLog.setPartName(pullingOrderInfo.getPartName());
                ConvertBean.serviceModelInitialize(snLog, userName);
                snLog.setOrganizeCode(bean.getOrganizeCode());
                snLogList.add(snLog);

            }
            mesPullingOrderInfoRepository.saveAll(pullingOrderInfos);
            //保存打印条码记录
            snLogRao.saveAll(snLogList);
        }
        return pullingOrderInfos;
    }

}
