package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesPullingOrderInfoService;
import cn.estsh.i3plus.ext.mes.api.base.IMesPullingOrderPartInfoService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesPullingOrderInfo;
import cn.estsh.i3plus.pojo.mes.bean.MesPullingOrderPartInfo;
import cn.estsh.i3plus.pojo.mes.repository.MesPullingOrderPartInfoRepository;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Slf4j
public class MesPullingOrderPartInfoService extends BaseMesService<MesPullingOrderPartInfo> implements IMesPullingOrderPartInfoService {


    protected void setPackQueryBean(MesPullingOrderPartInfo bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getPullingOrderNo(), "pullingOrderNo", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getPullCode(), "pullCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getWorkCellCode(), "workCellCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getWorkOrderNo(), "workOrderNo", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartNo(), "partNo", packBean);
    }
    @Override
    public void validateImport(List<MesPullingOrderPartInfo> beanList) {

        for (MesPullingOrderPartInfo item : beanList) {
            // 数据校验
            ValidatorBean.checkNotNull(item.getPullCode(), "拉动组代码不能为空");
            ValidatorBean.checkNotNull(item.getPartNo(), "物料代码不能为空");


            DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(item.getPullCode(), "pullCode", seriesPackBean);
            DdlPreparedPack.getStringEqualPack(item.getWorkOrderNo(), "workOrderNo", seriesPackBean);
            DdlPreparedPack.getStringEqualPack(item.getPartNo(), "partNo", seriesPackBean);
            boolean flg = baseRDao.isExitByHql(seriesPackBean);
            if (flg) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("工单【%s】拉动组代码【%s】物料号【%s】已经存在，请检查数据",
                                item.getWorkOrderNo(), item.getPullCode(), item.getPartNo())
                        .build();
            }
        }
    }
}
