package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesQmsScrapService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesQmsScrap;
import cn.estsh.i3plus.pojo.mes.bean.nc.MesPartInspection;
import cn.estsh.i3plus.pojo.mes.bean.nc.MesPartInspectionDetail;
import cn.estsh.i3plus.pojo.mes.repository.MesPartInspectionDetailRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description :MES_QMS报废品汇总
 * @Reference :
 * @Author : gsz
 * @CreateDate 2024/7/3 10:28
 * @Modify:
 **/
@Service
@Slf4j
public class MesQmsScrapService extends BaseMesService<MesQmsScrap> implements IMesQmsScrapService {
    @Autowired
    private MesPartInspectionDetailRepository partInspectionDetailRDao;

    @Override
    protected void setPackQueryBean(MesQmsScrap bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringLikerPack(bean.getPartNo(), "partNo", packBean);
    }

    @Override
    public void doQmsScrapByPartInspection(List<MesPartInspection> mesPartInspectionList, String organizeCode) {

        List<Long> idList = mesPartInspectionList.stream().map(MesPartInspection::getId).collect(Collectors.toList());
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(idList, "pid", packBean);
        List<MesPartInspectionDetail> partInspectionDetails = partInspectionDetailRDao.findByHqlWhere(packBean);
        Map<Long, List<MesPartInspectionDetail>> idDetailListMap = partInspectionDetails.stream().collect(Collectors.groupingBy(MesPartInspectionDetail::getPid));
        //43326 电子化检验，发送给QMS的数据，按照零件号+责任人类型分组发送
        Map<String, List<MesPartInspection>> workOrderPartMap = mesPartInspectionList.stream().collect(Collectors.groupingBy(p -> p.getPartNo() +"="+p.getResponsibleParty()));
        //根据物料对可疑品分组
        List<MesQmsScrap> qmsScrapList = new ArrayList<>();
        log.info("一共需要汇总{}条报废数据2", workOrderPartMap.size());

        for (Map.Entry<String, List<MesPartInspection>> stringListEntry : workOrderPartMap.entrySet()) {
            String keyPartNo = stringListEntry.getKey().split("=")[0];
            List<MesPartInspection> mesPartInspections = stringListEntry.getValue();
            MesPartInspection next = mesPartInspections.iterator().next();
            List<MesPartInspectionDetail> mesPartInspectionDetails = new ArrayList<>();
            for (MesPartInspection mesPartInspection : mesPartInspections) {
                mesPartInspectionDetails.addAll(idDetailListMap.get(mesPartInspection.getId()));
            }

            //新增一条数据到报废表中
            MesQmsScrap mesQmsScrap = new MesQmsScrap();
            mesQmsScrap.setOrganizeCode(organizeCode);
            mesQmsScrap.setAsnNo("");
            mesQmsScrap.setSource("MES");
            double sum = mesPartInspections.stream().mapToDouble(MesPartInspection::getRejectQty).sum();
            mesQmsScrap.setRejectQty(sum);
            mesQmsScrap.setPartNo(keyPartNo);
            mesQmsScrap.setPartName(next.getPartName());
            String defectTypeName = mesPartInspectionDetails.stream().map(MesPartInspectionDetail::getDefectTypeName).collect(Collectors.joining(";"));
            mesQmsScrap.setDefectTypeName(defectTypeName);
            mesQmsScrap.setReponsibilityId(next.getResponsibleParty());
            mesQmsScrap.setNcJudger(next.getModifyUser());
            ConvertBean.serviceModelInitialize(mesQmsScrap, "MesScrapJob");
            qmsScrapList.add(mesQmsScrap);
        }
        log.info("一共需要汇总{}条报废数据3", qmsScrapList.size());
        baseRDao.saveAll(qmsScrapList);

    }
}
