package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesQmsSuspiciousService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.MathOperation;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesQmsSuspicious;
import cn.estsh.i3plus.pojo.mes.bean.nc.MesPartInspection;
import cn.estsh.i3plus.pojo.mes.bean.nc.MesPartInspectionDetail;
import cn.estsh.i3plus.pojo.mes.bean.rework.MesReworkTask;
import cn.estsh.i3plus.pojo.mes.repository.MesPartInspectionDetailRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesPartInspectionRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesReworkTaskRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description :MES可疑品汇总
 * @Reference :
 * @Author : gsz
 * @CreateDate 2024/7/3 10:28
 * @Modify:
 **/
@Service
@Slf4j
public class MesQmsSuspiciousService extends BaseMesService<MesQmsSuspicious> implements IMesQmsSuspiciousService {

    @Autowired
    private MesPartInspectionDetailRepository partInspectionDetailRDao;

    @Autowired
    private MesReworkTaskRepository mesReworkTaskRepository;

    @Autowired
    private MesPartInspectionRepository partInspectionRepository;

    @Override
    protected void setPackQueryBean(MesQmsSuspicious bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringLikerPack(bean.getPartNo(), "partNo", packBean);
    }

    @Override
    public void doQmsSuspiciousByPartInspection(List<MesPartInspection> mesPartInspectionList, String organizeCode) {


        Map<String, List<MesPartInspection>> workOrderPartMap = mesPartInspectionList.stream().collect(Collectors.groupingBy(MesPartInspection::getPartNo));
        //根据物料对可疑品分组
        for (Map.Entry<String, List<MesPartInspection>> stringListEntry : workOrderPartMap.entrySet()) {
            String key = stringListEntry.getKey();
            List<MesPartInspection> mesPartInspectionsList = stringListEntry.getValue();
            //修改时间的集合
            List<MesPartInspection> mesPartInspectionsListTime = new ArrayList<>();
            for (MesPartInspection partInspection : mesPartInspectionsList) {
                MesPartInspection partInspectionNew = new MesPartInspection();
                BeanUtils.copyProperties(partInspection, partInspectionNew);
                String qmsTime = partInspection.getCreateDatetime().substring(0, 13) + ":00:00";
                if (partInspection.getSourceType() == MesExtEnumUtil.PART_INSPECTION_SOURCE_TYPE.LOT.getValue()) {
                    partInspectionNew.setInspectionDate(qmsTime);
                    mesPartInspectionsListTime.add(partInspectionNew);
                } else {
                    //检验日期年月日可以根据创建日期截取，批次的不用取最早出现的
                    //去创建日期换算整点2024-07-25 10:00:00
                    DdlPackBean packBeanNew = DdlPackBean.getDdlPackBean(organizeCode);
                    DdlPreparedPack.getStringEqualPack(partInspection.getSn(), "sn", packBeanNew);
                    packBeanNew.setOrderByStr(" order by createDatetime ");
                    MesPartInspection partInspectionDao = partInspectionRepository.getByProperty(packBeanNew);
                    //如果是最新的
                    if (!StringUtil.isEmpty(partInspectionDao) && partInspectionDao.getCreateDatetime().substring(0, 13).equals(partInspection.getCreateDatetime().substring(0, 13))) {
                        qmsTime = partInspectionDao.getCreateDatetime().substring(0, 13) + ":00:00";
                        partInspectionNew.setInspectionDate(qmsTime);
                        mesPartInspectionsListTime.add(partInspectionNew);
                    } else {
                        qmsTime = partInspectionDao.getCreateDatetime().substring(0, 13) + ":00:00";
                        partInspectionNew.setInspectionDate(qmsTime);
                        mesPartInspectionsListTime.add(partInspectionNew);
                        //如果第二次出现 找条码最早时间区间内 的非该条码 的 物料 集合
                        //获取最早条码创建时间 后一小时的时间
                        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
                        LocalDateTime nextHourDateTime = LocalDateTime.parse(partInspectionDao.getCreateDatetime(), formatter).plusHours(1);
                        String afterQmsTime = nextHourDateTime.format(formatter);
                        DdlPackBean packBeanPart = DdlPackBean.getDdlPackBean(organizeCode);
                        DdlPreparedPack.getStringEqualPack(partInspectionDao.getPartNo(), "partNo", packBeanPart);
                        DdlPreparedPack.getNotInPack(partInspectionDao.getSn(), "sn", packBeanPart);
                        DdlPreparedPack.timeBuilder(partInspectionDao.getCreateDatetime(), afterQmsTime, "createDatetime", packBeanPart, true);
                        List<MesPartInspection> mesPartInspections = partInspectionRepository.findByHqlWhere(packBeanPart);
                        if (!CollectionUtils.isEmpty(mesPartInspections)) {
                            //排除其他非最早出现的条码
                            for (MesPartInspection mesPartInspection : mesPartInspections) {
                                MesPartInspection partInspectionNew2 = new MesPartInspection();
                                BeanUtils.copyProperties(mesPartInspection, partInspectionNew2);
                                qmsTime = mesPartInspection.getCreateDatetime().substring(0, 13) + ":00:00";
                                partInspectionNew2.setInspectionDate(qmsTime);
                                if (mesPartInspection.getSourceType()==MesExtEnumUtil.PART_INSPECTION_SOURCE_TYPE.SINGLE.getValue()){
                                    //如果条码重复出现时，对应最早时间段的零件不是最早，则不统计在汇总数据中
                                    DdlPackBean packBeanSn = DdlPackBean.getDdlPackBean(organizeCode);
                                    DdlPreparedPack.getStringEqualPack(mesPartInspection.getSn(), "sn", packBeanSn);
                                    packBeanSn.setOrderByStr(" order by createDatetime ");
                                    MesPartInspection partInspectionSn = partInspectionRepository.getByProperty(packBeanSn);
                                    if (!StringUtil.isEmpty(partInspectionSn)&& partInspectionSn.getCreateDatetime().substring(0, 13).equals(mesPartInspection.getCreateDatetime().substring(0, 13))){
                                        mesPartInspectionsListTime.add(partInspectionNew2);
                                    }
                                }else {
                                    mesPartInspectionsListTime.add(partInspectionNew2);
                                }
                            }

                        }

                    }
                }
            }
            List<Long> idList = mesPartInspectionsListTime.stream().map(MesPartInspection::getId).collect(Collectors.toList());
            DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getInPackList(idList, "pid", packBean);
            List<MesPartInspectionDetail> partInspectionDetails = partInspectionDetailRDao.findByHqlWhere(packBean);
            Map<Long, List<MesPartInspectionDetail>> idDetailListMap = partInspectionDetails.stream().collect(Collectors.groupingBy(MesPartInspectionDetail::getPid));

            //根据时间分组发送
            // 批次的：
            //A  2024-08-21 08:00 10
            //A  2024-08-21 08:00 10
            //B  2024-08-20 08:00 10  检验日期 2024-08-21 08:00
            //B  2024-08-21 08:00 10  检验日期 2024-08-21 08:00
            //单件的：
            //A  2024-08-21 08:00 1  0 0 1
            //
            //qms汇总表=====
            //A  2024-08-21 08:00 21 0 0 1
            //B  2024-08-21 08:00 20
            Map<String, List<MesPartInspection>> mesPartInspectionsMap = mesPartInspectionsListTime.stream().collect(Collectors.groupingBy(MesPartInspection::getInspectionDate));
            for (Map.Entry<String, List<MesPartInspection>> partInspection : mesPartInspectionsMap.entrySet()) {
                List<MesPartInspection> mesPartInspections = partInspection.getValue();
                MesPartInspection next = mesPartInspections.iterator().next();

                //新增一条数据到报废表中
                MesQmsSuspicious mesQmsSuspicious = new MesQmsSuspicious();
                mesQmsSuspicious.setOrganizeCode(organizeCode);
                mesQmsSuspicious.setInspectionDate(next.getInspectionDate());

                //检验数量是全部数量汇总
                double sumQty = mesPartInspections.stream().mapToDouble(MesPartInspection::getQty).sum();
                mesQmsSuspicious.setInspectionQty(sumQty);
                //一次合格数 nc_status=10 inspection_status=10
                double qualifiedQty = mesPartInspections.stream().filter(m -> m.getNcStatus() == MesExtEnumUtil.PART_INSPECTION_NC_STATUS.CREATE.getValue()
                        && m.getInspectionStatus() == MesExtEnumUtil.PART_INSPECTION_STATUS.QUALIFIED.getValue()).mapToDouble(MesPartInspection::getQty).sum();
                mesQmsSuspicious.setQualifiedQty(qualifiedQty);
                //不合格数 nc_status=10 inspection_status=20
                double unqualifiedQty = mesPartInspections.stream().filter(m -> m.getNcStatus() == MesExtEnumUtil.PART_INSPECTION_NC_STATUS.CREATE.getValue()
                        && m.getInspectionStatus() == MesExtEnumUtil.PART_INSPECTION_STATUS.FAIL.getValue()).mapToDouble(MesPartInspection::getQty).sum();
                mesQmsSuspicious.setUnqualifiedQty(unqualifiedQty);
                //误判数量  nc_status=30 inspection_status=20
                double misjudgmentQty = mesPartInspections.stream().filter(m -> m.getNcStatus() == MesExtEnumUtil.PART_INSPECTION_NC_STATUS.RELEASE.getValue()).mapToDouble(MesPartInspection::getQty).sum();
                mesQmsSuspicious.setMisjudgmentQty(misjudgmentQty);
                //一次报废数  nc_status=40 inspection_status=20
                double scrappedQty = mesPartInspections.stream().filter(m -> m.getNcStatus() == MesExtEnumUtil.PART_INSPECTION_NC_STATUS.SCRAP.getValue() && m.getInspectionStatus() == MesExtEnumUtil.PART_INSPECTION_STATUS.FAIL.getValue()).mapToDouble(MesPartInspection::getQty).sum();
                mesQmsSuspicious.setScrappedQty(scrappedQty);

                //返工合格数 nc_status=50 返工单 status=20
                Double reworkQualified = new Double(0);
                //返工pending（创建返工单，未处理）
                Double reworkPending = new Double(0);
                //返工数
                Double reworkQty = new Double(0);
                List<MesPartInspectionDetail> mesPartInspectionDetails = new ArrayList<>();
                for (MesPartInspection mesPartInspection : mesPartInspections) {
                    if (!StringUtil.isEmpty(idDetailListMap.get(mesPartInspection.getId()))) {
                        mesPartInspectionDetails.addAll(idDetailListMap.get(mesPartInspection.getId()));
                    }
                    //计算返工数量
                    if (mesPartInspection.getNcStatus() == MesExtEnumUtil.PART_INSPECTION_NC_STATUS.REWORK.getValue()) {
                        packBean = DdlPackBean.getDdlPackBean(organizeCode);
                        DdlPreparedPack.getNumEqualPack(mesPartInspection.getReworkTaskId(),"id",packBean);
                        MesReworkTask reworkTask = mesReworkTaskRepository.getByProperty(packBean);
                        if (!StringUtil.isEmpty(reworkTask)) {
                            //返工是否完成
                            if (reworkTask.getStatus() == MesExtEnumUtil.REWORK_TASK_STATUS.FINISH.getValue()) {
                                reworkQualified = MathOperation.add(reworkQualified, mesPartInspection.getQty());
                            } else {
                                reworkPending = MathOperation.add(reworkPending, mesPartInspection.getQty());
                            }
                        } else {
                            reworkPending = MathOperation.add(reworkPending, mesPartInspection.getQty());
                        }
                        reworkQty = MathOperation.add(reworkQty, mesPartInspection.getQty());
                    }
                }
                mesQmsSuspicious.setReworkQty(reworkQty);
                mesQmsSuspicious.setReworkQualified(reworkQualified);
                mesQmsSuspicious.setReworkPending(reworkPending);
                mesQmsSuspicious.setNcPending(new Double(0));
                mesQmsSuspicious.setReworkScrapped(new Double(0));
                mesQmsSuspicious.setPartNo(key);
                mesQmsSuspicious.setPartName(next.getPartName());
                //缺陷分类，缺陷数量JOSN格式:【批次＆单件数量汇总时需要根据检验时间段合并】
                //[{"Defect_Name":"缺陷01","Defect_Qty":1},{"Defect_Name":"缺陷02","Defect_Qty":2}]
                Map<String, List<MesPartInspectionDetail>> listDefectMap = mesPartInspectionDetails.stream().collect(Collectors.groupingBy(MesPartInspectionDetail::getDefectTypeName));
                List<Map<String, Object>> defectsList = new ArrayList<>();
                for (Map.Entry<String, List<MesPartInspectionDetail>> listEntry : listDefectMap.entrySet()) {
                    Map<String, Object> defectMap = new HashMap<>();
                    defectMap.put("Defect_Name", listEntry.getKey());
                    defectMap.put("Defect_Qty", listEntry.getValue().size());
                    defectsList.add(defectMap);
                }
                //缺陷分布json
                mesQmsSuspicious.setDefects(JSON.toJSONString(defectsList));
                ConvertBean.serviceModelInitialize(mesQmsSuspicious, "MesSuspiciousJob");

                baseRDao.insert(mesQmsSuspicious);
            }

        }
    }
}
