package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesRawPackagePartService;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesPartSap;
import cn.estsh.i3plus.pojo.mes.bean.MesRawPackagePart;
import cn.estsh.i3plus.pojo.mes.repository.MesPartSapRepository;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.UUID;
import java.util.stream.Collectors;

/**
 * @Description : 原材料容器零件关系表
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/7/26 10:56
 * @Modify:
 **/
@Service
@Slf4j
public class MesRawPackagePartServiceImpl extends BaseMesService<MesRawPackagePart> implements IMesRawPackagePartService {

    @Autowired
    private MesPartSapRepository mesPartSapRepository;

    protected void setPackQueryBean(MesRawPackagePart bean, DdlPackBean packBean) {
        DdlPreparedPack.getNumEqualPack(bean.getPackageId(), "packageId", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getPackageDetailId(), "packageDetailId", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPackageSn(), "packageSn", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartNo(), "partNo", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getScanType(), "scanType", packBean);
    }

    protected void setBeanModel(List<MesRawPackagePart> beanList) {
        if(CollectionUtils.isEmpty(beanList))return;
        Map<String, MesPartSap> partSapMap = getMesPartSapMap(beanList.iterator().next().getOrganizeCode(), beanList.stream().map(MesRawPackagePart::getPartNo).distinct().collect(Collectors.toList()));
        for (MesRawPackagePart mesRawPackagePart : beanList) {
            if (!Objects.isNull(partSapMap) && partSapMap.containsKey(mesRawPackagePart.getPartNo())) {
                mesRawPackagePart.setPartName(partSapMap.get(mesRawPackagePart.getPartNo()).getPartName());
            }
        }
    }

    protected void onInsertBean(MesRawPackagePart item) {
        //数据校验
        checkData(item);
        //生成fid
        item.setFid(UUID.randomUUID().toString());
    }

    protected void onUpdateBean(MesRawPackagePart item) {
        //数据校验
        checkData(item);
    }

    private void checkData(MesRawPackagePart item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getPackageId(), "原材料容器id不能为空");
        ValidatorBean.checkNotNull(item.getPackageDetailId(), "原材料容器明细id不能为空");
        ValidatorBean.checkNotNull(item.getPackageSn(), "容器编号不能为空");
        ValidatorBean.checkNotNull(item.getPartNo(), "物料名称不能为空");
        ValidatorBean.checkNotNull(item.getScanType(), "扫描类型不能为空");
    }

    private Map<String, MesPartSap> getMesPartSapMap(String organizeCode, List<String> partNoList) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(partNoList, "partNo", packBean);
        List<MesPartSap> partSapList = mesPartSapRepository.findByHqlWhere(packBean);
        Map<String, MesPartSap> partSapMap = null;
        if (!CollectionUtils.isEmpty(partSapList)) {
            partSapMap = partSapList.stream().collect(Collectors.toMap(MesPartSap::getPartNo, t -> t));
        }
        return partSapMap;
    }
}
