package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesRecyclablePackageTypeService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.recyclablepackage.MesRecyclablePackageType;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

/**
 * @Description :
 * @Reference :
 * @Author : PZJ
 * @CreateDate : 2024/9/13 17:38
 * @Modify:
 **/
@Slf4j
@Service
public class MesRecyclablePackageTypeServiceImpl extends BaseMesService<MesRecyclablePackageType> implements IMesRecyclablePackageTypeService {



    @Override
    protected void onInsertBean(MesRecyclablePackageType item) {
        ValidatorBean.checkNotNull(item.getPackageTypeCode(), "包类型不能为空");
        ValidatorBean.checkNotNull(item.getPackageTypeName(), "包描述不能为空");
        ValidatorBean.checkNotNull(item.getRuleType(),"规则类型不能为空");
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPackageTypeCode(), "packageTypeCode", ddlPackBean);
        if (baseRDao.isExitByHql(ddlPackBean)) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode( ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("回收包装类型已存在{}", item.getPackageTypeCode())
                    .build();
        }
    }

    @Override
    protected void setPackQueryBean(MesRecyclablePackageType bean, DdlPackBean packBean) {
        String packageTypeCode = bean.getPackageTypeCode();
        if (!StringUtils.isEmpty(packageTypeCode)) {
            DdlPreparedPack.getStringLikerPack(packageTypeCode, "packageTypeCode", packBean);
        }
    }

    @Override
    protected void onUpdateBean(MesRecyclablePackageType item) {
        ValidatorBean.checkNotNull(item.getId(), "id不能为空");
        ValidatorBean.checkNotNull(item.getPackageTypeCode(), "包类型不能为空");
        ValidatorBean.checkNotNull(item.getPackageTypeName(), "包描述不能为空");
        ValidatorBean.checkNotNull(item.getRuleType(),"规则类型不能为空");
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPackageTypeCode(), "packageTypeCode", ddlPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", ddlPackBean);
        if (baseRDao.isExitByHql(ddlPackBean)) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode( ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("回收包装类型已存在{}", item.getPackageTypeCode())
                    .build();
        }
    }

}
