package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesScriptPersistenceService;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesScriptPersistence;
import cn.estsh.i3plus.pojo.mes.dbinterface.MesInterfaceDataMapper;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

/**
 * @Description : 动态脚本
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/8/22 10:04
 * @Modify:
 **/
@Service
@Slf4j
public class MesScriptPersistenceServiceImpl extends BaseMesService<MesScriptPersistence> implements IMesScriptPersistenceService {

    @Override
    protected void onInsertBean(MesScriptPersistence item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getScriptNo(), "脚本编号不能为空");

        DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getScriptNo(), "scriptNo", seriesPackBean);
        if (baseRDao.isExitByHql(seriesPackBean)) {
            MesException.throwMesBusiException("脚本编号【%s】已存在", item.getScriptNo());
        }

    }

    @Override
    protected void onUpdateBean(MesScriptPersistence item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getScriptNo(), "脚本编号不能为空");

        DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getScriptNo(), "scriptNo", seriesPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", seriesPackBean);
        if (baseRDao.isExitByHql(seriesPackBean)) {
            MesException.throwMesBusiException("脚本编号【%s】已存在", item.getScriptNo());
        }
    }

    @Override
    protected void setPackQueryBean(MesScriptPersistence bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringLikerPack(bean.getScriptNo(), "scriptNo", packBean);
        DdlPreparedPack.getStringRightLikerPack(bean.getScriptName(), "scriptName", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getScriptType(), "scriptType", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getLanguageType(), "languageType", packBean);
    }

}
