package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.print.strategy;

import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.print.IPrintTemplateStrategyService;
import cn.estsh.i3plus.ext.mes.pojo.model.MesProduceSnPrintModel;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description : 装配目视单标签打印
 * @Reference :
 * @Author : Castle
 * @CreateDate : 2024/6/17 16:43
 * @Modify:
 **/
@Component
@Slf4j
public class AssemblyVisualListPrintStrategy implements IPrintTemplateStrategyService {

    @Override
    public MesProduceSnPrintModel execute(GenSerialNoModel genSerialNoModel, MesProduceSnPrintModel model, MesNumberRule numberRule) {

        Map<String, Object> modelMap = (Map<String, Object>) model.getSourceData();

        // 工单信息
        MesWorkOrder workOrder = (MesWorkOrder) modelMap.get("mesWorkOrder");
        // 零件生产组信息
        MesPartProdGroup partProdGroup = (MesPartProdGroup) modelMap.get("mesPartProdGroup");
        // 客户车型配置信息
        MesCustomerCarModel customerCarModel = modelMap.containsKey("mesCustomerCarModel") ? (MesCustomerCarModel) modelMap.get("mesCustomerCarModel") : null;
        // 工单装配件信息
        List<MesWorkOrderAssembly> workOrderAssemblyList = modelMap.containsKey("mesWorkOrderAssemblyList") ? (List<MesWorkOrderAssembly>) modelMap.get("mesWorkOrderAssemblyList") : new ArrayList<>();

        workOrderAssemblyList = CollectionUtils.isEmpty(workOrderAssemblyList) ? new ArrayList<>() :
                workOrderAssemblyList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getDisplaySeq()) && o.getDisplaySeq().compareTo(MesExtConstWords.ZERO) != 0)).collect(Collectors.toList());

        // 返回的结果集合
        List<Map<String, Object>> printDataMapList = new ArrayList<>();
        // 单个标签参数值
        Map<String, Object> resultMap = new HashMap<>();
        // 工单号
        resultMap.put(MesExtConstWords.WORK_ORDER_NO, workOrder.getWorkOrderNo());
        // 产线名称
        resultMap.put(MesExtConstWords.WORK_CENTER_NAME, workOrder.getWorkCenterName());
        // 车型名称
        resultMap.put(MesExtConstWords.CAR_MODEL_NAME, customerCarModel == null ? "" : customerCarModel.getCarModelName());
        // vinCode
        resultMap.put(MesExtConstWords.VIN_CODE, workOrder.getVinCode());
        // vinCode 后四位
        resultMap.put(MesExtConstWords.VIN_CODE_AFTER_FOUR, !StringUtils.isEmpty(workOrder.getVinCode()) && workOrder.getVinCode().length() > MesExtConstWords.FOUR ? workOrder.getVinCode().substring(workOrder.getVinCode().length() - MesExtConstWords.FOUR) : workOrder.getVinCode());
        // 总成零件号
        resultMap.put(MesExtConstWords.PART_NO, workOrder.getPartNo());
        // 工单标识
        resultMap.put(MesExtConstWords.ORDER_FLAG, workOrder.getOrderFlag().equals(MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.PTR.getValue()) ? MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue() : workOrder.getOrderFlag());
        // 客户订单号
        resultMap.put(MesExtConstWords.CUST_ORDER_NO, workOrder.getCustOrderNo());
        // 客户零件号
        resultMap.put(MesExtConstWords.CUST_PART_NO, workOrder.getPartNo());
        // 客户条码
        resultMap.put(MesExtConstWords.SN, workOrder.getCustSn());
        // 备注
        resultMap.put(MesExtConstWords.REMARK, workOrder.getDescription());
        // 重打印
        resultMap.put(MesExtConstWords.REPRINT, "重打印");
        // 打印时间
        resultMap.put(MesExtConstWords.PRINT_TIME, TimeTool.getNowTime("yyyy-MM-dd HH:mm:ss"));
        // EDI接口时间 取工单plan_start_time
        resultMap.put(MesExtConstWords.EDI_REC_TIME, workOrder.getPlanStartTime());

        //替换总成SN
        resultMap.put(MesExtConstWords.REPLACE_CSN,MesExtConstWords.CP+workOrder.getCustPartNo().substring(workOrder.getCustPartNo().length() -4));

        Map<Integer, List<MesWorkOrderAssembly>> collect = CollectionUtils.isEmpty(workOrderAssemblyList) ? null : workOrderAssemblyList.stream().filter(o -> null != o).collect(Collectors.groupingBy(MesWorkOrderAssembly::getDisplaySeq));
        // 为空时显示内容
        String isNullViewContent = StringUtils.isEmpty(partProdGroup.getIsNullViewContent()) ? "" : partProdGroup.getIsNullViewContent();
        for (int i = 1; i <= 9; i ++) {
            List<MesWorkOrderAssembly> orderAssemblyList = CollectionUtils.isEmpty(collect) ? null : collect.get(i);
            List<String> disPlayValues = CollectionUtils.isEmpty(orderAssemblyList) ? null :
                    (orderAssemblyList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getDisplayValue()))).map(MesWorkOrderAssembly::getDisplayValue).collect(Collectors.toList()))
                            .stream().filter(o -> !StringUtils.isEmpty(o)).distinct().collect(Collectors.toList());
            String displayValue = isNullViewContent;
            if(!CollectionUtils.isEmpty(disPlayValues)){
                displayValue = String.join("\r\n", disPlayValues);
            }
            resultMap.put(MesExtConstWords.ASSEMBLY_PARAM + i, displayValue);
        }

        printDataMapList.add(resultMap);

        model.setPrintContextList(packResultMapList(model, printDataMapList));

        return model;
    }

    private List<Map<String, Object>> packResultMapList(MesProduceSnPrintModel printModel, List<Map<String, Object>> printTemplateDateList) {
        List<Map<String, Object>> resultMapList = new ArrayList<>();
        Map<String, Object> resultMap = new HashMap<>();
        resultMap.put(MesExtConstWords.LABEL_TEMPLATE, printModel.getMesLabelTemplate());
        resultMap.put(MesExtConstWords.TEMPLATE_DATA, printTemplateDateList);
        resultMap.put(MesExtConstWords.TEMPLATE_CODE, printModel.getMesLabelTemplate().getTemplateCode());
        resultMap.put(MesExtConstWords.PRINTER, printModel.getPrinter());
        resultMapList.add(resultMap);
        return resultMapList;
    }

}
