package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.report;

import cn.estsh.i3plus.ext.mes.api.base.IMesConfigService;
import cn.estsh.i3plus.ext.mes.api.busi.report.IMesEquipmentLogDetailService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.BaseMesService;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentLogDetail;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.StringJoiner;

@Service
public class MesEquipmentLogDetailService extends BaseMesService<MesEquipmentLogDetail> implements IMesEquipmentLogDetailService {

    @Autowired
    private IMesConfigService configService;

    @Override
    protected void setPackQueryBean(MesEquipmentLogDetail bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(new StringJoiner(MesExtConstWords.E_UNDERLINE).add(bean.getOrganizeCode()).add(bean.getEquipId().toString()).toString(), MesExtConstWords.ORG_EQUIP_ID, packBean);
//        DdlPreparedPack.getNumEqualPack(bean.getEquipId(), MesExtConstWords.EQUIP_ID, packBean);
        DdlPreparedPack.getNumEqualPack(bean.getVariableType(), MesExtConstWords.VARIABLE_TYPE, packBean);
//        DdlPreparedPack.getStringEqualPack(bean.getEquipmentCode(), MesExtConstWords.EQUIPMENT_CODE, packBean);
        if (!StringUtils.isEmpty(bean.getEquipVariableName()) && bean.getEquipVariableName().split(",").length > 1) {
            DdlPreparedPack.getInPackList(Arrays.asList(bean.getEquipVariableName().split(",")), MesExtConstWords.EQUIP_VARIABLE_NAME, packBean);
        } else {
            DdlPreparedPack.getStringLikerPack(bean.getEquipVariableName(), MesExtConstWords.EQUIP_VARIABLE_NAME, packBean);
        }
        DdlPreparedPack.timeBuilder(bean.getCreateDateTimeStart(), bean.getCreateDateTimeEnd(), MesExtConstWords.CREATE_DATE_TIME, packBean, true);
        if (!StringUtils.isEmpty(bean.getEquipVariableIds())) {
//            Long[] ids = Arrays.stream(bean.getEquipVariableIds().split(",")).map(Long::parseLong).toArray(Long[]::new);
            List<String> stringList = Arrays.asList(bean.getEquipVariableIds().split(","));
            packBean.setWhereAppend(packBean.getWhereAppend() + " and equipVariableId  in (" + String.join(",", stringList) + ") ");
        }

        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{MesExtConstWords.CREATE_DATE_TIME}, packBean);
    }

    @Override
    public ListPager<MesEquipmentLogDetail> queryPager(MesEquipmentLogDetail bean, Pager pager) {
        if (StringUtils.isEmpty(bean.getEquipId())) MesException.throwBusiException("查询条件的参数缺失设备ID[equipId]");
        String orgEquipId = new StringJoiner(MesExtConstWords.E_UNDERLINE).add(bean.getOrganizeCode()).add(bean.getEquipId().toString()).toString();
        if (!configService.checkShardingTablesObjectCfg(bean.getOrganizeCode(), MesEquipmentLogDetail.class.getSimpleName(), orgEquipId))
            return new ListPager<>(new ArrayList<>(), pager);
        return super.queryPager(bean, pager);
    }

}
