package cn.estsh.i3plus.ext.mes.apiservice.utils;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.exception.ExceptionUtils;

import javax.persistence.Column;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;


/**
 * Map转Object 工具类
 */
public class BeanUtil {

    public static Object populateBean(Map<String, ? extends Object> map, Class<? extends Object> clazz) throws Exception {
        Object obj = clazz.newInstance();
        BeanUtils.populate(obj, map);
        return obj;
    }

    public static List<Object> popListToList(Object object, List<Map<String, Object>> list) throws Exception {

        List<Object> listRetun = new ArrayList<Object>();
        for (Map<String, Object> map : list) {
            listRetun.add(BeanUtil.populateBean(map, object.getClass()));
        }
        return listRetun;
    }

    /**
     * 获取对象父类和字类所有的数据库字段
     *
     * @param c 对象类
     * @return 属性名数组
     */
    public static String[] getAllColumnFields(Class c) {
        //父子类属性合并
        Field[] fields = ArrayUtils.addAll(c.getFields(), c.getDeclaredFields());
        //循环所有属性把名称存入数组
        List<String> fieldsNameList = new ArrayList<>();
        for (Field field : fields) {
            field.setAccessible(true);
            // 判断是否为数据库字段
            if (field.isAnnotationPresent(Column.class)) {
                fieldsNameList.add(field.getName());
            }
        }
        return fieldsNameList.toArray(new String[fieldsNameList.size()]);
    }

    public static String getProperty(Object bean, String name) {
        String proVal = null;
        try {
            proVal = BeanUtils.getProperty(bean, name);
        } catch (Exception e) {
            MesException.throwMesBusiException("反射获取对象【" + bean.getClass().getName() + "】属性【" + name +
                    "】值时异常：详情如下" + ExceptionUtils.getStackTrace(e));
        }
        return proVal;
    }

    public static void setProperty(Object bean, String name, Object values) {
        try {
            BeanUtils.setProperty(bean, name, values);
        } catch (Exception e) {
            MesException.throwMesBusiException("反射设置对象【" + bean.getClass().getName() + "】属性【" + name +
                    "】值时异常：详情如下" + ExceptionUtils.getStackTrace(e));
        }
    }

}
